@@ -1,15 +1,20 @@
 package Net::DNS::RR;
 #
-# $Id: RR.pm 631 2004-02-16 17:30:06Z daniel $
+# $Id: RR.pm 7464 2006-05-19 15:08:57Z andy $
 #
 use strict;
-use vars qw($VERSION $AUTOLOAD);
 
+BEGIN { 
+    eval { require bytes; }
+} 
+
+use vars qw($VERSION $AUTOLOAD     %rrsortfunct );
 use Carp;
 use Net::DNS;
 use Net::DNS::RR::Unknown;
 
-$VERSION = (qw$Revision: 1.1 $)[1];
+
+$VERSION = (qw$LastChangedRevision: 552 $)[1];
 
 =head1 NAME
 
@@ -73,17 +78,22 @@ BEGIN {
 		TXT
 		X25
 		OPT
+		SSHFP
+		SPF
 	);
 
 	#  Only load DNSSEC if available
-	# 
-
-	eval { require Net::DNS::RR::SIG; };
 
+	eval { 	    
+	    local $SIG{'__DIE__'} = 'DEFAULT';
+	    require Net::DNS::RR::SIG; 
+	};
 	unless ($@) {
 		$RR{'SIG'} = 1;
-	
-		eval { require Net::DNS::RR::NXT; };
+		eval { 	    
+		    local $SIG{'__DIE__'} = 'DEFAULT';
+		    require Net::DNS::RR::NXT; 
+		};
 		
 		unless ($@) {
 		    $RR{'NXT'}	= 1;
@@ -91,7 +101,10 @@ BEGIN {
 		    die $@;
 		}
 		
-		eval { require Net::DNS::RR::KEY; };
+		eval { 
+		    local $SIG{'__DIE__'} = 'DEFAULT';
+		    require Net::DNS::RR::KEY; 
+		};
 		
 		unless ($@) {
 		    $RR{'KEY'} = 1;
@@ -99,7 +112,10 @@ BEGIN {
 		    die $@;
 		}
 
-	 	eval { require Net::DNS::RR::DS; };
+	 	eval { 
+		    local $SIG{'__DIE__'} = 'DEFAULT';
+		    require Net::DNS::RR::DS; 
+		};
 
 	 	unless ($@) {
 		    $RR{'DS'} = 1;
@@ -108,18 +124,28 @@ BEGIN {
 		    die $@;
 		}
 
-	 	eval { require Net::DNS::RR::RRSIG; };
+	 	eval { 
+		    local $SIG{'__DIE__'} = 'DEFAULT';
+		    require Net::DNS::RR::RRSIG; 
+		};
 
 	 	unless ($@) {
 		    $RR{'RRSIG'} = 1;
 		    # If RRSIG is available so should the other DNSSEC types
-		    eval { require Net::DNS::RR::NSEC; };
+		    eval {		    
+			local $SIG{'__DIE__'} = 'DEFAULT';
+			require Net::DNS::RR::NSEC; 
+		    };
 		    unless ($@) {
 		      $RR{'NSEC'} = 1;
 		    } else {
 		    die $@;
 		  }
-		    eval { require Net::DNS::RR::DNSKEY; };
+		    eval { 
+			local $SIG{'__DIE__'} = 'DEFAULT';
+			require Net::DNS::RR::DNSKEY; 
+		    };
+
 		    unless ($@) {
 		      $RR{'DNSKEY'} = 1;
 		    } else {
@@ -160,7 +186,7 @@ sub build_regex {
  $a     = Net::DNS::RR->new("foo.example.com. 86400 A 10.1.2.3");
  $mx    = Net::DNS::RR->new("example.com. 7200 MX 10 mailhost.example.com.");
  $cname = Net::DNS::RR->new("www.example.com 300 IN CNAME www1.example.com");
- $txt   = Net::DNS::RR->new("baz.example.com 3600 HS TXT 'text record'");
+ $txt   = Net::DNS::RR->new('baz.example.com 3600 HS TXT "text record"');
 
 Returns a C<Net::DNS::RR> object of the appropriate type and
 initialized from the string passed by the user.  The format of the
@@ -187,7 +213,7 @@ All names must be fully qualified.  The trailing dot (.) is optional.
  
  $rr = Net::DNS::RR->new(
 	 name => "foo.example.com",
-	 yype => "A",
+	 type => "A",
  );
 
 Returns an RR object of the appropriate type, or a C<Net::DNS::RR>
@@ -248,9 +274,11 @@ sub new_from_string {
 	my ($class, $rrstring, $update_type) = @_;
 
 	build_regex() unless $RR_REGEX;
-	
+
 	# strip out comments
-	$rrstring   =~ s/;.*//g;
+	# Test for non escaped ";" by means of the look-behind assertion
+	# (the backslash is escaped)
+	$rrstring   =~ s/(?<!\\);.*//og;
 	
 	($rrstring =~ m/$RR_REGEX/xso) || 
 		confess qq|qInternal Error: "$rrstring" did not match RR pat.\nPlease report this to the author!\n|;
@@ -263,14 +291,17 @@ sub new_from_string {
 	my $rrtype  = $4 || '';
 	my $rdata   = $5 || '';
 
-	$rdata =~ s/\s+$// if $rdata;
-	$name  =~ s/\.$//  if $name;
+	$rdata =~ s/\s+$//o if $rdata;
+	$name  =~ s/\.$//o  if $name;
+
+	
+
 
 
 	# RFC3597 tweaks
 	# This converts to known class and type if specified as TYPE###
-	$rrtype  = Net::DNS::typesbyval(Net::DNS::typesbyname($rrtype))      if $rrtype  =~ m/^TYPE\d+/;
-	$rrclass = Net::DNS::classesbyval(Net::DNS::classesbyname($rrclass)) if $rrclass =~ m/^CLASS\d+/;
+	$rrtype  = Net::DNS::typesbyval(Net::DNS::typesbyname($rrtype))      if $rrtype  =~ m/^TYPE\d+/o;
+	$rrclass = Net::DNS::classesbyval(Net::DNS::classesbyname($rrclass)) if $rrclass =~ m/^CLASS\d+/o;
 
 
 	if (!$rrtype && $rrclass && $rrclass eq 'ANY') {
@@ -303,9 +334,9 @@ sub new_from_string {
 			$rrclass = 'NONE';
 			$rrtype  = 'ANY';
 			$rdata   = '';
-		} elsif ($update_type =~ /^(rr_)?add$/) {
+		} elsif ($update_type =~ /^(rr_)?add$/o) {
 			$ttl = 86400 unless $ttl;
-		} elsif ($update_type =~ /^(rr_)?del(ete)?$/) {
+		} elsif ($update_type =~ /^(rr_)?del(ete)?$/o) {
 			$ttl     = 0;
 			$rrclass = $rdata ? 'NONE' : 'ANY';
 		}
@@ -323,19 +354,17 @@ sub new_from_string {
 		'rdata'    => '',
 	};
 
-	
-	# $rdata!~/^\s*\\\#/ means that the rdata does not start with \# (rfc3597)
 
-	if ($RR{$rrtype} && $rdata !~ m/^\s*\\\#/ ) {
+	if ($RR{$rrtype} && $rdata !~ m/^\s*\\#/o ) {
 		my $subclass = $class->_get_subclass($rrtype);
 		
 		return $subclass->new_from_string($self, $rdata);
 	} elsif ($RR{$rrtype}) {   # A RR type known to Net::DNS starting with \#
-		$rdata =~ m/\\\#\s+(\d+)\s+(.*)$/;
+		$rdata =~ m/\\\#\s+(\d+)\s+(.*)$/o;
 
 		my $rdlength = $1;
 		my $hexdump  = $2;		
-		$hexdump =~ s/\s*//g;
+		$hexdump =~ s/\s*//og;
 
 		die "$rdata is inconsistent; length does not match content" 
 			if length($hexdump) != $rdlength*2;
@@ -351,14 +380,14 @@ sub new_from_string {
 			\$rdata, 
 			length($rdata) - $rdlength
 		);
-	} elsif ($rdata=~/\s*\\\#\s+\d+\s+/) {   
+	} elsif ($rdata=~/\s*\\\#\s+\d+\s+/o) {   
 		#We are now dealing with the truly unknown.
 		die 'Expected RFC3597 representation of RDATA' 
-			unless $rdata =~ m/\\\#\s+(\d+)\s+(.*)$/;
+			unless $rdata =~ m/\\\#\s+(\d+)\s+(.*)$/o;
 
 		my $rdlength = $1;
 		my $hexdump  = $2;		
-		$hexdump =~ s/\s*//g;
+		$hexdump =~ s/\s*//og;
 
 		die "$rdata is inconsistent; length does not match content" 
 			if length($hexdump) != $rdlength*2;
@@ -417,6 +446,9 @@ sub new_from_hash {
 			# documentation
 			return $subclass->new_from_hash($self);
 	    }
+	} elsif ($self->{'type'} =~ /TYPE\d+/o) {
+		bless $self, 'Net::DNS::RR::Unknown';
+		return $self;
 	} else {
 	 	bless $self, $class;
 	 	return $self;
@@ -461,7 +493,7 @@ sub string {
 	 	 $self->{'ttl'},
 		 $self->{'class'},
 		 $self->{'type'},
-		 $self->rdatastr || '; no data',
+		 (defined $self->rdatastr and length $self->rdatastr) ? $self->rdatastr : '; no data',
    );
 }
 
@@ -581,12 +613,13 @@ sub data {
 		$data  = $packet->dn_comp($self->{'name'}, $offset);
 	}
 
+
 	my $qtype     = uc($self->{'type'});
-	my $qtype_val = ($qtype =~ m/^\d+$/) ? $qtype : Net::DNS::typesbyname($qtype);
+	my $qtype_val = ($qtype =~ m/^\d+$/o) ? $qtype : Net::DNS::typesbyname($qtype);
 	$qtype_val    = 0 if !defined($qtype_val);
 
 	my $qclass     = uc($self->{'class'});
-	my $qclass_val = ($qclass =~ m/^\d+$/) ? $qclass : Net::DNS::classesbyname($qclass);
+	my $qclass_val = ($qclass =~ m/^\d+$/o) ? $qclass : Net::DNS::classesbyname($qclass);
 	$qclass_val    = 0 if !defined($qclass_val);
 	$data .= pack('n', $qtype_val);
 	
@@ -605,7 +638,7 @@ sub data {
 	my $rdata = $self->rdata($packet, $offset);
 
 	$data .= pack('n', length $rdata);
-	$data .= $rdata;
+	$data.=$rdata;
 
 	return $data;
 }
@@ -625,12 +658,13 @@ sub _canonicaldata {
 	my $self = shift;
 	my $data='';
 	{   
-		my @dname= split /\./,lc($self->{'name'});
-		for (my $i=0;$i<@dname;$i++){
-			$data .= pack ('C',length $dname[$i] );
-			$data .= $dname[$i] ;
-		}
-		$data .= pack ('C','0');
+	    my $name=$self->{'name'};
+	    my @dname=Net::DNS::name2labels($name);
+	    for (my $i=0;$i<@dname;$i++){
+		$data .= pack ('C',length $dname[$i] );
+		$data .= lc($dname[$i] );
+	    }
+	    $data .= pack ('C','0');
 	}
 	$data .= pack('n', Net::DNS::typesbyname(uc($self->{'type'})));
 	$data .= pack('n', Net::DNS::classesbyname(uc($self->{'class'})));
@@ -650,73 +684,86 @@ sub _canonicaldata {
 # have domain names in them. Verification works only on RRs with
 # uncompressed domain names. (Canonical format as in sect 8 of
 # RFC2535) _canonicalRdata is overwritten in those RR objects that
-# have domain names in the RDATA and _name2label is used to convert a
+# have domain names in the RDATA and _name2wire is used to convert a
 # domain name to "wire format"
 
+
 sub _canonicalRdata {
-    my $self = shift;
-    my $rdata = $self->rr_rdata;
+    my $self=shift;
+    my $packet=Net::DNS::Packet->new();
+    my $rdata = $self->rr_rdata($packet,0);
     return $rdata;
 }
 
 
 
-sub _name2wire   {   
-    my ($self,$name)=@_;
 
-    my $rdata = '';
-    my @dname = split(m/\./, lc $name);
-    
+
+sub _name2wire   {   
+    my ($self, $name) = @_;
+
+    my $rdata="";
+    my $compname = "";
+    my @dname = Net::DNS::name2labels($name);
+
+
     for (@dname) {
 		$rdata .= pack('C', length $_);
 		$rdata .= $_ ;
     }
     
     $rdata .= pack('C', '0');
-    
     return $rdata;
 }
 
+
+
+
+
 sub AUTOLOAD {
 	my ($self) = @_;  # If we do shift here, it will mess up the goto below.
-	
-	my ($name) = $AUTOLOAD =~ m/^.*::(.*)$/;
-	
+	my ($name) = $AUTOLOAD =~ m/^.*::(.*)$/o;
+	if ($name =~ /set_rrsort_func/){
+	    return Net::DNS::RR::set_rrsort_func(@_);
+	}
+	if ($name =~ /get_rrsort_func/){
+	    return Net::DNS::RR::get_rrsort_func(@_);
+	}
 	# XXX -- We should test that we do in fact carp on unknown methods.	
 	unless (exists $self->{$name}) {
-		my $rr_string = $self->string;
-		Carp::carp(<<"AMEN");
-
+	    my $rr_string = $self->string;
+	    Carp::carp(<<"AMEN");
+	    
 ***
 ***  WARNING!!!  The program has attempted to call the method
 ***  "$name" for the following RR object:
 ***
 ***  $rr_string
 ***
-***  This object doesn't have a method "$name".  THIS IS A BUG
+***  This object doesn not have a method "$name".  THIS IS A BUG
 ***  IN THE CALLING SOFTWARE, which has incorrectly assumed that
 ***  the object would be of a particular type.  The calling
 ***  software should check the type of each RR object before
 ***  calling any of its methods.
 ***
 ***  Net::DNS has returned undef to the caller.
-***
+*** 
 
 AMEN
-		return;
+return;
 	}
 	
 	no strict q/refs/;
 	
 	# Build a method in the class.
 	*{$AUTOLOAD} = sub {
-		my ($self, $new_val) = @_;
-				
-		if (defined $new_val) {
-			$self->{$name} = $new_val;
-		}
-		
-		return $self->{$name};
+	    my ($self, $new_val) = @_;
+	    
+	    if (defined $new_val) {
+		$self->{$name} = $new_val;
+	    }
+	    
+	    return $self->{$name};
 	};
 	
 	# And jump over to it.
@@ -724,6 +771,7 @@ AMEN
 }
 
 
+
 #
 #  Net::DNS::RR->_get_subclass($type)
 #
@@ -744,7 +792,116 @@ sub _get_subclass {
 	
 	return $subclass;
 }	
+
+
+
+
+=head1 Sorting of RR arrays
+
+As of version 0.55 there is functionality to help you sort RR
+arrays. The sorting is done by Net::DNS::rrsort(), see the
+L<Net::DNS> documentation. This package provides class methods to set
+the sorting functions used for a particular RR based on a particular
+attribute.
+
+
+=head2 set_rrsort_func
+
+Net::DNS::RR::SRV->set_rrsort_func("priority",
+			       sub {
+				   my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
+				   $a->priority <=> $b->priority
+				   ||
+				   $b->weight <=> $a->weight
+                     }
+
+Net::DNS::RR::SRV->set_rrsort_func("default_sort",
+			       sub {
+				   my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
+				   $a->priority <=> $b->priority
+				   ||
+				   $b->weight <=> $a->weight
+                     }
+
+set_rrsort_func needs to be called as a class method. The first
+argument is the attribute name on which the sorting will need to take
+place. If you specify "default_sort" than that is the sort algorithm
+that will be used in the case that rrsort() is called without an RR
+attribute as argument.
+
+The second argument is a reference to a function that uses the
+variables $a and $b global to the C<from Net::DNS>(!!)package for the
+sorting. During the sorting $a and $b will contain references to
+objects from the class you called the set_prop_sort from. In other
+words, you can rest assured that the above sorting function will only
+get Net::DNS::RR::SRV objects.
+
+The above example is the sorting function that actually is implemented in 
+SRV.
+
+=cut
+
+
+
+
+sub set_rrsort_func{
+    my $class=shift;
+    my $attribute=shift;
+    my $funct=shift;
+#    print "Using ".__PACKAGE__."set_rrsort: $class\n";
+    my ($type) = $class =~ m/^.*::(.*)$/o;
+    $Net::DNS::RR::rrsortfunct{$type}{$attribute}=$funct;
+}
 		
+sub get_rrsort_func {
+    my $class=shift;    
+    my $attribute=shift;  #can be undefined.
+    my $sortsub;
+    my ($type) = $class =~ m/^.*::(.*)$/o;
+
+
+#    print "Using ".__PACKAGE__." get_rrsort: $class ($type,$attribute)\n";
+#    use Data::Dumper;
+#    print Dumper %Net::DNS::rrsortfunct;
+
+    if (defined($attribute) &&
+	exists($Net::DNS::RR::rrsortfunct{$type}) &&
+	exists($Net::DNS::RR::rrsortfunct{$type}{$attribute})
+	){
+	#  The default overwritten by the class variable in Net::DNS
+	return $Net::DNS::RR::rrsortfunct{$type}{$attribute};
+    }elsif(
+	! defined($attribute) &&
+	exists($Net::DNS::RR::rrsortfunct{$type}) &&
+	exists($Net::DNS::RR::rrsortfunct{$type}{'default_sort'})
+	){
+	#  The default overwritten by the class variable in Net::DNS
+	return $Net::DNS::RR::rrsortfunct{$type}{'default_sort'};
+    }
+    elsif( defined($attribute) ){
+	
+	return sub{
+	    my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
+	    ( exists($a->{$attribute}) &&   
+	      $a->{$attribute} <=> $b->{$attribute})
+		||
+		$a->_canonicaldata() cmp $b->_canonicaldata()
+	};
+    }else{
+	return sub{
+	    my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
+	    $a->_canonicaldata() cmp $b->_canonicaldata()
+	};
+    }    
+
+    return $sortsub;
+}
+
+
+
+
+
+
 	
 sub STORABLE_freeze {
 	my ($self, $cloning) = @_;
@@ -773,7 +930,9 @@ RR objects.
 
 Copyright (c) 1997-2002 Michael Fuhr. 
 
-Portions Copyright (c) 2002-2003 Chris Reinhardt.
+Portions Copyright (c) 2002-2004 Chris Reinhardt.
+
+Portions Copyright (c) 2005 Olaf Kolkman 
 
 All rights reserved.  This program is free software; you may redistribute
 it and/or modify it under the same terms as Perl itself.
