@@ -1,6 +1,6 @@
 # Plugin for Slimserver to monitor Server and Network Health
 
-# $Id: Plugin.pm 4587 2005-10-09 20:12:46Z adrian $
+# $Id: Plugin.pm 6717 2006-03-25 15:08:45Z adrian $
 
 # This code is derived from code with the following copyright message:
 #
@@ -61,12 +61,12 @@ sub summary {
 			push @warn, string("PLUGIN_HEALTH_STREAMINACTIVE_DESC");
 		}
 
-		if ($client->signalStrengthLog()->percentBelow(50) < 1) {
+		if ($client->signalStrengthLog()->percentBelow(30) < 1) {
 			$signal = string("PLUGIN_HEALTH_OK");
-		} elsif ($client->signalStrengthLog()->percentBelow(50) < 5) {
+		} elsif ($client->signalStrengthLog()->percentBelow(30) < 5) {
 			$signal = string("PLUGIN_HEALTH_SIGNAL_INTERMIT");
 			push @warn, string("PLUGIN_HEALTH_SIGNAL_INTERMIT_DESC");
-		} elsif ($client->signalStrengthLog()->percentBelow(50) < 20) {
+		} elsif ($client->signalStrengthLog()->percentBelow(30) < 20) {
 			$signal = string("PLUGIN_HEALTH_SIGNAL_POOR");
 			push @warn, string("PLUGIN_HEALTH_SIGNAL_POOR_DESC");
 		} else {
@@ -187,118 +187,147 @@ sub strings {
 PLUGIN_HEALTH
 	DE	Server & Netzwerk Zustand
 	EN	Server & Network Health
+	NL	Server- en netwerktoestand
 
 PLUGIN_HEALTH_PERF_ENABLE
 	DE	Leistungsüberwachung aktivieren
 	EN	Enable Performance Monitoring
+	NL	Schakel prestatiemonitoring in
 
 PLUGIN_HEALTH_PERF_DISABLE
 	DE	Leistungsüberwachung deaktivieren
 	EN	Disable Performance Monitoring
+	NL	Schakel prestatiemonitoring uit
 
 PLUGIN_HEALTH_PERF_CLEAR
 	DE	Zähler zurücksetzen
 	EN	Reset Counters
+	NL	Terugzetten tellers
 
 PLUGIN_HEALTH_PERF_UPDATE
 	DE	Seite aktualisieren
 	EN	Update Page
+	NL	Ververs pagina
 
 PLUGIN_HEALTH_PERFOFF_DESC
 	DE	Die Leistungsüberwachung ist zurzeit nicht aktiviert.
 	EN	Performance monitoring is not currently enabled on your server.
+	NL	Prestatiemonitoring is op dit moment niet actief op je server.
 
 PLUGIN_HEALTH_PERFON_DESC
 	DE	Die Leistungsüberwachung ist auf ihrem Server aktiviert. Der Server sammelt während der Ausführung Leistungsdaten.
 	EN	Performance monitoring is currently enabled on your server.	Performance statistics are being collected in the background while your server is running.
+	NL	Prestatiemonitoring is op dit moment actief op je server. Prestatiestatistieken worden bijgehouden in de achtergrond terwijl je server draait.
 
 PLUGIN_HEALTH_SUMMARY
 	DE	Zusammenfassung
 	EN	Summary
+	NL	Samenvatting
 
 PLUGIN_HEALTH_SUMMARY_DESC
-	DE	Bitte erstellen Sie eine Wiedergabeliste auf ihrem Player und starten Sie die Wiedergabe. Drücken Sie dann "Zähler zurücksetzen", um die Statistiken neu zu starten und die Anzeige zu aktualisieren. 
+	DE	Bitte erstellen Sie eine Wiedergabeliste auf ihrem Player und starten Sie die Wiedergabe. Drücken Sie dann "Zähler zurücksetzen", um die Statistiken neu zu starten und die Anzeige zu aktualisieren.
 	EN	Please queue up several tracks to play on this player and start them playing.  Then press the Reset Counters link above to clear the statistics and update this display.
+	NL	et een aantal liedjes in de playlist voor deze speler en start afspelen. Klik dan op Terugzetten tellers hierboven om de statistieken leeg te maken en het scherm bij te werken.
 
 PLUGIN_HEALTH_PLAYERDETAIL
 	DE	Player-Leistung
 	EN	Player Performance
+	NL	Speler prestatie
 
 PLUGIN_HEALTH_PLAYERDETAIL_DESC
 	DE	Die folgenden Graphen zeigen den Langzeit-Trend für alle Player-Leistungsdaten auf. Sie zeigen die Anzahl und den Prozentanteil der Messungen, die in eine bestimmte Wertekategorie fallen.<p>Es ist wichtig, den Player eine Weile Musik spielen zu lassen, um aussagekräftige Werte zu erhalten.
 	EN	The graphs shown here record the long term trend for each of the player performance measurements below.  They display the number and percentage of measurements which fall within each measurement band.<p>It is imporant to leave the player playing for a while and then assess the graphs.
+	NL	De hier getoonde grafieken laten de langetermijn trend zien voor de speler prestatiemetingen hieronder. Ze laten het getal en percentage zien van de metingen die in elk metingsgebied valt.  <br>Het is belangrijk om de speler een tijdje te laten spelen en daarna de grafieken te raadplegen.
 
 PLUGIN_HEALTH_SIGNAL
 	DE	Signalstärke
 	EN	Player Signal Strength
+	NL	Spelersignaalsterkte
 
 PLUGIN_HEALTH_SIGNAL_DESC
 	DE	Diese Graphik zeigt die Signalstärke der Wireless Netzwerkverbindung ihres Players. Höhere Werte sind besser. Der Player gibt die Signalstärke während der Wiedergabe zurück.
 	EN	This graph shows the strength of the wireless signal received by your player.  Higher signal strength is better.  The player reports signal strength while it is playing.
+	NL	Deze grafiek toont de signaalsterkte van je draadloze netwerk zoals ontvangen door je speler. Hogere signaalsterkte is beter. De speler rapporteert de signaalsterkte tijdens het afspelen.
 
 PLUGIN_HEALTH_BUFFER
 	DE	Puffer-Füllstand
 	EN	Buffer Fullness
+	NL	Bufferniveau
 
 PLUGIN_HEALTH_BUFFER_DESC
-	DE	Diese Graphik zeigt den Puffer-Füllstand ihres Players. Höhere Werte sind besser. Beachten Sie bitte, dass der Puffer nur während der Wiedergabe gefüllt wird.<p>Die Squeezebox1 besitzt nur einen kleinen Puffer, der während der Wiedergabe stets voll sein sollte. Fällt der Wert auf 0, so ist mit Aussetzern in der Wiedergabe zu rechnen. Dies wäre vermutlich auf Netzwerkprobleme zurückzuführen.<p>Die Squeezebox2 verwendet einen grossen Puffer. Dieser wird am Ende jedes wiedergegebenen Liedes geleert (Füllstand 0) um dann wieder aufzufüllen. Der Füllstand sollte also meist hoch sein.<p>Die Wiedergabe von Online-Radiostationen kann zu niedrigem Puffer-Füllstand führen, da der Player auf die Daten von einem entfernten Server warten muss. Dies ist normales Verhalten und kein Grund zur Beunruhigung. 
+	DE	Diese Graphik zeigt den Puffer-Füllstand ihres Players. Höhere Werte sind besser. Beachten Sie bitte, dass der Puffer nur während der Wiedergabe gefüllt wird.<p>Die Squeezebox1 besitzt nur einen kleinen Puffer, der während der Wiedergabe stets voll sein sollte. Fällt der Wert auf 0, so ist mit Aussetzern in der Wiedergabe zu rechnen. Dies wäre vermutlich auf Netzwerkprobleme zurückzuführen.<p>Die Squeezebox2 verwendet einen grossen Puffer. Dieser wird am Ende jedes wiedergegebenen Liedes geleert (Füllstand 0) um dann wieder aufzufüllen. Der Füllstand sollte also meist hoch sein.<p>Die Wiedergabe von Online-Radiostationen kann zu niedrigem Puffer-Füllstand führen, da der Player auf die Daten von einem entfernten Server warten muss. Dies ist normales Verhalten und kein Grund zur Beunruhigung.
 	EN	This graph shows the fill of the player\'s buffer.  Higher buffer fullness is better.  Note the buffer is only filled while the player is playing tracks.<p>Squeezebox1 uses a small buffer and it is expected to stay full while playing.  If this value drops to 0 it will result in audio dropouts.  This is likely to be due to network problems.<p>Squeezebox2 uses a large buffer.  This drains to 0 at the end of each track and then refills for the next track.  You should only be concerned if the buffer fill is not high for the majority of the time a track is playing.<p>Playing remote streams can lead to low buffer fill as the player needs to wait for data from the remote server.  This is not a cause for concern.
+	NL	Deze grafiek toont bufferniveau. Hoger niveau is beter. De buffer is alleen gevuld tijdens het afspelen van muziek.  <br>Squeezebox1 gebruikt een kleine buffer die normaal gesproken altijd vol is. Als het niveau naar 0 gaat zal er hapering in het geluid optreden. Dit komt vaak door netwerkproblemen.  <br>Squeezebox2/3 gebruikt een grote buffer. Hier loopt het bufferniveau naar 0 toe aan het einde van een liedje en vult zich weer aan het begin van het volgende liedje. Alleen als de buffer de meeste tijd niet gevuld is tijdens het spelen moet je actie nemen.  <br>Het spelen van streams op afstand (Internet radio) geeft een laag bufferniveau omdat de speler moet wachten op de server op afstand. Dit is geen gevolg van problemen.
 
 PLUGIN_HEALTH_CONTROL
 	DE	Kontrollverbindung
 	EN	Control Connection
+	NL	Controleconnectie
 
 PLUGIN_HEALTH_CONTROL_DESC
 	DE	Diese Graphik zeigt die Anzahl von aufgestauten Meldungen, die über die Kontroll-Verbindung zum Player geschickt werden sollten. Die Messung findet statt, wenn eine Meldung zum Player geschickt wird. Werte über 1-2 weisen auf eine mögliche Netzwerk-Überlastung hin, oder dass die Verbindung zum Player unterbrochen wurde.
 	EN	This graph shows the number of messages queued up to send to the player over the control connection.  A measurement is taken every time a new message is sent to the player.  Values above 1-2 indicate potential network congestion or that the player has become disconnected.
+	NL	Deze grafiek toont de hoeveelheid boodschappen in de rij gezet om te versturen naar de speler over de controleconnectie. Bij elke verstuurde boodschap wordt een meting gedaan. Waarden boven 1-2 geven een potentieel een netwerkcongestie aan of dat de speler losgekoppeld is van het netwerk.
 
 PLUGIN_HEALTH_STREAM
 	DE	Streaming-Verbindung
 	EN	Streaming Connection
+	NL	Streaming connectie
 
 PLUGIN_HEALTH_SERVER_PERF
 	DE	Server-Leistung
 	EN	Server Performance
+	NL	Serverprestatie
 
 PLUGIN_HEALTH_SERVER_PERF_DESC
 	DE	Die folgenden Graphen zeigen den Langzeit-Trend für alle Server-Leistungsdaten auf. Sie zeigen die Anzahl und den Prozentanteil der Messungen, die in eine bestimmte Wertekategorie fallen.
 	EN	The graphs shown here record the long term trend for each of the server performance measurements below.  They display the number and percentage of measurements which fall within each measurement band.
+	NL	De hier getoonde grafieken laten de langetermijn trend zien van de server prestatiemetingen hieronder. Ze laten het getal en percentage zien van de metingen die in elk metingsgebied valt.
 
 PLUGIN_HEALTH_TIMER_LATE
 	DE	Timer Genauigkeit
 	EN	Timer Accuracy
+	NL	Timeraccuraatheid
 
 PLUGIN_HEALTH_TIMER_LATE_DESC
-	DE	SlimServer benutzt einen Timer, um Ereignisse wie z.B. Updates der Programmoberfläche zu steuern. Diese Graphik zeigt die Genauigkeit, mit welcher Timer-gesteuerte Abläufe im Vergleich zum vorgesehenen zeitlichen Ablauf ausgeführt werden. Die Masseinheit ist Sekunden.<p>Aufgaben werden auf einen bestimmten Zeitpunkt festgelegt. Da stets nur ein Timer ablaufen kann und der Server auch andere Aktivitäten ausführt, kommt es stets zu einer minimalen Verzögerung. Kommt es allerdings zu einer markanten Verzögerung, so kann es zu wahrnehmbaren Störungen der Benutzeroberfläche kommen. 
+	DE	SlimServer benutzt einen Timer, um Ereignisse wie z.B. Updates der Programmoberfläche zu steuern. Diese Graphik zeigt die Genauigkeit, mit welcher Timer-gesteuerte Abläufe im Vergleich zum vorgesehenen zeitlichen Ablauf ausgeführt werden. Die Masseinheit ist Sekunden.<p>Aufgaben werden auf einen bestimmten Zeitpunkt festgelegt. Da stets nur ein Timer ablaufen kann und der Server auch andere Aktivitäten ausführt, kommt es stets zu einer minimalen Verzögerung. Kommt es allerdings zu einer markanten Verzögerung, so kann es zu wahrnehmbaren Störungen der Benutzeroberfläche kommen.
 	EN	Slimserver uses a timer mechanism to trigger events such as updating the user interface.  This graph shows how accurately each timer task is run relative to the time it was intended to be run.  It is measured in seconds.<p>Timer tasks are scheduled by the server to run at some point in the future.  As only one timer task can run at once and the server may also be performing other activity, timer tasks always run slightly after the time they are scheduled for.  However if timer tasks run significantly after they are scheduled this can become noticable through delay in the user interface.
+	NL	SlimServer gebruikt een timermechanisme om zaken zoals het bijwerken van de gebruikersinterface te activeren. Deze grafiek toont hoe accuraat elke timertaak is uitgevoerd relatief tot de tijd waarin het uitgevoerd had moeten worden. De uitkomst is in seconden.  <br>Timertaken worden gepland door de server om op een moment in de toekomst te draaien. Daar slechts &eacute;&eacute;n timertaak tegelijk kan draaien en de server ook andere taken uitvoert draaien timertaken altijd korte tijd nadat ze gepland zijn. Als timertaken echter significant later draaien dan gepland kan dit in de gebruikersinterface merkbaar worden als vertraging.
 
 PLUGIN_HEALTH_TIMER_LENGTH
-	DE	Timer Ausführungsdauer 
+	DE	Timer Ausführungsdauer
 	EN	Timer Task Duration
+	NL	Timertaakduur
 
 PLUGIN_HEALTH_TIMER_LENGTH_DESC
-	DE	Diese Graphik zeigt die Dauer, während der Timer-gesteuerte Abläufe ausgeführt werden. Die Masseinheit ist Sekunden. Braucht ein Vorgang länger als 0.5 Sekunden, so führt das mit grosser Wahrscheinlichkeit zu Störungen der Benutzeroberfläche.  
+	DE	Diese Graphik zeigt die Dauer, während der Timer-gesteuerte Abläufe ausgeführt werden. Die Masseinheit ist Sekunden. Braucht ein Vorgang länger als 0.5 Sekunden, so führt das mit grosser Wahrscheinlichkeit zu Störungen der Benutzeroberfläche.
 	EN	This graph shows how long each timer task runs for.  It is measured in seconds.  If any timer task takes more than 0.5 seconds this is likely to impact the user interface.
+	NL	Deze grafiek toont hoe lang elke timertaak duurt. De uitkomst is in seconden. Als een timertaak langer duurt dan 0.5 seconden is het waarschijnlijk dat deze impact heeft op de gebruikersinterface.
 
 PLUGIN_HEALTH_RESPONSE
 	DE	Server Antwortzeiten
 	EN	Server Response Time
+	NL	Serverreactietijd
 
 PLUGIN_HEALTH_RESPONSE_DESC
 	DE	Diese Graphik zeigt die Zeitdauer, die zwischen zwei Anfragen von beliebigen Playern vergeht. Die Masseinheit ist Sekunden. Geringere Werte sind besser. Antwortzeiten über einer Sekunde können zu Problemen bei der Audio-Wiedergabe führen.<p>Gründe für solche Verzögerungen können andere ausgeführte Programme oder komplexe Verarbeitungen im SlimServer sein.
 	EN	This graph shows the length of time between slimserver responding to requests from any player.  It is measured in seconds. Lower numbers are better.  If you notice response times of over 1 second this could lead to problems with audio performance.<p>The cause of long response times could be either other programs running on the server or slimserver processing a complex task.
+	NL	Deze grafiek toont de tijd waarbinnen SlimServer reageert op verzoeken van de speler. De uitkomst is in seconden. Lagere waardes zijn beter. Als je reactietijden hebt van meer dan 1 seconde kan dit leiden tot problemen bij afspelen van audio.  <br>De oorzaak van lange reactietijden kan liggen bij andere programma\'s die draaien op de server of dat SlimServer een complexe taak uitvoert.
 
 PLUGIN_HEALTH_SCHEDULER
 	DE	Geplante Aufgaben
 	EN	Scheduled Tasks
+	NL	Geplande taken
 
 PLUGIN_HEALTH_SCHEDULER_DESC
-	DE	Der Server führt Prozessor-intensive Aufgaben wie z.B. das Durchsuchen nach neuen Musikstücken in Etappen aus, welche zwischen Anfragen von Playern durchgeführt werden. Diese Graphik zeigt die Länge in Sekunden, die eine Ausführung dauert, bevor der Server die Kontrolle wieder übernehmen kann. Aufgaben, welche länger als 0.5 Sekunden dauern, können zu Störungen der Benutzeroberfläche führen. 
+	DE	Der Server führt Prozessor-intensive Aufgaben wie z.B. das Durchsuchen nach neuen Musikstücken in Etappen aus, welche zwischen Anfragen von Playern durchgeführt werden. Diese Graphik zeigt die Länge in Sekunden, die eine Ausführung dauert, bevor der Server die Kontrolle wieder übernehmen kann. Aufgaben, welche länger als 0.5 Sekunden dauern, können zu Störungen der Benutzeroberfläche führen.
 	EN	The server runs processor intensive tasks (such as scanning your music collection) by breaking them into short pieces which are scheduled when when active players are not requesting data.  This graph shows the length of time in seconds that a scheduled task runs for before returning control to the server.  Tasks taking over 0.5 second may lead to reduced performance for the user interface.
+	NL	De server draait processorintensieve taken (zoals het scannen van je muziekcollectie) door deze op te breken in korte stukken die vervolgens gepland worden op momenten dat spelers niet vragen om gegevens. Deze grafiek toont de tijd in seconden dat een geplande taak draait voordat hij controle teruggeeft aan de server. Taken die meer tijd in beslag nemen dan 0.5 seconden kunnen leiden tot een slechtere prestatie van de gebruikersinterface (haperende menu\'s).
 
 PLUGIN_HEALTH_WARNINGS
 	DE	Warnungen
 	EN	Warnings
+	NL	Waarschuwingen
 
 PLUGIN_HEALTH_OK
 	EN	OK
@@ -306,91 +335,112 @@ PLUGIN_HEALTH_OK
 PLUGIN_HEALTH_FAIL
 	DE	Gestört
 	EN	Fail
+	NL	Gefaald
 
 PLUGIN_HEALTH_CONGEST
 	DE	Überlastung
 	EN	Congested
+	NL	Congestie
 
 PLUGIN_HEALTH_INACTIVE
 	DE	Inaktiv
 	EN	Inactive
+	NL	Inactief
 
 PLUGIN_HEALTH_STREAMINACTIVE_DESC
 	DE	Derzeit ist keine aktive Verbindung für diesen Player vorhanden. Eine Verbindung wird aufgebaut, wenn Sie eine Datei vom Server wiedergeben, nicht aber, wenn Sie eine Internet Radio-Station af einer Squeezebox2 hören.<p>Falls Sie versuchen, eine lokale Datei auf diesem Player abzuspielen, so deutet dies auf ein Netzwerkproblem hin. Bitte überprüfen Sie die Netzwerkkonfiguration und/oder Firewall (TCP Port 9000 darf nicht blockiert sein).
 	EN	There is currently no active connection for streaming to this player.  A connection is required whenever you play a file from the server (but not when you play remote radio streams on a Squeezebox2 player).<p>If you are attempting to play a local file on this player, then this indicates a network problem.  Please check that your network and/or server firewall do not block connections to TCP port 9000.
+	NL	Er is op dit moment geen actieve connectie voor het streamen naar deze speler. Een connectie is altijd nodig om bestanden te spelen vanaf de server (maar niet als je een radiostream op afstand gebruikt bij een Squeezebox2 of 3)  <br>  Als je een lokaal bestand probeert af te spelen dan wijst dit op een netwerkprobleem. Controleer of je netwerk en/of server firewall niet TCP poort 9000 blokkeren.
 
 PLUGIN_HEALTH_CONTROLFAIL_DESC
 	DE	Derzeit ist keine aktive Kontroll-Verbindung für diesen Player vorhanden. Bitte stellen Sie sicher, dass das Gerät eingeschaltet ist. Falls der Player keine Netzwerkverbindung aufbauen kann, überprüfen sie bitte die Netzwerkkonfiguration und/oder Firewall. Diese darf TCP und UPD Ports 3483 nicht blockieren.
 	EN	There is no currently active control connection to this player.  Please check the player is powered on.  If the player is unable to establish a connection, please check your network and and/or server firewall do not block connections to TCP & UDP port 3483.
+	NL	Er is momenteel geen actieve controleconnectie naar deze speler. Controleer of de speler aan staat. Controleer of je netwerk en/of server firewall geen connecties blokkeren naar TCP & UDP poort 3483 als je speler geen connectie kan maken.
 
 PLUGIN_HEALTH_CONTROLCONGEST_DESC
 	DE	Die Kontroll-Verbindung zu diesem Player hat Überlastungen erfahren. Dies ist üblicherweise ein Hinweis auf schlechte Netzwerkverbindung, oder dass das Gerät vor kurzem vom Netz genommen wurde.
 	EN	The control connection to this player has experienced congestion.  This usually is an indication of poor network connectivity (or the player being recently being disconnected from the network).
+	NL	De controleconnectie naar deze speler heeft last gehad van congestie. Dit is meestal een indicatie van een slechte netwerkconnectie (of een speler die recent van het netwerk losgekoppeld is geweest).
 
 PLUGIN_HEALTH_SIGNAL_INTERMIT
-	DE	Gut, aber mit vereinzelten Ausfällen 
+	DE	Gut, aber mit vereinzelten Ausfällen
 	EN	Good, but Intermittent Drops
+	NL	Goed maar af en toe haperingen
 
 PLUGIN_HEALTH_SIGNAL_INTERMIT_DESC
 	DE	Die Signalstärke dieses Players ist im Grossen und Ganzen gut, hatte aber vereinzelte Ausfälle. Dies kann auf andere Wireless Netzwerke, kabellose Telephone oder Mikrowellen-Öfen zurückzuführen sein. Falls Sie vereinzelte Ton-Aussetzer wahrnehmen, so sollten Sie der Ursache des Problems nachgehen.
 	EN	The signal strength received by this player is normally good, but occasionally drops.  This may be caused by other wireless networks, cordless phones or microwaves nearby.  If you hear occasional audio dropouts on this player, you should investigate what is causing drops in signal strength.
+	NL	De signaalsterkte ontvangen door de speler is goed met af en toe haperingen. De oorzaak kunnen andere draadloze netwerken zijn, draadloze telefoons of magnetrons die dichtbij zijn. Als je haperingen hoort in de audio moet je de oorzaak onderzoeken van de haperingen in de signaalsterkte.
 
 PLUGIN_HEALTH_SIGNAL_POOR
 	DE	Schwach
 	EN	Poor
+	NL	Matig
 
 PLUGIN_HEALTH_SIGNAL_POOR_DESC
 	DE	Die Signalstärke dieses Players ist grösstenteils schwach. Bitte überprüfen Sie das Wireless Netzwerk.
 	EN	The signal strength received by this player is poor for significant periods, please check your wireless network.
+	NL	De signaalsterkte ontvangen door de speler is matig over een langere periode. Controleer je draadloze netwerk.
 
 PLUGIN_HEALTH_SIGNAL_BAD
 	DE	Schlecht
 	EN	Bad
+	NL	Slecht
 
 PLUGIN_HEALTH_SIGNAL_BAD_DESC
 	DE	Die Signalstärke dieses Players ist grösstenteils schlecht. Bitte überprüfen Sie das Wireless Netzwerk.
 	EN	The signal strength received by this player is bad for significant periods, please check your wireless network.
+	NL	De signaalsterkte ontvangen door je speler is slecht over een  aanzienlijke periode. Controleer je draadloze netwerk.
 
 PLUGIN_HEALTH_BUFFER_LOW
 	DE	Niedrig
 	EN	Low
+	NL	Laag
 
 PLUGIN_HEALTH_BUFFER_LOW_DESC1
 	DE	Der Wiedergabe-Puffer dieses Players ist zeitweise niedriger als wünschenswert. Dies kann zu Tonaussetzern führen, v.a. falls Sie WAV oder AIFF verwenden. Falls Sie solche Aussetzer wahrnehmen, überprüfen Sie bitte die Signalstärke und Server Antwortzeiten.
 	EN	The playback buffer for this player is occasionally falling lower than ideal.  This may result in audio dropouts especually if you are streaming as WAV/AIFF.  If you are hearing these, please check your network signal strength and server response times.
+	NL	De afspeelbuffer van deze speler is af en toe minder gevuld dan in de ideale situatie. Dit kan resulteren in audio haperingen, zeker als je WAV/AIFF streamt. Controleer de netwerksignaalsterkte en de snelheid waarmee de server reageert als je haperingen hoort.
 
 PLUGIN_HEALTH_BUFFER_LOW_DESC2
 	DE	Der Wiedergabe-Puffer dieses Players ist zeitweise niedriger als wünschenswert. Dies ist eine Squeezebox2, es ist daher normal, dass der Puffer am Ende eines Liedes geleert wird. Diese Warnung wird ev. angezeigt, falls Sie viele kurze Lieder wiedergeben. Falls Sie Tonaussetzer feststellen, überprüfen Sie bitte die Signalstärke.
 	EN	The playback buffer for this player is occasionally falling lower than ideal.  This is a Squeezebox2 and so the buffer fullness is expected to drop at the end of each track.  You may see this warning if you are playing lots of short tracks.  If you are hearing audio dropouts, please check our network signal strength.
+	NL	De afspeelbuffer van deze speler is af en toe minder gevuld dan in de ideale situatie. Dit is een Squeezebox2. Daar mag het bufferniveau laag zijn aan het einde van een liedje. Je kunt deze waarschuwing krijgen als je veel korte liedjes afspeelt. Controleer de netwerksignaalsterkte als je haperingen hoort in het geluid.
 
 PLUGIN_HEALTH_RESPONSE_INTERMIT
 	DE	Teilweise schlechte Antwortzeiten
 	EN	Occasional Poor Response
+	NL	Af en toe slechte reactietijd
 
 PLUGIN_HEALTH_RESPONSE_INTERMIT_DESC
 	DE	Die Antwortzeiten des Servers sind zeitweise länger als wünschenswert. Dies kann zu hörbaren Tonaussetzern führen, v.a. auf SliMP3 und Squeezebox1 Playern. Gründe hierfür können andere laufene Programme im Hintergrund oder komplexe Aufgaben im Slimserver sein.
 	EN	Your server response time is occasionally longer than desired.  This may cause audio dropouts, especially on Slimp3 and Squeezebox1 players.  It may be due to background load on your server or a slimserver task taking longer than normal.
+	NL	De serverreactietijd is af en toe lager dan gewenst. Dit kan audio haperingen veroorzaken, zeker bij de Slimp3 en Squeezebox1 spelers. De oorzaak kunnen de overige programma\'s zijn die op je server draaien of een SlimServer taak die langer duurt dan normaal.
 
 PLUGIN_HEALTH_RESPONSE_POOR
 	DE	Schlechte Antwortzeiten
 	EN	Poor Response
+	NL	Slechte reactietijd
 
 PLUGIN_HEALTH_RESPONSE_POOR_DESC
 	DE	Die Antwortzeiten des Servers sind oft länger als wünschenswert. Dies kann zu hörbaren Tonaussetzern führen, v.a. auf SliMP3 und Squeezebox1 Playern. Überprüfen Sie bitte die Leistung ihres Servers. Falls diese ok ist, vergewissern Sie sich, ob SlimServer komplexe Aufgaben (z.B. Durchsuchen der Musiksammlung) durchführt oder ein Plugin die Ursache für das Problem darstellt.
 	EN	Your server response time is regularly falling below normal performance levels.  This may lead to audio dropouts, especially on Slimp3 and Squeezebox1 players.  Please check the performance of your server.  If this is OK, then check slimserver is not running intensive tasks (e.g. scanning music library) or a Plugin is not causing this.
+	NL	De serverreactietijd is regelmatig lager dan gewenst. Dit kan audio haperingen veroorzaken, zeker bij de Slimp3 en Squeezebox1 spelers. Controleer de prestatie van je server. Is die goed, controleer dan of SlimServer geen intensieve taken draait (zoals scannen van de muziekcollectie) of dat een plugin dit veroorzaakt.
 
 PLUGIN_HEALTH_NORMAL
 	DE	Dieser Player verhält sich normal.
 	EN	This player is performing normally.
+	NL	Deze speler functioneert normaal.
 
 PLUGIN_HEALTH_NO_PLAYER_DESC
 	DE	SlimServer kann keinen Player finden. Falls einer angeschlossen ist, so kann dies durch eine blockierte Netzwerkverbindung ausgelöst werden. Überprüfen sie bitte die Netzwerkkonfiguration und/oder Firewall. Diese darf TCP und UPD Ports 3483 nicht blockieren.
 	EN	Slimserver cannot find a player.  If you own a player this could be due to your network blocking connection between the player and server.  Please check your network and/or server firewall does not block connection to TCP & UDP port 3483.
+	NL	SlimServer kan geen speler vinden. Als je een speler hebt kan dit komen door een netwerk dat connecties blokkeert tussen de speler en server. Controleer of je netwerk en/of server firewall niet TCP & UDP poort 3483 blokkeert.
 
 PLUGIN_HEALTH_SLIMP3_DESC
 	DE	Sie verwenden einen SliMP3 Player. Für diesen stehen nicht die vollen Messungen zur Verfügung.
 	EN	This is a SLIMP3 player.  Full performance measurements are not available for this player.
-
+	NL	Dit is een Slimp3 speler. Volledige prestatiemonitoring is niet beschikbaar voor deze speler.
 '
 }
 
