@@ -4,6 +4,7 @@ package Plugins::MoodLogic::Plugin;
 use strict;
 
 use File::Spec::Functions qw(catfile);
+use Scalar::Util qw(blessed);
 
 use Slim::Utils::Misc;
 use Slim::Utils::Strings qw(string);
@@ -406,6 +407,7 @@ sub exportFunction {
 
 			$::d_moodlogic && msg("MoodLogic: Couldn't create track for: $url\n");
 
+			$isScanning++;
 			return 1;
 		};
 
@@ -446,12 +448,20 @@ sub exportFunction {
 	
 	$::d_moodlogic && msg("MoodLogic: Song scan complete, checking playlists\n");
 
+	if (blessed($playlist) && !$playlist->EOF) {
 
-	if (defined $playlist && !$playlist->EOF) {
+		my $nameObj    = $playlist->Fields('name');
 
-		my $name = $playlist->Fields('name')->value;
+		# Bug: 2763 - somehow the name object can be undef
+		if (!blessed($nameObj)) {
+
+			errorMsg("MoodLogic: Playlist scan - couldn't find a valid object for Fields->('name')\n");
+			return 1;
+		}
+
+		my $name       = $nameObj->value;
 		my %cacheEntry = ();
-		my $url = 'moodlogicplaylist:' . Slim::Web::HTTP::escape($name);
+		my $url        = 'moodlogicplaylist:' . Slim::Web::HTTP::escape($name);
 
 		$::d_moodlogic && msg("MoodLogic: Found MoodLogic Playlist: $url\n");
 
@@ -807,8 +817,25 @@ sub mood_wheel {
 	#$params->{'pwd_list'} = Slim::Web::Pages::generate_pwd_list($genre, $artist, $album, $player,$params->{'webroot'});
 	
 	$params->{'pwd_list'} .= ${Slim::Web::HTTP::filltemplatefile("plugins/MoodLogic/mood_wheel_pwdlist.html", $params)};
-	$params->{'mood_list'} = $items;
-
+	
+	for my $item (@$items) { 	 
+	
+		my %list_form = %$params; 	 
+		
+		$list_form{'mood'}     = $item; 	 
+		$list_form{'genre'}    = $genre; 	 
+		$list_form{'artist'}   = $artist; 	 
+		$list_form{'album'}    = $album; 	 
+		$list_form{'player'}   = $player; 	 
+		$list_form{'itempath'} = $item; 	 
+		$list_form{'item'}     = $item; 	 
+		$list_form{'odd'}      = ($itemnumber + 1) % 2; 	 
+		
+		$itemnumber++; 	 
+		
+		$params->{'mood_list'} .= ${Slim::Web::HTTP::filltemplatefile("plugins/MoodLogic/mood_wheel_list.html", \%list_form)}; 	 
+	}
+	
 	return Slim::Web::HTTP::filltemplatefile("plugins/MoodLogic/mood_wheel.html", $params);
 }
 
