@@ -959,16 +959,27 @@ sub getMix {
 	
 	my %args = (
 		# Set the size of the list (default 12)
-		size	 => $client->prefGet('MMMSize') || Slim::Utils::Prefs::get('MMMSize'),
+		size	   => $client->prefGet('MMMSize') || Slim::Utils::Prefs::get('MMMSize'),
 
 		# (tracks|min|mb) Set the units for size (default tracks)
-		sizetype => $type[$client->prefGet('MMMMixType') || Slim::Utils::Prefs::get('MMMMixType')],
+		sizetype   => $type[$client->prefGet('MMMMixType') || Slim::Utils::Prefs::get('MMMMixType')],
 
 		# Set the style slider (default 20)
-		style	 => $client->prefGet('MMMStyle') || Slim::Utils::Prefs::get('MMMStyle'),
+		style	   => $client->prefGet('MMMStyle') || Slim::Utils::Prefs::get('MMMStyle'),
 
 		# Set the variety slider (default 0)
-		variety	 => $client->prefGet('MMMVariety') || Slim::Utils::Prefs::get('MMMVariety'),
+		variety	   => $client->prefGet('MMMVariety') || Slim::Utils::Prefs::get('MMMVariety'),
+
+		# Let the mix use other genres
+		mixgenre   => 0,
+
+		# Set the number of songs before allowing dupes (default 12)
+		# currently sets to the selected list size.
+		rejectsize => $client->prefGet('MMMSize') || Slim::Utils::Prefs::get('MMMSize'),
+
+		# (tracks|min|mb) Set the units for rejecting dupes (default tracks)
+		# currently just matches the mix type
+		rejecttype => $type[$client->prefGet('MMMMixType') || Slim::Utils::Prefs::get('MMMMixType')],
 	);
 
 	my $filter = $client->prefGet('MMMFilter') || Slim::Utils::Prefs::get('MMMFilter');
