@@ -89,14 +89,16 @@ sub getGenres {
 	
 	# Extract each genre name into a hash
 	my %clientGenres = ();
+
 	foreach my $item (@$items) {
+
 		$clientGenres{$item->{'name'}} = {
-		                                 # Put the name here as well so the hash can be passed to
-		                                 # INPUT.Choice as part of listRef later on
-		                                 name    => $item->{'name'},
-		                                 id      => $item->{'id'},
-		                                 enabled => 1,
-									 };
+			 # Put the name here as well so the hash can be passed to
+			 # INPUT.Choice as part of listRef later on
+			 name    => $item->name,
+			 id      => $item->id,
+			 enabled => 1,
+		};
 	}
 
 	my @exclude = Slim::Utils::Prefs::getArray('plugin_random_exclude_genres');
@@ -649,124 +651,165 @@ PLUGIN_RANDOM
 	DE	Zufalls Mix
 	EN	Random Mix
 	ES	Mezcla al azar
+	FR	Mix aléatoire
+	NL	Willekeurige mix
 
 PLUGIN_RANDOM_DISABLED
 	DE	Zufalls Mix angehalten
 	EN	Random Mix Stopped
+	NL	Willekeurige mix gestopt
 
 PLUGIN_RANDOM_TRACK
 	DE	Zufälliger Lieder Mix
 	EN	Random Song Mix
+	NL	Willekeurige liedjes mix
 
 PLUGIN_RANDOM_TRACK_PLAYING
 	DE	Spiele zufällige Liederauswahl
 	EN	Playing Random Songs
+	NL	Afspelen willekeurige liedjes
 
 PLUGIN_RANDOM_TRACK_DISABLE
 	DE	keine Lieder mehr hinzuzufügen
 	EN	stop adding songs
+	NL	Stop toevoegen liedjes
 
 PLUGIN_RANDOM_ALBUM
 	DE	Zufälliger Album Mix
 	EN	Random Album Mix
-	ES	Mezcla al azar por ?lbum
+	ES	Mezcla al azar por Álbum
+	NL	Willekeurige album mix
 
 PLUGIN_RANDOM_ALBUM_ITEM
-	DE	Zufälliges album
+	DE	Zufälliges Album
 	EN	Random Album
+	NL	Willekeurig album
 
 PLUGIN_RANDOM_ALBUM_PLAYING
 	DE	Spiele zufällige Albenauswahl
 	EN	Playing Random Albums
+	NL	Afspelen willekeurige albums
 
 PLUGIN_RANDOM_ALBUM_DISABLE
 	DE	keine Alben mehr hinzuzufügen
 	EN	stop adding albums
+	NL	Stoppen toevoegen albums
 
 PLUGIN_RANDOM_ARTIST
 	DE	Zufälliger Interpreten Mix
 	EN	Random Artist Mix
+	NL	Willekeurige artiesten mix
 
 PLUGIN_RANDOM_ARTIST_ITEM
 	DE	Zufälliger Interpret
 	EN	Random Artist
+	NL	Willekeurige artiesten
 
 PLUGIN_RANDOM_ARTIST_PLAYING
 	DE	Spiele zufälligen Interpreten
 	EN	Playing Random Artists
+	NL	Afspelen willekeurige artiesten
 
 PLUGIN_RANDOM_ARTIST_DISABLE
 	DE	keine Interpreten mehr hinzuzufügen
 	EN	stop adding artists
+	NL	Stop toevoegen artiesten
 
 PLUGIN_RANDOM_YEAR
 	DE	Zufälliger Jahr Mix
 	EN	Random Year Mix
+	NL	Willekeurige jaar mix
 
 PLUGIN_RANDOM_YEAR_ITEM
 	DE	Zufälliger Jahrgang
 	EN	Random Year
+	NL	Willekeurig jaar
 
 PLUGIN_RANDOM_YEAR_PLAYING
 	DE	Spiele zufälligen Jahrgang
 	EN	Playing Random Years
+	NL	Afspelen willekeurige jaren
 
 PLUGIN_RANDOM_YEAR_DISABLE
 	DE	keine Jahrgänge mehr hinzuzufügen
 	EN	stop adding years
+	NL	Stop toevoegen jaren
 
 PLUGIN_RANDOM_GENRE_FILTER
 	DE	Zu berücksichtigende Stile wählen
 	EN	Select Genres To Include
+	FR	Sélectionnez les genres à inclure
+	NL	Selecteer genres om toe te voegen
 
 PLUGIN_RANDOM_SELECT_ALL
 	DE	Alle wählen
 	EN	Select All
+	FR	Tous les genres
+	NL	Selecteer alles
 
 PLUGIN_RANDOM_SELECT_NONE
 	DE	Alle abwählen
 	EN	Select None
+	NL	Niets selecteren
 
 PLUGIN_RANDOM_CHOOSE_BELOW
 	DE	Wählen Sie einen zufälligen Mix von Musik aus Ihrer Sammlung:
 	EN	Choose a random mix of music from your library:
+	FR	Sélectionnez un mix aléatoire de morceaux tirés de votre collection :
+	NL	Kies een willekeurige mix van je muziek collectie:
 
 PLUGIN_RANDOM_TRACK_WEB
 	DE	Zufällige Lieder
 	EN	Random songs
+	FR	Morceaux aléatoires
+	NL	Willekeurige liedjes
 
 PLUGIN_RANDOM_ARTIST_WEB
 	DE	Zufällige Interpreten
 	EN	Random artists
+	FR	Artistes aléatoires
+	NL	Willekeurige artiesten
 
 PLUGIN_RANDOM_ALBUM_WEB
 	DE	Zufällige Alben
 	EN	Random albums
+	FR	Albums aléatoires
+	NL	Willekeurige albums
 
 PLUGIN_RANDOM_YEAR_WEB
 	DE	Zufällige Jahrgänge
 	EN	Random years
+	FR	Années aléatoires
+	NL	Willekeurige jaren
 
 PLUGIN_RANDOM_GENRE_FILTER_WEB
-	DE	Im Mix zu berücksichtigende Stile: 
+	DE	Im Mix zu berücksichtigende Stile:
 	EN	Genres to include in your mix:
+	FR	Genres à inclure dans le mix :
+	NL	Genres om op te nemen in je mix:
 
 PLUGIN_RANDOM_BEFORE_NUM_TRACKS
 	DE	Die Wiedergabeliste wird
 	EN	Now Playing will show
+	FR	Lecture en cours affichera
+	NL	Speelt nu zal tonen:
 
 PLUGIN_RANDOM_AFTER_NUM_TRACKS
 	DE	noch abzuspielende und
 	EN	upcoming songs and
+	FR	morceau(x) à venir et
+	NL	komende liedjes en
 
 PLUGIN_RANDOM_AFTER_NUM_OLD_TRACKS
 	DE	wiedergegebene Lieder anzeigen.
 	EN	recently played songs.
+	FR	morceau(x) joué(s) précédemment.
+	NL	recent gespeelde liedjes.
 
 PLUGIN_RANDOM_GENERAL_HELP
 	DE	Sie können jederzeit Lieder aus dem Mix entfernen oder neue hinzufügen. Um den Zufallsmix anzuhalten löschen Sie bitte die Wiedergabeliste oder klicken, um
 	EN	You can add or remove songs from your mix at any time. To stop a random mix, clear your playlist or click to
-
+	NL	Je kunt op elk moment liedjes toevoegen bij of verwijderen uit je mix. Om de willekeurige mix te stoppen maak je de playlist leeg of klik op
 EOF
 
 }
