@@ -15,12 +15,6 @@ package Plugins::iTunes;
 #	itunes	-- 1 to attempt to use iTunes library XML file,
 #		0 to simply scan filesystem.
 #
-#	itunes_library_autolocate
-#		-- if this is set (1), attempt to automatically set both
-#		itunes_library_xml_path or itunes_library_music_path.  If
-#		this is unset (0) or undefined, you MUST explicitly set both
-#		itunes_library_xml_path and itunes_library_music_path.
-#
 #	itunes_library_xml_path
 #		-- full path to 'iTunes Music Library.xml' file.
 #
@@ -54,6 +48,7 @@ use Slim::Utils::Strings qw(string);
 
 my $lastMusicLibraryFinishTime = undef;
 my $lastITunesMusicLibraryDate = 0;
+my $currentITunesMusicLibraryDate = 0;
 my $iTunesScanStartTime = 0;
 
 my $isScanning = 0;
@@ -66,7 +61,6 @@ my $inTracks;
 our %tracks;
 
 my $iTunesLibraryFile;
-my $iTunesLibraryPath;
 my $iTunesParser;
 my $iTunesParserNB;
 my $offset = 0;
@@ -114,16 +108,6 @@ sub useiTunesLibrary {
 	
 	my $can = canUseiTunesLibrary();
 
-	if (!defined($use) && $can) {
-
-		Slim::Utils::Prefs::set('itunes', 1);
-
-	} elsif (!defined($use) && !$can) {
-
-		Slim::Utils::Prefs::set('itunes', 0);
-	}
-
-	$use = Slim::Utils::Prefs::get('itunes');
 	Slim::Music::Import::useImporter('ITUNES',$use && $can);
 
 	$::d_itunes && msg("iTunes: using itunes library: $use\n");
@@ -137,21 +121,7 @@ sub canUseiTunesLibrary {
 
 	checkDefaults();
 
-	my $oldMusicPath = Slim::Utils::Prefs::get('itunes_library_music_path');
-
-	$iTunesLibraryPath = defined $iTunesLibraryPath ? $iTunesLibraryPath : findMusicLibrary();
-
-	# The user may have moved their music folder location. We need to nuke the db.
-	if ($iTunesLibraryPath && $oldMusicPath && $oldMusicPath ne $iTunesLibraryPath) {
-
-		$::d_itunes && msg("iTunes: Music Folder has changed from previous - wiping db\n");
-
-		Slim::Music::Info::wipeDBCache();
-
-		$lastITunesMusicLibraryDate = -1;
-	}
-
-	return defined findMusicLibraryFile() && defined $iTunesLibraryPath;
+	return defined findMusicLibraryFile();
 }
 
 sub getDisplayName {
@@ -167,10 +137,10 @@ sub getFunctions {
 }
 
 sub initPlugin {
-	return 1 if $initialized;
-
 	addGroups();
 
+	return 1 if $initialized;
+
 	return unless canUseiTunesLibrary();
 
 	Slim::Music::Import::addImporter('ITUNES', {
@@ -180,7 +150,7 @@ sub initPlugin {
 
 	Slim::Music::Import::useImporter('ITUNES',Slim::Utils::Prefs::get('itunes'));
 	Slim::Player::Source::registerProtocolHandler("itunesplaylist", "0");
-
+	
 	$initialized = 1;
 
 	# Pass checker a value, to let it know that we're just seeing if we're
@@ -325,113 +295,72 @@ sub findLibraryFromRegistry {
 
 sub findMusicLibraryFile {
 
-	my $path = undef;
+	my $explicit_xml_path = Slim::Utils::Prefs::get('itunes_library_xml_path');
 
+	if ($explicit_xml_path) {
+		if (-d $explicit_xml_path) {
+			$explicit_xml_path =  catfile(($explicit_xml_path), 'iTunes Music Library.xml');
+		}
+
+		if (-r $explicit_xml_path) {
+			return $explicit_xml_path;
+		}
+	}		
+
+	$::d_itunes && msg("iTunes: attempting to locate iTunes Music Library.xml automatically\n");
+	my $path = undef;
 	my $base = "";
 	$base = $ENV{'HOME'} if $ENV{'HOME'};
 
-	my $audiodir = Slim::Utils::Prefs::get('audiodir');
-	my $autolocate = Slim::Utils::Prefs::get('itunes_library_autolocate');
+	$path = findLibraryFromPlist($base);
 
-	if ($autolocate) {
-		$::d_itunes && msg("iTunes: attempting to locate iTunes Music Library.xml\n");
-
-		# This defines the list of directories we will search for
-		# the 'iTunes Music Library.xml' file.
-		my @searchdirs = (
-			catdir($base, 'Music', 'iTunes'),
-			catdir($base, 'Documents', 'iTunes'),
-			$base,
-		);
-
-		if (defined $audiodir) {
-			push @searchdirs, (
-				catdir($audiodir, 'My Music', 'iTunes'),
-				catdir($audiodir, 'iTunes'),
-				$audiodir
-			);
-		}
-
-		$path = findLibraryFromPlist($base);
-
-		if ($path && -r $path) {
-			$::d_itunes && msg("iTunes: found path via iTunes preferences at: $path\n");
-			return $path;
-		}
-
-		$path = findLibraryFromRegistry();
-
-		if ($path && -r $path) {
-			$::d_itunes && msg("iTunes: found path via Windows registry at: $path\n");
-			return $path;
-		}
-
-		for my $dir (@searchdirs) {
-			$path = catfile(($dir), 'iTunes Music Library.xml');
-
-			if ($path && -r $path) {
-				$::d_itunes && msg("iTunes: found path via directory search at: $path\n");
-				Slim::Utils::Prefs::set('itunes_library_xml_path',$path);
-				return $path;
-			}
-
-		}
+	if ($path && -r $path) {
+		$::d_itunes && msg("iTunes: found path via iTunes preferences at: $path\n");
+		Slim::Utils::Prefs::set( 'itunes_library_xml_path', $path );
+		return $path;
 	}
 
-	if (!$path) {
-		$path = Slim::Utils::Prefs::get('itunes_library_xml_path');
+	$path = findLibraryFromRegistry();
 
-		if ($path && -d $path) {
-			$path = catfile(($path), 'iTunes Music Library.xml');
-		}
+	if ($path && -r $path) {
+		$::d_itunes && msg("iTunes: found path via Windows registry at: $path\n");
+		Slim::Utils::Prefs::set( 'itunes_library_xml_path', $path );
+		return $path;
+	}
+
+	# This defines the list of directories we will search for
+	# the 'iTunes Music Library.xml' file.
+	my @searchdirs = (
+		catdir($base, 'Music', 'iTunes'),
+		catdir($base, 'Documents', 'iTunes'),
+		$base,
+	);
+
+	my $audiodir = Slim::Utils::Prefs::get('audiodir');
+	if (defined $audiodir) {
+		push @searchdirs, (
+			catdir($audiodir, 'My Music', 'iTunes'),
+			catdir($audiodir, 'iTunes'),
+			$audiodir
+		);
+	}
+
+	for my $dir (@searchdirs) {
+		$path = catfile(($dir), 'iTunes Music Library.xml');
 
 		if ($path && -r $path) {
-			Slim::Utils::Prefs::set('itunes_library_xml_path',$path);
-			$::d_itunes && msg("iTunes: found path via config file at: $path\n");
+			$::d_itunes && msg("iTunes: found path via directory search at: $path\n");
+			Slim::Utils::Prefs::set( 'itunes_library_xml_path', $path );
 			return $path;
 		}
-	}		
+
+	}
 
 	$::d_itunes && msg("iTunes: unable to find iTunes Music Library.xml.\n");
 
 	return undef;
 }
 
-sub findMusicLibrary {
-	my $autolocate = Slim::Utils::Prefs::get('itunes_library_autolocate');
-	my $path = undef;
-
-	my $file = findMusicLibraryFile();
-
-	if (defined($file) && $autolocate) {
-		$::d_itunes && msg("iTunes: attempting to locate iTunes library relative to $file.\n");
-
-		my $itunesdir = dirname($file);
-		$path = catdir($itunesdir, 'iTunes Music');
-
-		if ($path && -d $path) {
-			$::d_itunes && msg("iTunes: set iTunes library relative to $file: $path\n");
-			Slim::Utils::Prefs::set('itunes_library_music_path',$path);
-			return $path;
-		}
-	}
-
-	$path = Slim::Utils::Prefs::get('itunes_library_music_path');
-
-	if ($path && -d $path) {
-		$::d_itunes && msg("iTunes: set iTunes library to itunes_library_music_path value of: $path\n");
-		return $path;
-	}
-
-	$path = Slim::Utils::Prefs::get('audiodir') || return undef;
-
-	$::d_itunes && msg("iTunes: set iTunes library to audiodir value of: $path\n");
-
-	Slim::Utils::Prefs::set('itunes_library_music_path', $path);
-
-	return $path;
-}
-
 sub isMusicLibraryFileChanged {
 
 	my $file      = findMusicLibraryFile();
@@ -481,12 +410,18 @@ sub checker {
 	if (!$firstTime && !stillScanning() && isMusicLibraryFileChanged()) {
 		startScan();
 	}
-
+	
 	# make sure we aren't doing this more than once...
 	Slim::Utils::Timers::killTimers(0, \&checker);
-
-	# Call ourselves again after 10 seconds
-	Slim::Utils::Timers::setTimer(0, (Time::HiRes::time() + 10.0), \&checker);
+	
+	my $interval = Slim::Utils::Prefs::get('itunesscaninterval') || 3600;
+	
+	# the very first time, we do want to scan right away
+	if ( $firstTime ) {
+		$interval = 10;
+	}
+	
+	Slim::Utils::Timers::setTimer(0, Time::HiRes::time() + $interval, \&checker);
 }
 
 sub startScan {
@@ -509,9 +444,6 @@ sub startScan {
 	$isScanning = 1;
 	$iTunesScanStartTime = time();
 
-	# start the checker
-	checker();
-
 	$iTunesParser = XML::Parser->new(
 		'ErrorContext'     => 2,
 		'ProtocolEncoding' => 'UTF-8',
@@ -566,7 +498,6 @@ sub doneScanning {
 	$locked = 0;
 	$opened = 0;
 
-	$iTunesLibraryPath = undef;
 	$iTunesLibraryFile = undef;
 	$lastMusicLibraryFinishTime = time();
 	$isScanning = 0;
@@ -574,19 +505,13 @@ sub doneScanning {
 	# Don't leak filehandles.
 	close(ITUNESLIBRARY);
 
-	# Set the last change time for the next go-round.
-	my $file  = findMusicLibraryFile();
-	my $mtime = (stat($file))[9];
-
-	$lastITunesMusicLibraryDate = $mtime;
-
 	setPodcasts();
 
 	if ($::d_itunes) {
 		msgf("iTunes: scan completed in %d seconds.\n", (time() - $iTunesScanStartTime));
 	}
 
-	Slim::Utils::Prefs::set('lastITunesMusicLibraryDate', $lastITunesMusicLibraryDate);
+	Slim::Utils::Prefs::set('lastITunesMusicLibraryDate', $currentITunesMusicLibraryDate);
 
 	# Take the scanner off the scheduler.
 	Slim::Utils::Scheduler::remove_task(\&scanFunction);
@@ -611,7 +536,10 @@ sub scanFunction {
 
 		resetScanState();
 
-		Slim::Utils::Prefs::set('lastITunesMusicLibraryDate', (stat($iTunesLibraryFile))[9]);
+		# Set the last change time for the next go-round.
+		my $mtime = (stat($iTunesLibraryFile))[9];
+	
+		$currentITunesMusicLibraryDate = $mtime;
 	}
 
 	if ($opened && !$locked) {
@@ -648,16 +576,10 @@ sub scanFunction {
 	# parse a little more from the stream.
 	if (defined $iTunesParserNB) {
 
-		$::d_itunes && msg("iTunes: Parsing next bit of XML..\n");
+		#$::d_itunes && msg("iTunes: Parsing next bit of XML..\n");
 
 		local $/ = '</dict>';
-		my $line;
-
-		for (my $i = 0; $i < 25; $i++) {
-			$line .= <ITUNESLIBRARY>;
-		}
-
-		$line =~ s/&#(\d*);/Slim::Web::HTTP::escape(chr($1))/ge;
+		my $line = <ITUNESLIBRARY>;
 
 		$iTunesParserNB->parse_more($line);
 
@@ -699,6 +621,17 @@ sub handleTrack {
 	if (Slim::Music::Info::isFileURL($url)) {
 
 		$file  = Slim::Utils::Misc::pathFromFileURL($url);
+		
+		# Bug 3402
+		# If the file can't be found using itunes_library_music_path,
+		# we want to fall back to the real file path from the XML file
+		if ( !-e $file ) {
+			
+			if ( Slim::Utils::Prefs::get('itunes_library_music_path') ) {
+				$url  = normalize_location( $location, 'fallback' ); 
+				$file = Slim::Utils::Misc::pathFromFileURL($url);
+			}
+		}
 
 		if ($] > 5.007 && $file && $Slim::Utils::Unicode::locale ne 'utf8') {
 
@@ -715,6 +648,19 @@ sub handleTrack {
 	# Use this for playlist verification.
 	$tracks{$id} = $url;
 
+	# skip track if Disabled in iTunes
+	if ($curTrack->{'Disabled'} && !Slim::Utils::Prefs::get('ignoredisableditunestracks')) {
+
+		$::d_itunes && msg("iTunes: deleting disabled track $url\n");
+
+		$ds->markEntryAsInvalid($url);
+
+		# Don't show these tracks in the playlists either.
+		delete $tracks{$id};
+
+		return 1;
+	}
+
 	if (Slim::Music::Info::isFileURL($url)) {
 
 		# dsully - Sun Mar 20 22:50:41 PST 2005
@@ -733,10 +679,13 @@ sub handleTrack {
 		#
 		# A value of -1 for lastITunesMusicLibraryDate
 		# means the user has pressed 'wipe db'.
+		# also check to see if that track is in the library by looking for 
+		# a lightweighttrack via objectForUrl()
 		if ($lastITunesMusicLibraryDate &&
 		    $lastITunesMusicLibraryDate != -1 &&
 		    ($ctime && $ctime < $lastITunesMusicLibraryDate) &&
-		    ($mtime && $mtime < $lastITunesMusicLibraryDate)) {
+		    ($mtime && $mtime < $lastITunesMusicLibraryDate) &&
+		    ref($ds->objectForUrl($url,0,0,1))) {
 
 			$::d_itunes && msg("iTunes: not updated, skipping: $file\n");
 
@@ -756,19 +705,6 @@ sub handleTrack {
 		}
 	}
 
-	# skip track if Disabled in iTunes
-	if ($curTrack->{'Disabled'} && !Slim::Utils::Prefs::get('ignoredisableditunestracks')) {
-
-		$::d_itunes && msg("iTunes: deleting disabled track $url\n");
-
-		$ds->markEntryAsInvalid($url);
-
-		# Don't show these tracks in the playlists either.
-		delete $tracks{$id};
-
-		return 1;
-	}
-
 	# We don't need to do all the track processing if we just want to map
 	# the ID to url, and then proceed to the playlist parsing.
 	if (Slim::Music::Import::scanPlaylistsOnly()) {
@@ -844,6 +780,7 @@ sub handleTrack {
 			'url'        => $url,
 			'attributes' => \%cacheEntry,
 			'readTags'   => 1,
+			'checkMTime' => 1,
 
 		}) || do {
 
@@ -932,7 +869,7 @@ sub handleStartElement {
 	}
 
 	# Store this value somewhere.
-	if ($element eq 'string' || $element eq 'integer') {
+	if ($element eq 'string' || $element eq 'integer' || $element eq 'date') {
 		$inValue = 1;
 	}
 }
@@ -970,9 +907,9 @@ sub handleCharElement {
 	if ($inTracks && $inValue) {
 
 		if ($] > 5.007) {
-			$item{$currentKey} = $value;
+			$item{$currentKey} .= $value;
 		} else {
-			$item{$currentKey} = Slim::Utils::Unicode::utf8toLatin1($value);
+			$item{$currentKey} .= Slim::Utils::Unicode::utf8toLatin1($value);
 		}
 
 		return;
@@ -1030,7 +967,7 @@ sub handleEndElement {
 		return;
 	}
 
-	if ($element eq 'string' || $element eq 'integer') {
+	if ($element eq 'string' || $element eq 'integer' || $element eq 'date') {
 		$inValue = 0;
 	}
 
@@ -1074,7 +1011,6 @@ sub resetScanState {
 
 	$::d_itunes && msg("iTunes: Resetting scan state.\n");
 
-	$iTunesLibraryPath = undef;
 	$inPlaylists = 0;
 	$inTracks = 0;
 
@@ -1092,16 +1028,18 @@ sub resetScanState {
 
 sub normalize_location {
 	my $location = shift;
+	my $fallback = shift;   # if set, ignore itunes_library_music_path
 	my $url;
 
 	my $stripped = strip_automounter($location);
 
 	# on non-mac or windows, we need to substitute the itunes library path for the one in the iTunes xml file
-	if (Slim::Utils::OSDetect::OS() eq 'unix') {
+	my $explicit_path = Slim::Utils::Prefs::get('itunes_library_music_path');
+	
+	if ( $explicit_path && !$fallback ) {
 
 		# find the new base location.  make sure it ends with a slash.
-		my $path = $iTunesLibraryPath || findMusicLibrary();
-		my $base = Slim::Utils::Misc::fileURLFromPath($path);
+		my $base = Slim::Utils::Misc::fileURLFromPath($explicit_path);
 
 		$url = $stripped;
 		$url =~ s/$iBase/$base/isg;
@@ -1200,17 +1138,15 @@ sub checkDefaults {
 		Slim::Utils::Prefs::set('ignoredisableditunestracks',0);
 	}
 
-	if (!Slim::Utils::Prefs::isDefined('itunes_library_music_path')) {
-		Slim::Utils::Prefs::set('itunes_library_music_path',Slim::Utils::Prefs::defaultAudioDir());
-	}
-
-	if (!Slim::Utils::Prefs::isDefined('itunes_library_autolocate')) {
-		Slim::Utils::Prefs::set('itunes_library_autolocate',1);
-	}
-
 	if (!Slim::Utils::Prefs::isDefined('lastITunesMusicLibraryDate')) {
 		Slim::Utils::Prefs::set('lastITunesMusicLibraryDate',0);
 	}
+
+	if (!Slim::Utils::Prefs::isDefined('itunes')) {
+		if (defined(findMusicLibraryFile())) {
+			Slim::Utils::Prefs::set('itunes', 1);
+		}
+	}
 }
 
 sub setupCategory {
@@ -1226,8 +1162,10 @@ sub setupCategory {
 		'Groups' => {
 
 			'Default' => {
-				'PrefOrder' => ['itunesscaninterval','ignoredisableditunestracks',
-					'itunes_library_autolocate','itunes_library_xml_path','itunes_library_music_path']
+				'PrefOrder' => ['itunesscaninterval',
+								'ignoredisableditunestracks',
+								'itunes_library_xml_path',
+								'itunes_library_music_path']
 			},
 
 			'iTunesPlaylistFormat' => {
@@ -1272,6 +1210,7 @@ sub setupCategory {
 
 			'itunes_library_xml_path' => {
 				'validate' => \&Slim::Web::Setup::validateIsFile,
+				'validateArgs' => [1],
 				'changeIntro' => string('SETUP_OK_USING'),
 				'rejectMsg' => string('SETUP_BAD_FILE'),
 				'PrefSize' => 'large',
@@ -1279,18 +1218,11 @@ sub setupCategory {
 
 			'itunes_library_music_path' => {
 				'validate' => \&Slim::Web::Setup::validateIsDir,
+				'validateArgs' => [1],
 				'changeIntro' => string('SETUP_OK_USING'),
 				'rejectMsg' => string('SETUP_BAD_DIRECTORY'),
 				'PrefSize' => 'large',
 			},
-
-			'itunes_library_autolocate' => {
-				'validate' => \&Slim::Web::Setup::validateTrueFalse,
-				'options' => {
-					'1' => string('SETUP_ITUNES_LIBRARY_AUTOLOCATE_1'),
-					'0' => string('SETUP_ITUNES_LIBRARY_AUTOLOCATE_0'),
-				},
-			},
 		}
 	);
 
