@@ -88,6 +88,12 @@ sub init {
 			my $listIndex = $client->param('listIndex');
 			my $currentItem = $items->[$listIndex];
 
+			if ($client->param('header') eq 'CREATE_MIX') {
+				#Bug 3459: short circuit for mixers
+				mixerExitHandler($client,'RIGHT');
+				return;
+			}
+
 			return unless defined($currentItem);
 
 			my ($command, $line1, $line2, $string);
@@ -159,9 +165,24 @@ sub init {
 			}
 
 			# And all the search terms for the current mode
-			push @terms, map { $_ . '=' . 
-				(ref $findCriteria->{$_} eq 'ARRAY' ? join '',@{$findCriteria->{$_}} : $findCriteria->{$_}) } 
-					(keys %$findCriteria);
+			while (my ($key, $value) = each %{$findCriteria}) {
+
+				# Bug: 3582 - pass along album.compilation as
+				# 0, and reconstitute it in parseSearchTerms.
+				if (ref($value) eq 'ARRAY' && $key eq 'album.compilation') {
+					$value = 0;
+				}
+
+				if (ref($value) eq 'ARRAY') {
+
+					push @terms, join('=', $key, (join '', @$value));
+
+				} else {
+
+					push @terms, join('=', $key, $value);
+				}
+			}
+
 			my $termlist = join '&', @terms;
 
 			# If we're dealing with a group of tracks...
@@ -241,23 +262,21 @@ sub init {
 				&{$Imports->{$mixers[0]}->{'mixer'}}($client);
 				
 			} elsif (@mixers) {
-				my $params;
-				
 				# store existing browsedb params for use later.
-				$params->{'parentParams'} = $client->modeParameterStack(-1);
-				
-				$params->{'listRef'} = \@mixers;
-				$params->{'stringExternRef'} = 1;
-				
-				$params->{'header'} = 'INSTANT_MIX';
-				$params->{'headerAddCount'} = 1;
-				$params->{'callback'} = \&mixerExitHandler;
-		
-				$params->{'overlayRef'} = sub { return (undef, Slim::Display::Display::symbol('rightarrow')) };
-		
-				$params->{'overlayRefArgs'} = '';
-				$params->{'valueRef'} = \$mixer;
-				
+				my $params = {
+
+					'parentParams'    => $client->modeParameterStack(-1),
+					'listRef'         => \@mixers,
+					'stringExternRef' => 1,
+					'header'          => 'CREATE_MIX',
+					'headerAddCount'  => 1,
+					'stringHeader'    => 1,
+					'callback'        => \&mixerExitHandler,
+					'overlayRef'      => sub { return (undef, Slim::Display::Display::symbol('rightarrow')) },
+					'overlayRefArgs'  => '',
+					'valueRef'        => \$mixer,
+				};
+
 				Slim::Buttons::Common::pushModeLeft($client, 'INPUT.List', $params);
 			
 			} else {
@@ -270,25 +289,27 @@ sub init {
 }
 
 sub mixerExitHandler {
-	my ($client,$exittype) = @_;
-	
+	my ($client, $exittype) = @_;
+
 	$exittype = uc($exittype);
-	
+
 	if ($exittype eq 'LEFT') {
+
 		Slim::Buttons::Common::popModeRight($client);
-	
+
 	} elsif ($exittype eq 'RIGHT') {
+
 		my $Imports = Slim::Music::Import::importers();
-	
+
 		if (defined $Imports->{$mixer}->{'mixer'}) {
+
 			$::d_plugins && msg("Running Mixer $mixer\n");
 			&{$Imports->{$mixer}->{'mixer'}}($client);
+
 		} else {
+
 			$client->bumpRight();
 		}
-	
-	} else {
-		return;
 	}
 }
 
@@ -472,7 +493,7 @@ sub browsedbItemName {
 		}
 	}
 
-	if ($levels[$level] eq 'track') {
+	if ( $levels[$level] =~ /(?:track|playlistTrack)/ ) {
 
 		return Slim::Music::Info::standardTitle($client, $item);
 
