@@ -1,6 +1,6 @@
 package Slim::Buttons::Common;
 
-# $Id: Common.pm 5077 2005-11-07 06:44:18Z dean $
+# $Id: Common.pm 7756 2006-06-05 21:52:24Z andy $
 
 # SlimServer Copyright (c) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -10,12 +10,14 @@ package Slim::Buttons::Common;
 use strict;
 use File::Spec::Functions qw(:ALL);
 use File::Spec::Functions qw(updir);
+use Scalar::Util qw(blessed);
+
 use Slim::Player::Client;
 use Slim::Utils::Misc;
 use Slim::Buttons::Plugins;
 use Slim::Display::Display;
 use Slim::Buttons::SqueezeNetwork;
-use Slim::Buttons::PodcastBrowser;
+use Slim::Buttons::XMLBrowser;
 
 # hash of references to functions to call when we leave a mode
 our %leaveMode = ();
@@ -70,7 +72,7 @@ sub init {
 	Slim::Buttons::BrowseTree::init();
 	Slim::Buttons::Information::init();
 	Slim::Buttons::Playlist::init();
-	Slim::Buttons::PodcastBrowser::init();
+	Slim::Buttons::XMLBrowser::init();
 	Slim::Buttons::Power::init();
 	Slim::Buttons::ScreenSaver::init();
 	Slim::Buttons::Search::init();
@@ -139,17 +141,17 @@ our %functions = (
 		if ($playlistlen == 0 || ($rate != 0 && $rate != 1)) {
 			return;
 		}
-		
+
+		my $noplay = Slim::Player::Source::playmode($client) eq 'pause';
+
+		#either starts the same song over, or the previous one, depending on whether we jumped back.
 		if (Time::HiRes::time() - Slim::Hardware::IR::lastIRTime($client) < 1.0) {  #  less than second, jump back to the previous song
-			$client->execute(["playlist", "jump", "-1"]);
+			$client->execute(["playlist", "jump", "-1", $noplay]);
 		} else {
 			# otherwise, restart this song.
-			$client->execute(["playlist", "jump", "+0"]);
-		}
-		#either starts the same song over, or the previous one, depending on whether we jumped back.
-		if (Slim::Player::Source::playmode($client) ne 'pause') {
-			$client->execute(["play"]);
+			$client->execute(["playlist", "jump", "+0", $noplay]);
 		}
+
 		$client->showBriefly($client->currentSongLines());
 	},
 	
@@ -179,27 +181,26 @@ our %functions = (
 			return;	
 		}
 		
+		my $noplay = Slim::Player::Source::playmode($client) eq 'pause';
+		#either starts the same song over, or the previous one, or the next one depending on whether/how we jumped
 
 		if ($functarg eq 'rew') { 
 			my $now = Time::HiRes::time();
 			if (Slim::Player::Source::songTime($client) < 5 || Slim::Player::Source::playmode($client) eq "stop") {
 				#jump back a song if stopped, invalid songtime, or current song has been playing less
 				#than 5 seconds (use modetime instead of now when paused)
-				$client->execute(["playlist", "jump", "-1"]);
+				$client->execute(["playlist", "jump", "-1", $noplay]);
 			} else { #restart current song
-				$client->execute(["playlist", "jump", "+0"]);
+				$client->execute(["playlist", "jump", "+0", $noplay]);
 			}
 			
 		} elsif ($functarg eq 'fwd') { # jump to next song
-			$client->execute(["playlist", "jump", "+1"]);
+			$client->execute(["playlist", "jump", "+1", $noplay]);
 		} else { #restart current song
-			$client->execute(["playlist", "jump", "+0"]);
+			$client->execute(["playlist", "jump", "+0", $noplay]);
 		}
 
-		#either starts the same song over, or the previous one, or the next one depending on whether/how we jumped
-		if (Slim::Player::Source::playmode($client) ne 'pause') {
-			$client->execute(["play"]);
-		}$client->showBriefly($client->currentSongLines());
+		$client->showBriefly($client->currentSongLines());
 	},
 	'jumpinsong' => sub {
 		my ($client,$funct,$functarg) = @_;
@@ -336,10 +337,11 @@ our %functions = (
 		if ($client->power()) {
 			$brightmode = 'powerOnBrightness';
 			
-			if ($mode eq $client->prefGet('screensaver') ||
+			if (($mode eq $client->prefGet('screensaver') ||
 					$mode eq $client->prefGet('idlesaver') || 
-						($client->prefGet('autobrightness') &&
-							Slim::Hardware::IR::lastIRTime($client) &&
+						$client->prefGet('autobrightness')) 
+						&&
+						(	Slim::Hardware::IR::lastIRTime($client) &&
 							Slim::Hardware::IR::lastIRTime($client) < Time::HiRes::time() - $client->prefGet("screensavertimeout")
 						)
 					) {
@@ -351,6 +353,7 @@ our %functions = (
 			$brightmode = 'powerOffBrightness';
 		}
 		
+		$::d_ui && msg("UI: Brightness using $brightmode during mode: $mode\n");
 		my $newBrightness;
 		if ($buttonarg eq 'toggle') {
 			$newBrightness = $client->brightness() - 1;
@@ -405,7 +408,33 @@ our %functions = (
 		my $button = shift;
 		my $buttonarg = shift;
 		my $playdisp = undef;
-		if (mode($client) ne 'PLUGIN.Favorites') {
+		
+		if ($buttonarg eq "add") {
+			my $list = $client->param('listRef');
+			if ($list) {
+			
+				my $obj = $list->[$client->param('listIndex')];
+				
+				if (blessed($obj) && $obj->can('url')) {
+					my $title;
+					if ($obj->can('title')) {
+						$title = $obj->title;
+					} else {
+						$title = Slim::Music::Info::standardTitle($client, $obj);
+					}
+	
+					if ($title) {
+						Slim::Utils::Favorites->clientAdd($client, $obj, $title);
+						$client->showBriefly($client->string('FAVORITES_ADDING'), $title);
+					}
+
+				} else {
+					use Data::Dumper;
+					$::d_favorites && msg("list item is not an object, not adding favorite\n");
+					$::d_favorites && msg(Dumper($obj));
+				}
+			}			
+		} elsif (mode($client) ne 'PLUGIN.Favorites') {
 			Slim::Buttons::Common::setMode($client, 'home');
 			Slim::Buttons::Home::jump($client, 'PLUGIN.Favorites');
 			Slim::Buttons::Common::pushModeLeft($client, 'PLUGIN.Favorites');
@@ -1108,7 +1137,10 @@ sub pushModeLeft {
 
 	pushMode($client, $setmode, $paramHashRef);
 
-	$client->pushLeft(\@oldlines, [Slim::Display::Display::curLines($client)]);
+	if (!$client->param('handledTransition')) {
+		$client->pushLeft(\@oldlines, [Slim::Display::Display::curLines($client)]);
+		$client->param('handledTransition',0);
+	}
 }
 
 sub popModeRight {
