@@ -10,6 +10,7 @@ use IO::Socket;
 use Socket qw(:crlf);
 use Scalar::Util qw(blessed);
 use URI::Escape;
+use Encode;
 #use File::Spec::Functions qw(:ALL);
 
 use Slim::Utils::Misc;
@@ -376,7 +377,7 @@ sub cli_process {
 	return if !defined $cmdRef;
 	
 	my $cmd = $cmdRef->{'_command'};
-	my $encoding = $cmdRef->{'charset'} || "utf8";
+	my $encoding = $cmdRef->{'charset'} || 'utf8';
 	
 	$::d_cli && msg("CLI: Processing command [$cmd]\n");
 	
@@ -540,19 +541,6 @@ sub cli_command_parse {
 		}
 	}
 
-	# We'd really need to know the arguments types for each command
-	#  positional or tagged (to allow ":" in a positional parameter)
-	#  string or number or flag or whatever (for utf8 handling)
-	#  client is $p0 or not
-	
-	# check $p0, is it command?
-	# .. if yes and require client assign random client
-	# .. if yes and does not require client use params
-	# .. if no, is $p1 a command?
-	# ... if yes, is $p0 a client?
-	# .... if no, error
-	# .... if yes, does it require a client?
-	
 	return \%cmdHash;
 }
 
@@ -614,7 +602,7 @@ sub cli_response_write {
 	$d_cli_vv && msg("CLI: cli_response_write()\n");
 		
 	foreach my $elem (@{$connections{$client_socket}{'response'}}) {
-		$elem = Slim::Utils::Unicode::utf8encode($elem, $encoding);
+		$elem = Encode::encode($encoding, $elem);
 		$elem = URI::Escape::uri_escape($elem);
 	}
 	
@@ -785,9 +773,6 @@ sub cli_songdata {
 		sprintf('title:%s', $track->title()),
 	);
 
-	#use Data::Dumper;
-	#print Dumper($track);
-
 	for my $tag (split //, $tags) {
 
 		if (my $method = $cliTrackMap{$tag}) {
@@ -914,10 +899,15 @@ sub cli_cmd_login {
 	# if we're not authorized yet, try to be...
 	if ($connections{$client_socket}{'auth'} == 0) {
 		if (Slim::Web::HTTP::checkAuthorization($login, $pwd)) {
-			$::d_cli && msg("CLI: Connection requires authentication, authorized!\n");
+			$::d_cli && 
+				msg("CLI: Connection requires authentication, authorized!\n");
 			$connections{$client_socket}{'auth'} = 1;
 			return 0;
 		}
+		
+		errorMsg("CLI: Connections require authentication, "
+			."wrong creditentials received. Disconnecting: " 
+			. $connections{$client_socket}{'id'} . "\n");
 		return 1;
 	}
 	return 0;
