@@ -1,6 +1,6 @@
 package Slim::Control::Command;
 
-# $Id: Command.pm 5044 2005-11-04 22:45:19Z dean $
+# $Id: Command.pm 8149 2006-06-27 11:04:26Z fred $
 #
 # SlimServer Copyright (C) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -16,6 +16,7 @@ use IO::Socket qw(:DEFAULT :crlf);
 use Scalar::Util qw(blessed);
 use Time::HiRes;
 
+use Slim::Buttons::SqueezeNetwork;
 use Slim::DataStores::Base;
 use Slim::Display::Display;
 use Slim::Music::Info;
@@ -45,6 +46,7 @@ our %searchMap = (
 
 sub execute {
 	my($client, $parrayref, $callbackf, $callbackargs) = @_;
+	
 
 	my $p0 = $parrayref->[0];
 	my $p1 = $parrayref->[1];
@@ -154,6 +156,9 @@ sub execute {
 # Y    playlist        shuffle                     <0|1|2|?|>
 # Y    playlist        repeat                      <0|1|2|?|>
 
+# SQUEEZENETWORK
+# Y   squeezenetwork   connect
+
 	my $ds   = Slim::Music::Info::getCurrentDataStore();
 	my $find = {};
 
@@ -188,8 +193,10 @@ sub execute {
 				Slim::Music::Import::cleanupDatabase(1);
 			}
 
+			# rescan should not reset importers. currently iTunes
+			# is the only scanner that defines a reset function,
+			# and that's for the wipedb case only.
 			Slim::Music::Info::clearPlaylists();
-			Slim::Music::Import::resetImporters();
 			Slim::Music::Import::startScan();
 		}
 
@@ -315,26 +322,34 @@ sub execute {
 			}
 
 		} elsif ($p0 eq "sleep") {
+		
 			if ($p1 eq "?") {
+		
 				$p1 = $client->sleepTime() - Time::HiRes::time();
-				if ($p1 < 0) {
-					$p1 = 0;
-				}
+				$p1 = 0 if ($p1 < 0);
+
 			} else {
+
 				Slim::Utils::Timers::killTimers($client, \&sleepStartFade);
 				Slim::Utils::Timers::killTimers($client, \&sleepPowerOff);
 				
 				if ($p1 != 0) {
-					my $fadeTime = 0;
-					$fadeTime = $p1 - 40 if ($p1>40);
-					my $offTime = $p1;
+					
+					my $now = Time::HiRes::time();
+					
+					my $offTime = $now + $p1;
 				
-					Slim::Utils::Timers::setTimer($client, Time::HiRes::time() + $offTime, \&sleepPowerOff);
-					Slim::Utils::Timers::setTimer($client, Time::HiRes::time() + $fadeTime, \&sleepStartFade);
+					my $fadeDuration = $offTime - $now - 1;
+					$fadeDuration = 60 if ($fadeDuration > 60); 
+
+					my $fadeTime = $offTime - $fadeDuration;
+
+					Slim::Utils::Timers::setTimer($client, $offTime, \&sleepPowerOff);
+					Slim::Utils::Timers::setTimer($client, $fadeTime, \&sleepStartFade, $fadeDuration);
 
 
-					$client->sleepTime(Time::HiRes::time() + $offTime);
-					$client->currentSleepTime($offTime / 60);
+					$client->sleepTime($offTime);
+					$client->currentSleepTime($p1 / 60);
 				} else {
 					$client->sleepTime(0);
 					$client->currentSleepTime(0);
@@ -384,10 +399,23 @@ sub execute {
 			$p1 = $client->signalStrength() || 0;
 
 		} elsif ($p0 eq "power") {
+		
+			my $newPower;
 
 			if (!defined $p1) {
 				
-				my $newPower = $client->power() ? 0 : 1;
+				$newPower = $client->power() ? 0 : 1;
+
+			} elsif ($p1 eq "?") {
+
+				$p1 = $client->power();
+
+			} else {
+
+				$newPower = $p1;
+			}
+			
+			if (defined $newPower) {
 
 				if (Slim::Player::Sync::isSynced($client)) {
 
@@ -398,28 +426,10 @@ sub execute {
 
 				# send xpl message when power toggles
 				if (Slim::Utils::Prefs::get('xplsupport')) {
-					Slim::Control::xPL::sendXplHBeatMsg($client,1);
+					Slim::Control::xPL::sendXplHBeatMsg($client, 1);
                 }
-
-			} elsif ($p1 eq "?") {
-
-				$p1 = $client->power();
-
-			} else {
-
-				if (Slim::Player::Sync::isSynced($client)) {
-
-					syncFunction($client, $p1, "power", undef);
-				}
-
-				$client->power($p1);
-
-				# send xpl message when power toggles
-				if (Slim::Utils::Prefs::get('xplsupport')) {
-					Slim::Control::xPL::sendXplHBeatMsg($client,1);
-				}
 				
-				if ($p1 eq "0") {
+				if ($newPower eq "0") {
 					# Powering off cancels sleep...
 					Slim::Utils::Timers::killTimers($client, \&sleepStartFade);
 					Slim::Utils::Timers::killTimers($client, \&sleepPowerOff);
@@ -651,6 +661,7 @@ sub execute {
 					if ($p1 =~ /^(insert|insertlist)$/) {
 
 						my $playListSize = Slim::Player::Playlist::count($client);
+						
 						my @dirItems     = ();
 
 						Slim::Utils::Scan::addToList({
@@ -660,6 +671,7 @@ sub execute {
 
 						Slim::Utils::Scan::addToList({
 							'listRef'      => Slim::Player::Playlist::playList($client),
+							'client'       => $client,
 							'url'          => $path,
 							'recursive'    => 1,
 							'callback'     => \&insert_done,
@@ -676,6 +688,7 @@ sub execute {
 
 						Slim::Utils::Scan::addToList({
 							'listRef'      => Slim::Player::Playlist::playList($client),
+							'client'       => $client,
 							'url'          => $path,
 							'recursive'    => 1,
 							'callback'     => \&load_done,
@@ -686,6 +699,7 @@ sub execute {
 								$callbackargs,
 							],
 						});
+						
 					}
 					
 					$callcallback = 0;
@@ -744,8 +758,9 @@ sub execute {
 
 				if ($playlistObj && ref($playlistObj) && $playlistObj->content_type =~ /^(?:ssp|m3u)$/) {
 
-					$jumpToIndex = Slim::Formats::Parse::readCurTrackForM3U( $client->currentPlaylist->path );
-
+					unless  (Slim::Player::Playlist::shuffle($client)) {
+						$jumpToIndex = Slim::Formats::Parse::readCurTrackForM3U( $client->currentPlaylist->path );
+					}
 					# And set a callback so that we can
 					# update CURTRACK when the song changes.
 					setExecuteCallback(\&Slim::Player::Playlist::newSongPlaylistCallback);
@@ -795,7 +810,7 @@ sub execute {
 				my $title = $p2;
 
 				my $playlistObj = $ds->updateOrCreate({
-					'url'        => Slim::Utils::Misc::fileURLFromPath(
+					'url'       => Slim::Utils::Misc::fileURLFromPath(
 						catfile(Slim::Utils::Prefs::get('playlistdir'), $title . '.m3u')
 					),
 					'attributes' => {
@@ -804,7 +819,7 @@ sub execute {
 					},
 				});
 
-				my $annotatedList;
+				my $annotatedList = [];
 
 				if (Slim::Utils::Prefs::get('saveShuffled')) {
 				
@@ -994,7 +1009,9 @@ sub execute {
 				}
 
 				my $playlistObj = $ds->updateOrCreate({
-					'url'        => "playlist://$zapped",
+					'url'        => Slim::Utils::Misc::fileURLFromPath(
+						catfile( Slim::Utils::Prefs::get('playlistdir'), $zapped . '.m3u')
+					),
 					'attributes' => {
 						'TITLE' => $zapped,
 						'CT'    => 'ssp',
@@ -1006,6 +1023,7 @@ sub execute {
 
 				$playlistObj->setTracks(\@list);
 				$playlistObj->update();
+				Slim::Player::Playlist::scheduleWriteOfPlaylist($client, $playlistObj);
 
 				$client->currentPlaylistModified(1);
 				$client->currentPlaylistChangeTime(time());
@@ -1321,6 +1339,14 @@ sub execute {
  				}
  			}
 
+		} elsif ( $p0 eq 'squeezenetwork' ) {
+			
+			if ( $p1 eq 'connect' ) {
+			
+				$client->param('squeezenetwork.connect', 1);
+				Slim::Buttons::SqueezeNetwork::connectSqueezeNetwork( $client );
+			}
+
  		} else {
 
  			#treat anything we don't know as a query so that we don't bcast it!
@@ -1353,6 +1379,7 @@ sub execute {
 			(defined $p6 ? $p6 : "") . ") (" .
 			(defined $p7 ? $p7 : "") . ")\n");
 	
+	
 	return @returnArray;
 }
 
@@ -1439,7 +1466,7 @@ sub executeCallback {
 }
 
 sub load_done {
-	my ($client, $index, $callbackf, $callbackargs) = @_;
+	my ($client, $index, $callbackf, $callbackargs, $count, $url) = @_;
 
 	# dont' keep current song on loading a playlist
 	Slim::Player::Playlist::reshuffle($client,
@@ -1450,6 +1477,14 @@ sub load_done {
 		Slim::Player::Source::jumpto($client, $index);
 	}
 
+	if ( !$count && $url ) {
+		# If the playlist was unable to load a remote URL, display an error
+		$client->showBriefly({
+			'line1' => $client->string('PROBLEM_OPENING_REMOTE_URL'),
+			'line2' => $url,
+		}, { 'duration' => 2, 'block' => 1, 'scroll' => 1, 'firstline' => 1 });
+	}
+
 	$callbackf && (&$callbackf(@$callbackargs));
 
 	Slim::Control::Command::executeCallback($client, ['playlist','load_done']);
@@ -1508,11 +1543,12 @@ sub singletonRef {
 
 sub sleepStartFade {
 	my $client = shift;
+	my $fadeDuration = shift;
 
 	$::d_command && msg("sleepStartFade()\n");
 	
 	if ($client->isPlayer()) {
-		$client->fade_volume(-60, undef, [$client]);
+		$client->fade_volume(-$fadeDuration);
 	}
 }
 
@@ -1551,6 +1587,12 @@ sub parseSearchTerms {
 
 			$find{$key} = Slim::Utils::Text::ignoreCaseArticles($value);
 
+			# Bug: 3582 - reconstitute from 0 for album.compilation.
+			if ($key eq 'album.compilation' && $value == 0) {
+
+				$find{$key} = [ { 'is' => undef }, { '=' => 0 } ];
+			}
+
 		} elsif ($term =~ /^(fieldInfo)=(\w+)$/) {
 
 			$find{$1} = $2;
@@ -1592,7 +1634,7 @@ sub parseSearchTerms {
 	} else {
 
 		# Bug 2271 - allow VA albums.
-		if ($find{'album.compilation'}) {
+		if (defined $find{'album.compilation'} && $find{'album.compilation'} == 1) {
 
 			delete $find{'artist'};
 		}
