@@ -102,7 +102,7 @@ sub init {
 					return [];
 				}
 
-				if (Slim::Utils::Prefs::get('noGenreFilter')) {
+				if (Slim::Utils::Prefs::get('noGenreFilter') && defined $findCriteria->{'genre'}) {
 
 					if (defined $findCriteria->{'album'}) {
 
@@ -322,7 +322,9 @@ sub init {
 					$findCriteria->{'contributor.role'} = $roles;
 				}
 
-				if (Slim::Utils::Prefs::get('noGenreFilter') && defined $findCriteria->{'artist'}) {
+				if (Slim::Utils::Prefs::get('noGenreFilter') && 
+					defined $findCriteria->{'genre'} &&
+					defined $findCriteria->{'artist'}) {
 
 					# Don't filter by genre - it's unneccesary and
 					# creates a intensive query. We're already at
@@ -332,7 +334,7 @@ sub init {
 
 				# Bug: 2192 - Don't filter out compilation
 				# albums at the artist level - we want to see all of them for an artist.
-				if ($findCriteria->{'artist'} && !$findCriteria->{'album.compilation'}) {
+				if ($findCriteria->{'artist'} && $findCriteria->{'album.compilation'} != 1) {
 
 					delete $findCriteria->{'album.compilation'};
 				}
@@ -549,7 +551,7 @@ sub init {
 
 				if (Slim::Utils::Prefs::get('variousArtistAutoIdentification')) {
 
-					$findCriteria->{'album.compilation'} = 0;
+					$findCriteria->{'album.compilation'} = [ { 'is' => undef }, { '=' => 0 } ];
 				}
 
 				return $ds->find({
