@@ -1,6 +1,6 @@
 package Slim::DataStores::DBI::DBIStore;
 
-# $Id: DBIStore.pm 5099 2005-11-08 21:28:22Z dsully $
+# $Id: DBIStore.pm 8102 2006-06-22 16:12:17Z dsully $
 
 # SlimServer Copyright (c) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -37,9 +37,6 @@ use Slim::Utils::Unicode;
 # Save the persistant DB cache on an interval
 my $DB_SAVE_INTERVAL = 30;
 
-# cached value of commonAlbumTitles pref
-our $common_albums;
-
 # hold the current cleanup state
 our $cleanupIds;
 our $cleanupStage;
@@ -100,10 +97,6 @@ sub new {
 	
 	$self->_commitDBTimer();
 
-	$common_albums = Slim::Utils::Prefs::get('commonAlbumTitles');
-
-	Slim::Utils::Prefs::addPrefChangeHandler('commonAlbumTitles', \&commonAlbumTitlesChanged);
-
 	return $self;
 }
 
@@ -176,7 +169,7 @@ sub contentType {
 		return wantarray ? ($ct, $self->_retrieveTrack($url)) : $ct;
 	}
 
-	my $track = $self->objectForUrl($url);
+	my $track = $self->objectForUrl($url, 0, 0, 1);
 
 	# XXX - exception should go here. Comming soon.
 	if (blessed($track) && $track->can('content_type')) {
@@ -211,7 +204,9 @@ sub objectForUrl {
 
 	my $track = $self->_retrieveTrack($url, $lightweight);
 
-	if (blessed($track) && $track->can('url') && !$create && !$lightweight) {
+	# Bug: 3456 - we often call with $create set, but we need to check the
+	# validity, and update if necessary.
+	if (blessed($track) && $track->can('url') && !$lightweight) {
 		$track = $self->_checkValidity($track);
 	}
 
@@ -358,7 +353,7 @@ sub count {
 	#
 	# But don't restrict if we have an album (this may be wrong) - 
 	# for VA albums, we want the correct count.
-	if ($field eq 'contributor' && !$findCriteria{'album'}) {
+	if ($field eq 'contributor' && !$findCriteria{'album'} && !$findCriteria{'genre'}) {
 
 		if (my $roles = $self->artistOnlyRoles) {
 
@@ -367,7 +362,7 @@ sub count {
 
 		if (Slim::Utils::Prefs::get('variousArtistAutoIdentification') && !exists $findCriteria{'album.compilation'}) {
 
-			$findCriteria{'album.compilation'} = 0;
+			$findCriteria{'album.compilation'} = [ { 'is' => undef }, { '=' => 0 } ];
 		}
 	}
 
@@ -465,6 +460,13 @@ sub newTrack {
 
 	($attributeHash, $deferredAttributes) = $self->_preCheckAttributes($url, $attributeHash, 1);
 
+	# Abort early and don't add the track if it's DRM'd
+	if ($attributeHash->{'DRM'}) {
+		
+		$::d_info && msg("newTrack: Skipping [$url] - It's DRM hampered.\n");
+		return;
+	}
+
 	# Creating the track only wants lower case values from valid columns.
 	my $columnValueHash = {};
 
@@ -556,6 +558,15 @@ sub updateOrCreate {
 	# XXX - exception should go here. Comming soon.
 	if (blessed($track) && $track->can('url')) {
 
+		# Check the timestamp & size to make sure they've not changed.
+		if (Slim::Music::Info::isFile($url) && !Slim::Music::Info::isPlaylist($url) && 
+			!$self->_hasChanged($track, $url)) {
+
+			$::d_info && msg("Track is still valid! Skipping update! $url\n");
+
+			return $track;
+		}
+
 		$::d_info && msg("Merging entry for $url\n");
 
 		# Force a re-read if requested.
@@ -600,7 +611,7 @@ sub updateOrCreate {
 		});
 	}
 
-	if ($attributeHash->{'CT'}) {
+	if ($track && $attributeHash->{'CT'}) {
 		$contentTypeCache{$url} = $attributeHash->{'CT'};
 	}
 
@@ -890,7 +901,8 @@ sub mergeVariousArtistsAlbums {
 	return 1 if $albumObj->title eq string('NO_ALBUM');
 
 	# Don't need to process something we've already marked as a compilation.
-	return 1 if $albumObj->compilation;
+	# Bug: 3322 - check for defined.
+	return 1 if defined $albumObj->compilation;
 
 	my %trackArtists      = ();
 	my $markAsCompilation = 0;
@@ -1144,6 +1156,11 @@ sub readTags {
 
 		$::d_info && !defined($attributesHash) && msg("Info: no tags found for $filepath\n");
 
+		# Return early if we have a DRM track
+		if ($attributesHash->{'DRM'}) {
+			return $attributesHash;
+		}
+
 		if (defined $attributesHash->{'TRACKNUM'}) {
 			$attributesHash->{'TRACKNUM'} = Slim::Music::Info::cleanTrackNumber($attributesHash->{'TRACKNUM'});
 		}
@@ -1177,6 +1194,12 @@ sub readTags {
 
 			$attributesHash->{'AUDIO'} = 1;
 		}
+
+		# Set some defaults for the track if the tag reader didn't pull them.
+		for my $key (qw(DRM LOSSLESS)) {
+
+			$attributesHash->{$key} ||= 0;
+		}
 	}
 
 	# Last resort
@@ -1187,6 +1210,12 @@ sub readTags {
 		$attributesHash->{'TITLE'} = Slim::Music::Info::plainTitle($file, $type);
 	}
 
+	# Bug 2996 - check for multiple DISC tags.
+	if (ref($attributesHash->{'DISC'}) eq 'ARRAY') {
+
+		$attributesHash->{'DISC'} = $attributesHash->{'DISC'}->[0];
+	}
+
 	if (-e $filepath) {
 		# cache the file size & date
 		($attributesHash->{'FS'}, $attributesHash->{'AGE'}) = (stat($filepath))[7,9];
@@ -1464,6 +1493,12 @@ sub _preCheckAttributes {
 	# Remote index.
 	$attributes->{'REMOTE'} = Slim::Music::Info::isRemoteURL($url) ? 1 : 0;
 
+	# Don't insert non-numeric YEAR fields into the database. Bug: 2610 & 3112
+	if (defined $attributes->{'YEAR'} && $attributes->{'YEAR'} !~ /^\d+$/) {
+
+		delete $attributes->{'YEAR'};
+	}
+
 	# Munge the replaygain values a little
 	for my $gainTag (qw(REPLAYGAIN_TRACK_GAIN REPLAYGAIN_TRACK_PEAK)) {
 
@@ -1473,7 +1508,28 @@ sub _preCheckAttributes {
 		$attributes->{$shortTag} = delete $attributes->{$gainTag};
 		$attributes->{$shortTag} =~ s/\s*dB//gi;
 	}
-			
+
+	# We can take an array too - from vorbis comments, so be sure to handle that.
+	my $comments = [];
+
+	if ($attributes->{'COMMENT'} && !ref($attributes->{'COMMENT'})) {
+
+		$comments = [ $attributes->{'COMMENT'} ];
+
+	} elsif (ref($attributes->{'COMMENT'}) eq 'ARRAY') {
+
+		$comments = $attributes->{'COMMENT'};
+	}
+
+	# Bug: 2605 - Get URL out of the attributes - some programs, and
+	# services such as www.allofmp3.com add it.
+	if ($attributes->{'URL'}) {
+
+		push @$comments, delete $attributes->{'URL'};
+	}
+
+	$attributes->{'COMMENT'} = $comments;
+
 	# Normalize ARTISTSORT in Contributor->add() the tag may need to be split. See bug #295
 	#
 	# Push these back until we have a Track object.
@@ -1540,9 +1596,7 @@ sub _postCheckAttributes {
 
 		# Bug 1143: The user has updated the genre tag, and is
 		# rescanning We need to remove the previous associations.
-		for my $genreObj ($track->genres) {
-			$genreObj->delete;
-		}
+		Slim::DataStores::DBI::GenreTrack->sql_fastDelete->execute($track->id);
 
 		Slim::DataStores::DBI::Genre->add($genre, $track);
 	}
@@ -1589,6 +1643,21 @@ sub _postCheckAttributes {
 	my $disc     = $attributes->{'DISC'};
 	my $discc    = $attributes->{'DISCC'};
 
+	# Make a local variable for COMPILATION, that is easier to handle
+	my $isCompilation = undef;
+
+	if (defined $attributes->{'COMPILATION'}) {
+
+		if ($attributes->{'COMPILATION'} =~ /^yes$/i || $attributes->{'COMPILATION'} == 1) {
+
+			$isCompilation = 1;
+
+		} elsif ($attributes->{'COMPILATION'} =~ /^no$/i || $attributes->{'COMPILATION'} == 0) {
+		
+			$isCompilation = 0;
+		}
+	}
+
 	# we may have an album object already..
 	my $albumObj = $track->album if !$create;
 	
@@ -1600,6 +1669,7 @@ sub _postCheckAttributes {
 			'title'       => string('NO_ALBUM'),
 			'titlesort'   => Slim::Utils::Text::ignoreCaseArticles(string('NO_ALBUM')),
 			'titlesearch' => Slim::Utils::Text::ignoreCaseArticles(string('NO_ALBUM')),
+			'compilation' => $isCompilation,
 		});
 
 		$track->album($_unknownAlbum);
@@ -1663,26 +1733,35 @@ sub _postCheckAttributes {
 			if ($checkDisc) {
 
 				$search->{'disc'} = $disc;
+
+			} elsif ($discc && $discc > 1) {
+
+				# If we're not checking discs - ie: we're in
+				# groupdiscs mode, check discc if it exists,
+				# in the case where there are multiple albums
+				# of the same name by the same artist. bug3254
+				$search->{'discc'} = $discc;
+
+			} else {
+
+				$search->{'disc'}  = undef;
+				$search->{'discc'} = undef;
 			}
 
 			# If we have a compilation bit set - use that instead
 			# of trying to match on the artist. Having the
 			# compilation bit means that this is 99% of the time a
 			# Various Artist album, so a contributor match would fail.
-			if (defined $attributes->{'COMPILATION'}) {
+			if (defined $isCompilation) {
 
-				$search->{'compilation'} = $attributes->{'COMPILATION'};
+				# in the database this is 0 or 1                                                                                            
+				$search->{'compilation'} = $isCompilation;
 
 			} else {
 
-				# Check if the album name is one of the "common album names"
-				# we've identified in prefs. If so, we require a match on
-				# both album name and primary artist name.
-				my $commonAlbumTitlesToggle = Slim::Utils::Prefs::get('commonAlbumTitlesToggle');
-
-				if ($commonAlbumTitlesToggle && (grep $album =~ m/^$_$/i, @$common_albums)) {
-
-					$search->{'contributor'} = $contributor;
+				# Try to match based on the primary contributor
+				if (blessed($contributor)) {
+					$search->{'contributor'} = $contributor->id;
 				}
 			}
 
@@ -1704,7 +1783,7 @@ sub _postCheckAttributes {
 			# the other track is not in our current directory. If
 			# so, then we need to create a new album. If not, the
 			# album object is valid.
-			if ($albumObj && $checkDisc && !$attributes->{'COMPILATION'}) {
+			if ($albumObj && $checkDisc && !defined $isCompilation) {
 
 				my %tracks     = map { $_->tracknum, $_ } $albumObj->tracks;
 				my $matchTrack = $tracks{ $track->tracknum };
@@ -1736,6 +1815,12 @@ sub _postCheckAttributes {
 				Slim::Music::Import::artwork($albumObj, $track);
 			}
 		}
+	} elsif (!$create && !$isLocal && $album) {
+		 $::d_info && msg("_postCheckAttributes: Creating album '$album'\n");
+	
+		 $albumObj = Slim::DataStores::DBI::Album->find_or_create({ 
+	 					title => $album,
+					});
 	}
 
 	if (defined($album) && blessed($albumObj) && $albumObj->can('title') && ($albumObj ne $_unknownAlbum)) {
@@ -1751,15 +1836,8 @@ sub _postCheckAttributes {
 		# And our searchable version.
 		$albumObj->titlesearch(Slim::Utils::Text::ignoreCaseArticles($album));
 
-		# Bug 2393 - Check for 'no' instead of just true or false
-		if ($attributes->{'COMPILATION'} && $attributes->{'COMPILATION'} !~ /no/i) {
-
-			$albumObj->compilation(1);
-
-		} else {
-
-			$albumObj->compilation(0);
-		}
+		# Bug 2393 - was fixed here (now obsolete due to further code rework)
+		$albumObj->compilation($isCompilation);
 
 		$albumObj->musicbrainz_id($attributes->{'MUSICBRAINZ_ALBUM_ID'});
 
@@ -1868,19 +1946,7 @@ sub _postCheckAttributes {
 	$track->update();
 
 	# Add comments if we have them:
-	# We can take an array too - from vorbis comments, so be sure to handle that.
-	my $comments = [];
-
-	if ($attributes->{'COMMENT'} && !ref($attributes->{'COMMENT'})) {
-
-		$comments = [ $attributes->{'COMMENT'} ];
-
-	} elsif (ref($attributes->{'COMMENT'}) eq 'ARRAY') {
-
-		$comments = $attributes->{'COMMENT'};
-	}
-
-	for my $comment (@$comments) {
+	for my $comment (@{$attributes->{'COMMENT'}}) {
 
 		Slim::DataStores::DBI::Comment->find_or_create({
 			'track' => $trackId,
@@ -1972,22 +2038,6 @@ sub updateCoverArt {
  	}
 }
 
-# This is a callback that is run when the user changes the common album titles
-# preference in settings.
-sub commonAlbumTitlesChanged {
-	my ($value, $key, $index) = @_;
-
-	# Add the new value, or splice it out.
-	if ($value) {
-
-		$common_albums->[$index] = $value;
-
-	} else {
-
-		splice @$common_albums, $index, 1;
-	}
-}
-
 1;
 
 __END__
