@@ -1,6 +1,6 @@
 package Slim::DataStores::DBI::DataModel;
 
-# $Id: DataModel.pm 5002 2005-11-02 20:13:03Z dsully $
+# $Id: DataModel.pm 7881 2006-06-11 15:29:31Z dsully $
 
 # SlimServer Copyright (c) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -171,14 +171,14 @@ sub db_Main {
 		if (defined $version) {
 
 			$nextversion = $class->findUpgrade($version);
-			
-			if ($nextversion && ($nextversion ne 99999)) {
+
+			if ($nextversion && ($nextversion != 99999)) {
 
 				my $upgradeFile = catdir("Upgrades", $nextversion.".sql" );
 				$::d_info && msg("Upgrading to version ".$nextversion." from version ".$version.".\n");
 				$class->executeSQLFile($upgradeFile);
 
-			} elsif ($nextversion && ($nextversion eq 99999)) {
+			} elsif ($nextversion && ($nextversion == 99999)) {
 
 				$::d_info && msg("Database schema out of date and purge required. Purging db.\n");
 				$class->executeSQLFile("dbdrop.sql");
@@ -220,8 +220,15 @@ sub findUpgrade {
 		$from == $currVersion && last;
 	}
 
+	$::d_info && msg("findUpgrade: currVersion: [$currVersion] from: [$from] to: [$to]\n");
+
 	close($versionFile);
 
+	# Handle the case where the user tried 6.5 - but then wants to downgrade. 
+	if (defined $from && $from < $currVersion) {
+		return 99999;
+	}
+
 	if ((!defined $from) || ($from != $currVersion)) {
 		$::d_info && msg ("No upgrades found for database v. ". $currVersion."\n");
 		return 0;
@@ -395,10 +402,10 @@ our %cmpFields = (
 our %sortFieldMap = (
 	'title' => ['tracks.titlesort'],
 	'genre' => ['genres.namesort'],
-	'album' => ['albums.titlesort','albums.disc','tracks.multialbumsortkey'],
+	'album' => ['albums.titlesort','albums.disc'],
 	'contributor' => ['contributors.namesort'],
 	'artist' => ['contributors.namesort'],
-	'track' => ['tracks.multialbumsortkey', 'tracks.disc','tracks.tracknum','tracks.titlesort'],
+	'track' => ['albums.titlesort','tracks.disc','tracks.tracknum','tracks.titlesort'],
 	'tracknum' => ['tracks.disc','tracks.tracknum','tracks.titlesort'],
 	'year' => ['tracks.year'],
 	'lastPlayed' => ['tracks.lastPlayed'],
@@ -590,7 +597,12 @@ sub find {
 
 					} else {
 
-						$whereHash{$searchFieldMap{$key}} = { 'in', \@values };
+						# If the user is specifying a ref value, use it directly.
+						if (ref($values[0])) {
+							$whereHash{$searchFieldMap{$key}} = \@values;
+						} else {
+							$whereHash{$searchFieldMap{$key}} = { 'in', \@values };
+						}
 					}
 
 				} else {
