@@ -20,15 +20,16 @@ use strict;
 use Audio::FLAC::Header;
 use File::Basename;
 use IO::Seekable qw(SEEK_SET);
+use MIME::Base64 qw(decode_base64);
 
 use Slim::Formats::Parse;
 use Slim::Utils::Misc;
 use Slim::Utils::Unicode;
 
 my %tagMapping = (
-	'TRACKNUMBER'	=> 'TRACKNUM',
-	'DISCNUMBER'	=> 'DISC',
-	'URL'			=> 'URLTAG',
+	'TRACKNUMBER'			=> 'TRACKNUM',
+	'DISCNUMBER'			=> 'DISC',
+	'URL'				=> 'URLTAG',
 	'musicbrainz_sortname'		=> 'ARTISTSORT',
 	'MUSICBRAINZ_ALBUMARTISTID'	=> 'MUSICBRAINZ_ALBUMARTIST_ID',
 	'MUSICBRAINZ_ALBUMID'		=> 'MUSICBRAINZ_ALBUM_ID',
@@ -38,9 +39,15 @@ my %tagMapping = (
 	'MUSICBRAINZ_SORTNAME'		=> 'MUSICBRAINZ_SORTNAME',
 	'MUSICBRAINZ_TRACKID'		=> 'MUSICBRAINZ_ID',
 	'MUSICBRAINZ_TRMID'		=> 'MUSICBRAINZ_TRM_ID',
+
+	# J.River once again.. can't these people use existing standards?
+	'REPLAY GAIN'			=> 'REPLAYGAIN_TRACK_GAIN',
+	'PEAK LEVEL'			=> 'REPLAYGAIN_TRACK_PEAK',
+	'DISC #'			=> 'DISC',
 );
 
 my @tagNames = qw(ALBUM ARTIST BAND COMPOSER CONDUCTOR DISCNUMBER TITLE TRACKNUMBER DATE);
+my $tagCache = [];
 
 # peem id (http://flac.sf.net/id.html http://peem.iconoclast.net/)
 my $PEEM = 1885693293;
@@ -52,10 +59,11 @@ my $ESCIENT_ARTWORK = 1163084622;
 # Choose between returning a standard tag
 # or parsing through an embedded cuesheet
 sub getTag {
-	my $file   = shift || "";
+	my $file   = shift || return {};
 	my $anchor = shift || "";
 
 	my $flac   = Audio::FLAC::Header->new($file) || do {
+
 		errorMsg("Couldn't open file: [$file] for reading: $!\n");
 		return {};
 	};
@@ -159,21 +167,26 @@ sub getTag {
 sub getCoverArt {
 	my $file = shift;
 
+	if ($tagCache->[0] && $tagCache->[0] eq $file && ref($tagCache->[1]) eq 'HASH') {
+
+		my $artwork = $tagCache->[1]->{'ARTWORK'};
+
+		# Invalidate the cache
+		$tagCache = [];
+
+		return $artwork;
+	}
+
 	my $flac = Audio::FLAC::Header->new($file) || do {
-		errorMsg("Couldn't open file: [$file] for reading: $!\n");
+		errorMsg("FLAC: Couldn't open file: [$file] for reading: $!\n");
 		return;
 	};
 
-	my $artwork = '';
+	my $tags = $flac->tags() || {};
 
-	if ($artwork = $flac->application($ESCIENT_ARTWORK)) {
+	addArtworkTags($flac, $tags);
 
-		if (substr($artwork, 0, 4, '') eq 'PIC1') {
-			return $artwork;
-		}
-	}
-
-	return $artwork;
+	return $tags->{'ARTWORK'};
 }
 
 # Given a file, return a hash of name value pairs,
@@ -198,6 +211,9 @@ sub getStandardTag {
 
 	doTagMapping($tags);
 	addInfoTags($flac, $tags);
+	addArtworkTags($flac, $tags);
+
+	$tagCache = [ $file, $tags ];
 
 	return $tags;
 }
@@ -249,6 +265,26 @@ sub addInfoTags {
 	$tags->{'TIME'}	    = sprintf "%.2d:%.2d", @{$tags}{'MM', 'SS'};
 }
 
+sub addArtworkTags {
+	my ($flac, $tags) = @_;
+
+	# As seen in J.River Media Center FLAC files.
+	if ($tags->{'COVERART'}) {
+
+		$tags->{'ARTWORK'} = decode_base64($tags->{'COVERART'});
+
+		delete $tags->{'COVERART'};
+
+	} elsif (my $artwork = $flac->application($ESCIENT_ARTWORK)) {
+
+		if (substr($artwork, 0, 4, '') eq 'PIC1') {
+			$tags->{'ARTWORK'} = $artwork;
+		}
+	}
+
+	return $tags;
+}
+
 sub getSubFileTags {
 	my $flac   = shift;
 	my $tracks = shift;
