@@ -1,6 +1,6 @@
 package Slim::Formats::Parse;
 
-# $Id: Parse.pm 4972 2005-11-01 00:21:47Z dsully $
+# $Id: Parse.pm 8020 2006-06-16 16:47:35Z dsully $
 
 # SlimServer Copyright (c) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -10,11 +10,13 @@ package Slim::Formats::Parse;
 use strict;
 use File::Spec::Functions qw(:ALL);
 use FileHandle;
+use HTML::Entities ();
 use IO::Socket qw(:crlf);
 use IO::String;
 use Scalar::Util qw(blessed);
 use XML::Simple;
 use URI::Escape;
+use File::Slurp;
 
 use Slim::Music::Info;
 use Slim::Utils::Misc;
@@ -70,7 +72,7 @@ sub writeList {
 	my $writer;
 
 	if (exists $playlistInfo{$type} && ($writer = $playlistInfo{$type}->[1])) {
-		return &$writer($listref, $playlistname, Slim::Utils::Misc::pathFromFileURL($fulldir));
+		return &$writer($listref, $playlistname, Slim::Utils::Misc::pathFromFileURL($fulldir), 1);
 	}
 }
 
@@ -88,19 +90,19 @@ sub getPlaylistSuffix {
 
 sub _updateMetaData {
 	my $entry = shift;
-	my $title = shift;
+	my $metadata = shift;
 
 	my $ds    = Slim::Music::Info::getCurrentDataStore();
 	my $attributes = {};
 
 	# Update title MetaData only if its not a local file with Title information already cached.
-	if ($title && Slim::Music::Info::isRemoteURL($entry)) {
+	if ($metadata && Slim::Music::Info::isRemoteURL($entry)) {
 
 		my $track = $ds->objectForUrl($entry);
 
-		if ((blessed($track) && $track->can('title') && !$track->title) || !blessed($track)) {
+		if ((blessed($track) && $track->can('title')) || !blessed($track)) {
 
-			$attributes->{'TITLE'} = $title;
+			$attributes = $metadata;
 		}
 	}
 
@@ -117,7 +119,7 @@ sub readM3U {
 	my $url    = shift;
 
 	my @items  = ();
-	my $title;
+	my ($secs, $artist, $album, $title);
 	my $foundBOM = 0;
 
 	$::d_parse && msg("parsing M3U: $url\n");
@@ -132,6 +134,9 @@ sub readM3U {
 		# strip whitespace from beginning and end
 		$entry =~ s/^\s*//; 
 		$entry =~ s/\s*$//; 
+		
+		# decode any HTML entities in-place
+		HTML::Entities::decode_entities($entry);
 
 		if (!$foundBOM) {
 
@@ -147,10 +152,17 @@ sub readM3U {
 
 		$::d_parse && msg("  entry from file: $entry\n");
 
-		if ($entry =~ /^#EXTINF:.*?,(.*)$/) {
-			$title = $1;	
+		if ($entry =~ /^#EXTINF\:(.*?),<(.*?)> - <(.*?)> - <(.*?)>/) {
+			$secs   = $1;
+			$artist = $2;
+			$album  = $3;
+			$title  = $4;
+			$::d_parse && msg("  found secs: $secs, title: $title, artist: $artist, album: $album\n");
 
-			$::d_parse && msg("  found title: $title\n");
+		} elsif ($entry =~ /^#EXTINF:(.*?),(.*)$/) {
+			$secs  = $1;
+			$title = $2;	
+			$::d_parse && msg("  found secs: $secs, title: $title\n");
 		}
 
 		next if $entry =~ /^#/;
@@ -159,13 +171,19 @@ sub readM3U {
 
 		$entry =~ s|$LF||g;
 		
+		$entry = Slim::Utils::Unicode::utf8encode_locale($entry);
 		$entry = Slim::Utils::Misc::fixPath($entry, $m3udir);
 
 		if (playlistEntryIsValid($entry, $url)) {
 
 			$::d_parse && msg("    entry: $entry\n");
 
-			push @items, _updateMetaData($entry, $title);
+			push @items, _updateMetaData( $entry, {
+				'TITLE'  => $title,
+				'ALBUM'  => $album,
+				'ARTIST' => $artist,
+				'SECS'   => ( $secs > 0 ) ? $secs : undef,
+			} );
 
 			# reset the title
 			$title = undef;
@@ -221,6 +239,8 @@ sub writeCurTrackForM3U {
 	if (-w $path) {
 
 		rename("$path.tmp", $path);
+	} else {
+		unlink("$path.tmp");
 	}
 }
 
@@ -261,7 +281,7 @@ sub readPLS {
 		$line = Slim::Utils::Unicode::utf8decode_guess($line, $enc);
 
 		if ($line =~ m|File(\d+)=(.*)|i) {
-			$urls[$1] = $2;
+			$urls[$1] = Slim::Utils::Unicode::utf8encode_locale($2);
 			next;
 		}
 
@@ -279,7 +299,7 @@ sub readPLS {
 
 		if (playlistEntryIsValid($entry, $url)) {
 
-			push @items, _updateMetaData($entry, $titles[$i]);
+			push @items, _updateMetaData($entry, {'TITLE'=>$titles[$i]});
 		}
 	}
 
@@ -295,13 +315,7 @@ sub parseCUE {
 	my $cuedir = shift;
 	my $embedded = shift || 0;
 
-	my $artist;
-	my $album;
-	my $year;
-	my $genre;
-	my $comment;
-	my $filename;
-	my $currtrack;
+	my ($artist, $album, $year, $genre, $comment, $filename, $disc, $discc, $currtrack);
 	my $tracks = {};
 
 	$::d_parse && msg("parseCUE: cuedir: [$cuedir]\n");
@@ -322,7 +336,15 @@ sub parseCUE {
 		$line =~ s/\s*$//;
 
 		if ($line =~ /^TITLE\s+\"(.*)\"/i) {
-			$album = $1;
+
+			if (defined $currtrack) {
+
+				$tracks->{$currtrack}->{'TITLE'} = $1;
+
+			} else {
+
+				$album = $1;
+			}
 
 		} elsif ($line =~ /^PERFORMER\s+\"(.*)\"/i) {
 			$artist = $1;
@@ -333,6 +355,12 @@ sub parseCUE {
 		} elsif ($line =~ /^(?:REM\s+)?GENRE\s+\"(.*)\"/i) {
 			$genre = $1;
 
+		} elsif ($line =~ /^(?:REM\s+)?DISC\s+\"(.*)\"/i) {
+			$disc = $1;
+
+		} elsif ($line =~ /^(?:REM\s+)?DISCC\s+\"(.*)\"/i) {
+			$discc = $1;
+
 		} elsif ($line =~ /^(?:REM\s+)?COMMENT\s+\"(.*)\"/i) {
 			$comment = $1;
 
@@ -346,29 +374,42 @@ sub parseCUE {
 			$filename = $1;
 			$filename = Slim::Utils::Misc::fixPath($filename, $cuedir);
 
-		} elsif ($line =~ /^\s+TRACK\s+(\d+)\s+AUDIO/i) {
-			$currtrack = int ($1);
+		} elsif ($line =~ /^\s*TRACK\s+(\d+)\s+AUDIO/i) {
+
+			$currtrack = int($1);
+
+		} elsif (defined $currtrack and $line =~ /^\s*PERFORMER\s+\"(.*)\"/i) {
 
-		} elsif (defined $currtrack and $line =~ /^\s+PERFORMER\s+\"(.*)\"/i) {
 			$tracks->{$currtrack}->{'ARTIST'} = $1;
 
+		} elsif (defined $currtrack and $line =~ /^(?:\s+REM\s+)?REPLAYGAIN_TRACK_GAIN\s+(.*)dB/i) {
+
+			$tracks->{$currtrack}->{'REPLAYGAIN_TRACK_GAIN'} = $1;
+
+		} elsif (defined $currtrack and $line =~ /^(?:\s+REM\s+)?REPLAYGAIN_TRACK_PEAK\s+(.*)/i) {
+
+			$tracks->{$currtrack}->{'REPLAYGAIN_TRACK_PEAK'} = $1;
+
 		} elsif (defined $currtrack and
-			 $line =~ /^(?:\s+REM)?\s+(TITLE|YEAR|GENRE|COMMENT|COMPOSER|CONDUCTOR|BAND|DISC|DISCC)\s+\"(.*)\"/i) {
+
+			$line =~ /^(?:\s+REM )?\s*(TITLE|YEAR|GENRE|COMMENT|COMPOSER|CONDUCTOR|BAND|DISC|DISCC)\s+\"(.*)\"/i) {
 
 			$tracks->{$currtrack}->{uc $1} = $2;
 
-		} elsif (defined $currtrack and $line =~ /^\s+INDEX\s+00\s+(\d+):(\d+):(\d+)/i) {
+		} elsif (defined $currtrack and $line =~ /^\s*INDEX\s+00\s+(\d+):(\d+):(\d+)/i) {
 
 			$tracks->{$currtrack}->{'PREGAP'} = ($1 * 60) + $2 + ($3 / 75);
 
-		} elsif (defined $currtrack and $line =~ /^\s+INDEX\s+01\s+(\d+):(\d+):(\d+)/i) {
+		} elsif (defined $currtrack and $line =~ /^\s*INDEX\s+01\s+(\d+):(\d+):(\d+)/i) {
 
 			$tracks->{$currtrack}->{'START'} = ($1 * 60) + $2 + ($3 / 75);
 
 		} elsif (defined $currtrack and $line =~ /^\s*REM\s+END\s+(\d+):(\d+):(\d+)/i) {
+
 			$tracks->{$currtrack}->{'END'} = ($1 * 60) + $2 + ($3 / 75);			
 
 		} elsif (defined $currtrack and defined $filename) {
+
 			# Each track in a cue sheet can have a different
 			# filename. See Bug 2126 &
 			# http://www.hydrogenaudio.org/forums/index.php?act=ST&f=20&t=4586
@@ -485,6 +526,16 @@ sub parseCUE {
 			$::d_parse && msg("    GENRE: " . $track->{'GENRE'} . "\n");
 		}
 
+		if (!exists $track->{'DISC'} && defined $disc) {
+			$track->{'DISC'} = $disc;
+			$::d_parse && msg("    DISC: " . $track->{'DISC'} . "\n");
+		}
+
+		if (!exists $track->{'DISCC'} && defined $discc) {
+			$track->{'DISCC'} = $discc;
+			$::d_parse && msg("    DISCC: " . $track->{'DISCC'} . "\n");
+		}
+
 		if (!exists $track->{'COMMENT'} && defined $comment) {
 			$track->{'COMMENT'} = $comment;
 			$::d_parse && msg("    COMMENT: " . $track->{'COMMENT'} . "\n");
@@ -570,6 +621,11 @@ sub readCUE {
 		}
 
 		push @items, $track->{'URI'}; #url;
+		
+		# Bug 1855: force track size metadata from basetrack into indexed track.
+		# this forces the basetrack object expansion as well, so other metadata 
+		# properly copies below
+		$track->{'SIZE'} = $basetrack->audio_size;
 
 		# our tracks won't be visible if we don't include some data from the base file
 		for my $attribute (keys %$basetrack) {
@@ -761,13 +817,14 @@ sub readWPL {
 
 			$::d_parse && msg("  entry from file: $entry\n");
 		
+			$entry = Slim::Utils::Unicode::utf8encode_locale($entry);
 			$entry = Slim::Utils::Misc::fixPath($entry, $wpldir);
 
 			if (playlistEntryIsValid($entry, $url)) {
 
 				$::d_parse && msg("    entry: $entry\n");
 
-				push @items, _updateMetaData($entry, undef);
+				push @items, _updateMetaData($entry);
 			}
 		}
 	}
@@ -904,21 +961,13 @@ sub readASX {
 						# If this seems to happen in other ways, maybe we
 						# should URL escape before continuing.
 						$href =~ s/ /%20/;
-
-						my $url = URI->new($href);
-
-						$::d_parse && msg("Checking if we can handle the url: $url\n");
 						
-						my $scheme = $url->scheme();
+						# Bug 3160 (partial)
+						# 'ref' tags refer to audio content, so we need to force
+						# the use of the MMS protocol handler by making sure the URI starts with mms
+						$href =~ s/^http/mms/;
 
-						if ($scheme =~ s/^mms(.?)/mms/) {
-							$url->scheme($scheme);
-							$href = $url->as_string();
-						}
-
-						if (exists $Slim::Player::Source::protocolHandlers{lc $scheme}) {
-
-							$::d_parse && msg("Found a handler for: $url\n");
+						if ( $href =~ /^mms/ ) {
 							$path = $href;
 							last;
 						}
@@ -931,7 +980,7 @@ sub readASX {
 
 					if (playlistEntryIsValid($path, $url)) {
 
-						push @items, _updateMetaData($path, $title);
+						push @items, _updateMetaData($path, {'TITLE'=>$title});
 					}
 				}
 			}
@@ -979,6 +1028,8 @@ sub readASX {
 sub readPodcast {
 	my $in = shift;
 
+	my $content = read_file($in);
+
 	#$::d_parse && msg("Parsing podcast...\n");
 
 	my @urls = ();
@@ -986,7 +1037,7 @@ sub readPodcast {
 	# async http request succeeded.  Parse XML
 	# forcearray to treat items as array,
 	# keyattr => [] prevents id attrs from overriding
-	my $xml = eval { XMLin($in,
+	my $xml = eval { XMLin($content,
 						   forcearray => ["item"], keyattr => []) };
 
 	if ($@) {
@@ -1016,12 +1067,14 @@ sub readPodcast {
 				if ($item->{title}) {
 					# associate a title with the url
 					# XXX calling routine beginning with "_"
-					Slim::Formats::Parse::_updateMetaData($enclosure->{url}, $item->{title});
+					Slim::Formats::Parse::_updateMetaData($enclosure->{url}, { 'TITLE' => $item->{title} });
 				}
 			}
 		}
 	}
 
+	$::d_parse && msg("parsed " . scalar(@urls) . " items in playlist\n");
+
 	# it seems like the caller of this sub should be the one to close,
 	# since they openned it.  But I'm copying other read routines
 	# which call close at the end.
