@@ -1,6 +1,6 @@
 package Slim::Music::Info;
 
-# $Id: Info.pm 4981 2005-11-01 16:17:09Z grotus $
+# $Id: Info.pm 7818 2006-06-08 18:36:39Z adrian $
 
 # SlimServer Copyright (c) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -42,9 +42,10 @@ my $artworkDir   = '';
 our $articles = undef;
 
 # Make sure that these can't grow forever.
-tie our %lastFile, 'Tie::Cache::LRU', 64;
-tie our %displayCache, 'Tie::Cache::LRU', 64;
-tie our %currentTitles, 'Tie::Cache::LRU', 64;
+tie our %lastFile,        'Tie::Cache::LRU', 64;
+tie our %displayCache,    'Tie::Cache::LRU', 64;
+tie our %currentTitles,   'Tie::Cache::LRU', 64;
+tie our %currentBitrates, 'Tie::Cache::LRU', 64;
 
 our %currentTitleCallbacks = ();
 
@@ -140,49 +141,6 @@ sub init {
 
 	# precompute the valid extensions
 	validTypeExtensions();
-
-	# use all the genres we know about...
-	MP3::Info::use_winamp_genres();
-	
-	# also get the album, performer and title sort information
-	$MP3::Info::v2_to_v1_names{'TSOA'} = 'ALBUMSORT';
-	$MP3::Info::v2_to_v1_names{'TSOP'} = 'ARTISTSORT';
-	$MP3::Info::v2_to_v1_names{'XSOP'} = 'ARTISTSORT';
-	$MP3::Info::v2_to_v1_names{'TSOT'} = 'TITLESORT';
-
-	# get composers
-	$MP3::Info::v2_to_v1_names{'TCM'} = 'COMPOSER';
-	$MP3::Info::v2_to_v1_names{'TCOM'} = 'COMPOSER';
-
-	# get band/orchestra
-	$MP3::Info::v2_to_v1_names{'TP2'} = 'BAND';
-	$MP3::Info::v2_to_v1_names{'TPE2'} = 'BAND';	
-
-	# get artwork
-	$MP3::Info::v2_to_v1_names{'PIC'} = 'PIC';
-	$MP3::Info::v2_to_v1_names{'APIC'} = 'PIC';	
-
-	# Set info
-	$MP3::Info::v2_to_v1_names{'TPA'} = 'SET';
-	$MP3::Info::v2_to_v1_names{'TPOS'} = 'SET';	
-
-	# get conductors
-	$MP3::Info::v2_to_v1_names{'TP3'} = 'CONDUCTOR';
-	$MP3::Info::v2_to_v1_names{'TPE3'} = 'CONDUCTOR';
-	
-	$MP3::Info::v2_to_v1_names{'TBP'} = 'BPM';
-	$MP3::Info::v2_to_v1_names{'TBPM'} = 'BPM';
-
-	$MP3::Info::v2_to_v1_names{'ULT'} = 'LYRICS';
-	$MP3::Info::v2_to_v1_names{'USLT'} = 'LYRICS';
-
-	# Pull the Relative Volume Adjustment tags
-	$MP3::Info::v2_to_v1_names{'RVA'}  = 'RVAD';
-	$MP3::Info::v2_to_v1_names{'RVAD'} = 'RVAD';
-	$MP3::Info::v2_to_v1_names{'RVA2'} = 'RVA2';
-
-	# iTunes writes out it's own tag denoting a compilation
-	$MP3::Info::v2_to_v1_names{'TCMP'} = 'COMPILATION';
 }
 
 sub getCurrentDataStore {
@@ -279,9 +237,10 @@ sub clearStaleCacheEntries {
 
 sub clearFormatDisplayCache {
 
-	%displayCache  = ();
-	%lastFile      = ();
-	%currentTitles = ();
+	%displayCache    = ();
+	%lastFile        = ();
+	%currentTitles   = ();
+	%currentBitrates = ();
 }
 
 # Mark an item as having been rescanned
@@ -433,15 +392,30 @@ sub setTitle {
 	});
 }
 
+sub getCurrentBitrate {
+	my $url = shift || return undef;
+
+	return $currentBitrates{$url} || undef;
+}
+
 sub setBitrate {
-	my $url = shift;
+	my $url     = shift;
 	my $bitrate = shift;
+	my $vbr     = shift || undef;
 
 	$currentDB->updateOrCreate({
 		'url'        => $url,
-		'attributes' => { 'BITRATE' => $bitrate },
+		'attributes' => { 
+			'BITRATE'   => $bitrate,
+			'VBR_SCALE' => $vbr,
+		},
 		'readTags'   => 1,
 	});
+	
+	# Cache the bitrate string so it will appear in TrackInfo
+	my $mode = $vbr ? 'VBR' : 'CBR';
+	my $str = int ( $bitrate / 1000 ) . Slim::Utils::Strings::string('KBPS') . ' ' . $mode;
+	$currentBitrates{$url} = $str;
 }
 
 sub setCurrentTitleChangeCallback {
@@ -1270,87 +1244,32 @@ sub readCoverArtTags {
 		} else {
 			$file = $fullpath;
 		}
-			
-		if (isMP3($fullpath) || isWav($fullpath) || isAIFF($fullpath)) {
-	
-			$tags = MP3::Info::get_mp3tag($file, 2, 1);
-			if (defined $tags) {
-				$::d_artwork && msg("Looking for image in ID3 2.2 tag in file $file\n");
-				# look for ID3 v2.2 picture
-				my $pic = $tags->{'PIC'};
-				if (defined($pic)) {
-					if (ref($pic) eq 'ARRAY') {
-						$pic = (@$pic)[0];
-					}					
-					my ($encoding, $format, $picturetype, $description) = unpack 'Ca3CZ*', $pic;
-					my $len = length($description) + 1 + 5;
-					if ($encoding) { $len++; } # skip extra terminating null if unicode
-					
-					if ($len < (length($pic))) {		
-						my ($data) = unpack "x$len A*", $pic;
-						
-						$::d_artwork && msg( "PIC format: $format length: " . length($pic) . "\n");
 
-						if (length($pic)) {
-							if ($format eq 'PNG') {
-									$contenttype = 'image/png';
-									$body = $data;
-							} elsif ($format eq 'JPG') {
-									$contenttype = 'image/jpeg';
-									$body = $data;
-							}
-						}
-					}
-				} else {
-					# look for ID3 v2.3 picture
-					$pic = $tags->{'APIC'};
-					if (defined($pic)) {
-						# if there are more than one pictures, just grab the first one.
-						if (ref($pic) eq 'ARRAY') {
-							$pic = (@$pic)[0];
-						}					
-						my ($encoding, $format) = unpack 'C Z*', $pic;
-						my $len = length($format) + 2;
-						if ($len < length($pic)) {
-							my ($picturetype, $description) = unpack "x$len C Z*", $pic;
-							$len += 1 + length($description) + 1;
-							if ($encoding) { $len++; } # skip extra terminating null if unicode
-							
-							my ($data) = unpack"x$len A*", $pic;
-							
-							$::d_artwork && msg( "APIC format: $format length: " . length($data) . "\n");
-	
-							if (length($data)) {
-								$contenttype = $format;
-								$body = $data;
-							}
-						}
-					}
-				}
-			}
+		$::d_artwork && msg("Looking for image in metadata blocks - file $file\n");
+
+		if (isMP3($fullpath) || isWav($fullpath) || isAIFF($fullpath)) {
+
+			loadTagFormatForType('mp3');
+
+			$body = Slim::Formats::MP3::getCoverArt($file);
 
 		} elsif (isMOV($fullpath)) {
 
-			$::d_artwork && msg("Looking for image in Movie metadata in file $file\n");
-
 			loadTagFormatForType('mov');
 
 			$body = Slim::Formats::Movie::getCoverArt($file);
 
-			$::d_artwork && $body && msg("found image in $file of length " . length($body) . " bytes \n");
-
 		} elsif (isFLAC($fullpath)) {
 
-			$::d_artwork && msg("Looking for image in FLAC metadata blocks - in file $file\n");
-
 			loadTagFormatForType('flc');
 
 			$body = Slim::Formats::FLAC::getCoverArt($file);
-
-			$::d_artwork && $body && msg("found image in $file of length " . length($body) . " bytes \n");
 		}
 		
 		if ($body) {
+
+			$::d_artwork && msg("found image in $file of length " . length($body) . " bytes \n");
+
 			# iTunes sometimes puts PNG images in and says they are jpeg
 			if ($body =~ /^\x89PNG\x0d\x0a\x1a\x0a/) {
 				$::d_info && msg( "found PNG image\n");
@@ -1586,10 +1505,19 @@ sub isFileURL {
 
 sub isHTTPURL {
 	my $url = shift;
+	
+	# We access MMS via HTTP, so it counts as an HTTP URL
+	return 1 if isMMSURL($url);
 
 	return (defined($url) && ($url =~ /^(http|icy):\/\//i));
 }
 
+sub isMMSURL {
+	my $url = shift;
+
+	return (defined($url) && ($url =~ /^mms:\/\//i));
+}
+
 sub isRemoteURL {
 	my $url = shift;
 	return (defined($url) && ($url =~ /^([a-zA-Z0-9\-]+):/) && $Slim::Player::Source::protocolHandlers{$1});
@@ -1597,9 +1525,22 @@ sub isRemoteURL {
 
 sub isPlaylistURL {
 	my $url = shift;
+	
+	# XXX: This method is pretty wrong, it says every remote URL is a playlist
+	# Bug 3484, We want rhapsody tracks to display the proper title format so they can't be
+	# seen as a playlist which forces only the title to be displayed.
+	return if $url =~ /^rhap.+wma$/;
+	
 	return (defined($url) && ($url =~ /^([a-zA-Z0-9\-]+):/) && exists($Slim::Player::Source::protocolHandlers{$1}) && !isFileURL($url));
 }
 
+sub isAudioURL {
+	# return true if url scheme (http: etc) defined as audio in types
+	my $url = shift;
+
+	return ($url =~ /^([a-z]+:)/ && defined($suffixes{$1}) && $slimTypes{$suffixes{$1}} eq 'audio');
+}
+
 sub isURL {
 	my $url = shift;
 	return (defined($url) && ($url =~ /^([a-zA-Z0-9\-]+):/) && exists($Slim::Player::Source::protocolHandlers{$1}));
@@ -1836,6 +1777,11 @@ sub typeFromPath {
 
 			$type = $suffixes{$1};
 
+		}
+		elsif ( $fullpath =~ /^(?:radioio|live365)/ ) {
+			# Force mp3 for protocol handlers
+			return 'mp3';
+			
 		} elsif (isRemoteURL($fullpath)) {
 
 			$type = typeFromSuffix($fullpath, $defaultType);
