@@ -1,6 +1,6 @@
 package Slim::Networking::Slimproto;
 
-# $Id: Slimproto.pm 5103 2005-11-08 23:33:08Z dsully $
+# $Id: Slimproto.pm 7965 2006-06-14 15:32:33Z andy $
 
 # SlimServer Copyright (c) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -40,6 +40,7 @@ our %sock2client;	# reference to client for each sonnected sock
 our %status;
 
 our %callbacks;
+our %callbacksRAWI;
 
 sub setEventCallback {
 	my $event	= shift;
@@ -47,6 +48,16 @@ sub setEventCallback {
 	$callbacks{$event} = $funcptr;
 }
 
+sub setCallbackRAWI {
+	my $callbackRef = shift;
+	$callbacksRAWI{$callbackRef} = $callbackRef;
+}
+
+sub clearCallbackRAWI {
+	my $callbackRef = shift;
+	delete $callbacksRAWI{$callbackRef};
+}
+
 sub init {
 	my $listenerport = $SLIMPROTO_PORT;
 
@@ -125,9 +136,10 @@ sub slimproto_close {
 	$::d_slimproto && msg("Slimproto connection closed\n");
 
 	# stop selecting
-	Slim::Networking::Select::addRead($clientsock, undef);
-	Slim::Networking::Select::addError($clientsock, undef);
-	Slim::Networking::Select::addWrite($clientsock, undef);
+	Slim::Networking::Select::removeRead($clientsock);
+	Slim::Networking::Select::removeError($clientsock);
+	Slim::Networking::Select::removeWrite($clientsock);
+	Slim::Networking::Select::removeWriteNoBlockQ($clientsock);
 
 	# close socket
 	$clientsock->close();
@@ -259,7 +271,9 @@ sub process_slimproto_frame {
 
 	my $len = length($data);
 
-	$::d_slimproto && msg("Got Slimproto frame, op $op, length $len, $s\n");
+	if ( $::d_slimproto && $op ne 'DBUG' ) {	# DBUG is too chatty
+		msg("Got Slimproto frame, op $op, length $len, $s\n");
+	}
 
 	if ($op eq 'HELO') {
 		my ($deviceid, $revision, @mac, $bitmapped, $reconnect, $wlan_channellist, $bytes_received_H, $bytes_received_L, $bytes_received);
@@ -351,6 +365,9 @@ sub process_slimproto_frame {
 		$sock2client{$s}=$client;
 		
 		if ($client->needsUpgrade()) {
+			# don't start playing if we're upgrading
+			$client->execute(['stop']);
+
 			# ask for an update if the player will do it automatically
 			$client->sendFrame('ureq');
 			
@@ -407,6 +424,15 @@ sub process_slimproto_frame {
 		
 		#TODO: something interesting
 
+		{
+			no strict 'refs';
+		
+			foreach my $callbackRAWI (keys %callbacksRAWI) {
+				$callbackRAWI = $callbacksRAWI{$callbackRAWI};
+				&$callbackRAWI( $client, $data);
+			}
+		}
+
 	} elsif ($op eq 'RESP') {
 		# HTTP stream headers
 		$::d_slimproto && msg("Squeezebox got HTTP response:\n$data\n");
@@ -414,7 +440,19 @@ sub process_slimproto_frame {
 			$client->directHeaders($data);
 		}
 	} elsif ($op eq 'DSCO') {
-		$::d_slimproto && msg("Squeezebox got disconnection on the data channel why: ". unpack('C', $data) . " \n");
+		# disconnection reasons
+		# TCP_CLOSE_FIN 0			/* Connection closed normally */
+		# TCP_CLOSE_LOCAL_RST 1		/* Connection reset by local host */
+		# TCP_CLOSE_REMOTE_RST 2		/* Connection reset by remote host*/
+		# TCP_CLOSE_UNREACHABLE 3		/* Connection is no longer able to work */
+		# TCP_CLOSE_LOCAL_TIMEOUT 4	/* Connection timed out */
+
+		my $reason = unpack('C', $data);
+		$::d_slimproto && msg("Squeezebox got disconnection on the data channel why: ". $reason . " \n");
+		
+		if ($reason) {
+			$client->failedDirectStream();
+		}
 		
 	} elsif ($op eq 'BODY') {
 		$::d_slimproto && msg("Squeezebox got body response\n");
@@ -450,6 +488,7 @@ sub process_slimproto_frame {
 		#	STMl - FULL		// triggers start of synced playback
 		#	STMd - DECODE_READY	// decoder has no more data
 		#	STMs - TRACK_STARTED	// a new track started playing
+		#	STMn - NOT_SUPPORTED	// decoder does not support the track format
 	
 		my ($fullnessA, $fullnessB);
 		
@@ -532,7 +571,7 @@ sub process_slimproto_frame {
 	} elsif ($op eq 'ANIC') {
 		$client->endAnimation(0.5);
 	} elsif ($op eq 'META') {
-		$::d_directstream && msg("metadata (len: $len): $data\n");
+		$::d_directstream && msg("metadata (len: $len)\n");
 		if ($client->can('directMetadata')) {
 			$client->directMetadata($data);
 		}
@@ -547,6 +586,12 @@ sub process_slimproto_frame {
 			$client->unblock();
 			$client->upgradeFirmware();
 		}
+	} elsif ($op eq 'DBUG') {
+		
+		$::d_firmware && msgf("[%s] %s\n",
+			$client->id,
+			$data,
+		);
 		
 	} else {
 		$::d_slimproto && msg("Unknown slimproto op: $op\n");
@@ -567,6 +612,11 @@ sub signalStrength {
 
 sub fullness {
 	my $client = shift;
+	my $value  = shift;
+	
+	if ( defined $value ) {
+		return $status{$client}->{'fullness'} = $value;
+	}
 	
 	return $status{$client}->{'fullness'};
 }
