@@ -1,4 +1,4 @@
-# $Id: Client.pm 4980 2005-11-01 05:21:36Z kdf $
+# $Id: Client.pm 7912 2006-06-12 16:44:49Z andy $
 
 # SlimServer Copyright (c) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -716,6 +716,7 @@ sub new {
 	$client->[103] = 0; # lastVisMode
 	$client->[104] = 0; # animateState
 	$client->[105] = 0; # scrollState
+	$client->[106] = undef; # lastSong (last URL played in this play session - a play session ends when the player is stopped or a track is skipped)
 
 	$::d_protocol && msg("New client connected: $id\n");
 	$client->lastirtime(0);
@@ -855,8 +856,14 @@ sub getClient {
 	my $id = shift;
 	my $ret = $clientHash{$id};
 
-	# Try a brute for match for the client.
-	if (!defined($ret)) {
+	# Try a brute force match for the client.
+	if (!defined $ret) {
+	
+		# For some reason, if the line below is commented, each(%clientHash)
+		# returns nothing every other attempt. "Touching" the hash in some
+		# other manner than "each" seems to cure the problem...
+		my $ignored = scalar(keys %clientHash);
+		
 		while (my ($key, $value) = each(%clientHash)) {
 			return $value if (ipport($value) eq $id);
 			return $value if (ip($value) eq $id);
@@ -871,6 +878,12 @@ sub forgetClient {
 	my $client = shift;
 	
 	if ($client) {
+
+		# Bug 3115, temporarily unsync player when disconnecting
+		if ( Slim::Player::Sync::isSynced($client) ) {
+			Slim::Player::Sync::unsync( $client, 'temp' );
+		}
+
 		Slim::Web::HTTP::forgetClient($client);
 		Slim::Utils::Timers::forgetClient($client);
 		delete $clientHash{$client->id()};
@@ -1149,6 +1162,7 @@ sub scrollBottom {}
 sub parseLines {}
 sub playingModeOptions {}
 sub block{}
+sub symbols{}
 sub unblock{}
 
 sub pause {
@@ -1845,7 +1859,8 @@ sub currentPlaylistRender {
 }
 
 sub currentPlaylistChangeTime {
-	my $r = shift;
+	# This needs to be the same for all synced clients
+	my $r = Slim::Player::Sync::masterOrSelf(shift);
 	@_ ? ($r->[96] = shift) : $r->[96];
 }
 
@@ -1889,5 +1904,9 @@ sub scrollState {
 	@_ ? ($r->[105] = shift) : $r->[105];
 }    
 
+sub lastSong {
+	my $r = shift;
+	@_ ? ($r->[106] = shift) : $r->[106];
+}    
 
 1;
