@@ -109,6 +109,23 @@ sub repeat {
 
 	if (defined($repeat)) {
 		$client->prefSet("repeat", $repeat);
+
+		# Check the buffers for the client and reset based on repeat change
+		foreach my $everyclient ($client, Slim::Player::Sync::syncedWith($client)) {
+			
+			if ($everyclient->playmode() =~ /playout/) {
+				
+				if ($repeat) {
+					
+					# changing to repeat all or one, set to continue playback
+					$everyclient->playmode('playout-play');
+				} else {
+					
+					# repeat off, set to stop at end of track
+					$everyclient->playmode('playout-stop');
+				}
+			}
+		}
 	}
 	
 	return $client->prefGet("repeat");
@@ -386,7 +403,7 @@ sub fischer_yates_shuffle {
 #reshuffle - every time the playlist is modified, the shufflelist should be updated
 #		We also invalidate the htmlplaylist at this point
 sub reshuffle {
-	my $client = shift;
+	my $client = Slim::Player::Sync::masterOrSelf(shift);
 	my $dontpreservecurrsong = shift;
   
 	my $songcount = count($client);
@@ -409,6 +426,11 @@ sub reshuffle {
 	} elsif ($realsong > $songcount) {
 		$realsong = $songcount;
 	}
+	
+	$::d_playlist && msgf("Reshuffling, current song index: %d, preserve song? %s\n",
+		$realsong,
+		( $dontpreservecurrsong ) ? 'no' : 'yes',
+	);
 
 	my @realqueue;
 	my $song;
@@ -425,19 +447,23 @@ sub reshuffle {
 
 		fischer_yates_shuffle($listRef);
 
-		for (my $i = 0; $i < $songcount; $i++) {
+		# If we're preserving the current song
+		# this places it at the top of the playlist
+		if ( $realsong > -1 ) {
+			for (my $i = 0; $i < $songcount; $i++) {
 
-			if ($listRef->[$i] == $realsong) {
+				if ($listRef->[$i] == $realsong) {
 
-				if (shuffle($client)) {
+					if (shuffle($client)) {
+					
+						my $temp = $listRef->[$i];
+						$listRef->[$i] = $listRef->[0];
+						$listRef->[0] = $temp;
+						$i = 0;
+					}
 
-					my $temp = $listRef->[$i];
-					$listRef->[$i] = $listRef->[0];
-					$listRef->[0] = $temp;
-					$i = 0;
+					last;
 				}
-
-				last;
 			}
 		}
 
@@ -581,7 +607,13 @@ sub scheduleWriteOfPlaylist {
 				# This can happen if the user removes the
 				# playlist - because this is a closure, we get
 				# a bogus object back)
-				unless ($playlistObj->can('tracks')) {
+				if (!$playlistObj->can('tracks')) {
+					return 0;
+				}
+				
+				if ($playlistObj->title eq $client->string('UNTITLED')) {
+				
+					$::d_playlist && msg("Not writing out untitled playlist.\n");
 					return 0;
 				}
 
@@ -623,11 +655,11 @@ sub newSongPlaylistCallback {
 		$playlist = $client->currentPlaylist;
 	}
 
-	return if Slim::Music::Info::isRemoteURL($playlist);
+	return if Slim::Music::Info::isRemoteURL($playlist) || Slim::Player::Playlist::shuffle($client);
 
 	Slim::Formats::Parse::writeCurTrackForM3U(
 		$playlist,
-		Slim::Player::Source::playingSongIndex($client)
+		$params->[0] eq 'reset' ? 0 : Slim::Player::Source::playingSongIndex($client)
 	);
 }
 
