@@ -60,7 +60,7 @@ sub canDirectStream {
 	my $handler = Slim::Player::Source::protocolHandlerForURL($url);
 
 	if ($handler && $handler->can("canDirectStream") && !$handler->isa("Slim::Player::Protocols::MMS")) {
-		return $handler->canDirectStream($url);
+		return $handler->canDirectStream($client, $url);
 	}
 	
 	return undef;
