@@ -1,6 +1,6 @@
 package Slim::Player::Source;
 
-# $Id: Source.pm 5149 2005-11-11 22:38:23Z dean $
+# $Id: Source.pm 8113 2006-06-23 19:50:23Z andy $
 
 # SlimServer Copyright (C) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -383,6 +383,21 @@ sub playmode {
 		return _returnPlayMode($client);
 	}
 
+	my $currentSong = Slim::Player::Playlist::song($client, streamingSongIndex($client));
+	# Some protocol handlers don't allow pausing of active streams.
+	# We check if that's the case before continuing.
+	if ($newmode eq "pause" && defined($currentSong)) {
+		my $handler = Slim::Player::Source::protocolHandlerForURL($currentSong);
+		if ($handler && 
+			$handler->can("canDoAction") &&
+			!$handler->canDoAction($client, 
+								   $currentSong, 
+								   'pause')) {
+			$::d_source && msg("Protocol handler doesn't allow pausing.  Let's try stopping.\n");
+			return playmode($client, "stop", $seekoffset);
+		}
+	}
+
 	if ($newmode eq "play" && $prevmode eq "pause") {
 		$newmode = "resume";
 	}
@@ -432,7 +447,20 @@ sub playmode {
 		# when you resume, you go back to play mode
 		if (($newmode eq "resume") ||($newmode eq "resumenow")) {
 
-			$everyclient->playmode("play");
+			$everyclient->resume();
+			
+			my $song = Slim::Player::Source::streamingSong($client);
+			my $songLengthInBytes = $song->{totalbytes};
+			my $pos		      = $client->songBytes() || 0; 
+			
+			$::d_source && msg("resume: (pos: " . $client->songBytes() . ", totalbytes: " . $song->{totalbytes} . ")\n");
+			
+			# if song pos matches totalbytes, previous playmode was playout-play
+			if ($songLengthInBytes - $pos == 0) {
+				$everyclient->playmode("playout-play");
+			} else {
+				$everyclient->playmode("play");
+			}
 			
 		} elsif ($newmode eq "pausenow") {
 
@@ -477,9 +505,16 @@ sub playmode {
 			}
 			$everyclient->streamBytes(0);
 			
-			my $currentSong = Slim::Player::Playlist::song($client, streamingSongIndex($client));
+			my $paused = ( Slim::Player::Sync::isSynced($everyclient) ) ? 1 : 0;
 			
-			$everyclient->play(Slim::Player::Sync::isSynced($everyclient), $master->streamformat(), $currentSong, (defined($seekoffset) && $seekoffset > 0), shouldLoop($master), replayGain($master));
+			$everyclient->play(
+				$paused,
+				$master->streamformat(),
+				$currentSong,
+				(defined($seekoffset) && $seekoffset > 0),
+				shouldLoop($master),
+				replayGain($master)
+			);
 
 		} elsif ($newmode eq "pause") {
 
@@ -562,10 +597,10 @@ sub decoderUnderrun {
 
 sub underrun {
 	my $client = shift || return;
-	
+	my $error = shift;
 	$client->readytosync(-1);
 	
-	$::d_source && msg($client->id() . ": Underrun while this mode: " . $client->playmode() . "\n");
+	$::d_source && msg($client->id() . ": Underrun while this mode: " . $client->playmode() . "with error $error\n");
 
 	# if we're synced, then we tell the player to stop and then let resync restart us.
 
@@ -575,7 +610,7 @@ sub underrun {
 		}
 	} elsif ($client->playmode eq 'playout-play') {
 
-		skipahead($client);
+		skipahead($client, $error);
 
 	} elsif (($client->playmode eq 'playout-stop')) {
 
@@ -588,6 +623,7 @@ sub underrun {
 
 sub skipahead {
 	my $client = shift;
+	my $error = shift;
 
 	if (!$client->reportsTrackStart() || 
 		Slim::Player::Sync::isSynced($client)) {
@@ -596,10 +632,12 @@ sub skipahead {
 	}
 
 	$::d_source && msg("**skipahead: opening next song\n");
-	gotoNext($client, 0);
+	my $succeeded = gotoNext($client, 0, $error);
 
-	$::d_source && msg("**skipahead: restarting\n");
-	playmode($client, 'play');
+	if ($succeeded) {
+		$::d_source && msg("**skipahead: restarting\n");
+		playmode($client, 'play');
+	}
 } 
 
 sub nextChunk {
@@ -758,8 +796,10 @@ sub gototime {
 		$::d_source && msg("gototime: restarting playback\n");
 
 		$everybuddy->readytosync(0);
+		
+		my $paused = ( Slim::Player::Sync::isSynced($client) ) ? 1 : 0;
 
-		$everybuddy->play(Slim::Player::Sync::isSynced($client), $client->streamformat(),
+		$everybuddy->play($paused, $client->streamformat(),
 				Slim::Player::Playlist::song($client));
 
 		$everybuddy->playmode("play");
@@ -768,7 +808,8 @@ sub gototime {
 
 # jumpto - set the current song to a given offset
 sub jumpto {
-	my ($client, $offset, $noplay) = @_;
+	my $client = Slim::Player::Sync::masterOrSelf(shift);
+	my ($offset, $noplay) = @_;
 
 	my ($songcount) = Slim::Player::Playlist::count($client);
 
@@ -776,6 +817,18 @@ sub jumpto {
 		return;
 	}
 
+	if ($offset =~ /([\+\-])(\d+)/ &&
+		($1 eq '-' ||
+		 $2 eq '0')) {
+		my $currentSong = Slim::Player::Playlist::song($client, streamingSongIndex($client));
+		my $handler = Slim::Player::Source::protocolHandlerForURL($currentSong);
+		if ($handler && 
+			$handler->can("canDoAction") &&
+			!$handler->canDoAction($client, $currentSong, 'rew')) {
+			return;
+		}
+	}
+
 	playmode($client,"stop");
 
 	if ($songcount != 1) {
@@ -806,7 +859,7 @@ sub jumpto {
 
 	$client->currentPlaylistChangeTime(time());
 	Slim::Buttons::Common::syncPeriodicUpdates($client, Time::HiRes::time() + 0.1);
-	
+
 	playmode($client,"play") unless $noplay;
 }
 
@@ -819,11 +872,17 @@ sub jumpto {
 #                  0 if it stopped at the end of the list or failed to open the current song 
 # note: Only call this with a master or solo client
 sub gotoNext {
-	$::d_source && msg("opening next song...\n"); 
 	my $client = shift;
 	my $open = shift;
+	my $error = shift;
+	$::d_source && msg("opening next song open: $open error: $error...\n"); 
+	
 	my $result = 1;
 	
+	if ($error) {
+		$result = 0;
+	}
+	
 	my $oldstreamformat = $client->streamformat();
 	my $nextsong;
 
@@ -832,42 +891,70 @@ sub gotoNext {
 	# we're at the end of a song, let's figure out which song to open up.
 	# if we can't open a song, skip over it until we hit the end or a good song...
 	do {
-	
-		if (Slim::Player::Playlist::repeat($client) == 2  && $result) {
 
-			$nextsong = nextsong($client);
+		my $currsong = streamingSongIndex($client);
+		my $songcount = Slim::Player::Playlist::count($client);
+
+		if ($songcount == 0) {
+
+			$nextsong = 0;
 
 		} elsif (Slim::Player::Playlist::repeat($client) == 1 && $result) {
 
 			# play the same song again
 
+		# open the next song in the list if we have playlist repeat on OR we're not at the last song 
+		} elsif ( 	
+					($currsong != ($songcount - 1)) ||
+					(Slim::Player::Playlist::repeat($client) == 2  && $result)
+					 
+			) {			
+
+			my $direction = 1;
+			
+			if ($client->rate() < 0) { $direction = -1; }
+			 
+			$nextsong = streamingSongIndex($client) + $direction;
+		
+			if ($nextsong >= Slim::Player::Playlist::count($client)) {
+		
+				# play the next song and start over if necessary
+				if (Slim::Player::Playlist::shuffle($client) && 
+					Slim::Player::Playlist::repeat($client) == 2 &&
+					Slim::Utils::Prefs::get('reshuffleOnRepeat')) {
+					
+						Slim::Player::Playlist::reshuffle($client, 1);
+				}
+		
+				$nextsong = 0;
+			}
+			
+			if ($nextsong < 0) {
+				$nextsong = Slim::Player::Playlist::count($client) - 1;
+			}
+			
+			$::d_source && msg("the next song is number $nextsong, was $currsong\n");
+
 		} else {
 
-			# stop at the end of the list or when list is empty
-			if (streamingSongIndex($client) == (Slim::Player::Playlist::count($client) - 1) ||
-				!Slim::Player::Playlist::count($client)) {
+			$::d_source && msg("we've hit the end of the list, stopping\n");
 
-				$nextsong = 0;
+			$nextsong = 0;
 
-				playmode($client, $result ? 'playout-stop' : 'stop');
+			playmode($client, $result ? 'playout-stop' : 'stop');
 
-				# We're done streaming the song, so drop the streaming
-				# connection to the client.
-				dropStreamingConnection($client);
+			# We're done streaming the song, so drop the streaming
+			# connection to the client.
+			dropStreamingConnection($client);
 
-				# send xpl to tell them we're done
-				if (Slim::Utils::Prefs::get('xplsupport')) {
-					Slim::Control::xPL::sendXplStatusMsg($client,"end of list");
-				}
-
-				$client->update();
-
-				return 0;
-
-			} else {
-
-				$nextsong = nextsong($client);
+			# send xpl to tell them we're done
+			if (Slim::Utils::Prefs::get('xplsupport')) {
+				Slim::Control::xPL::sendXplStatusMsg($client,"end of list");
 			}
+
+			$client->update();
+
+			return 0;
 		}
 
 		my ($command, $type, $newstreamformat) = getConvertCommand(
@@ -901,6 +988,15 @@ sub gotoNext {
 				"opening next song (old format: $oldstreamformat, " .
 				"new: $newstreamformat) current playmode: " . playmode($client) . "\n"
 			);
+			
+			# Fail if the next song index is the same as the current song index,
+			# as this can cause an infinite loop
+			# The only case where this is OK is Rhapsody Radio
+			if ( $nextsong == streamingSongIndex($client) ) {
+				if ( $client->lastSong !~ /\.rhr$/ ) {	
+					return 0;
+				}
+			}
 
 			streamingSongIndex($client, $nextsong);
 			return 1 if !$open;
@@ -1034,7 +1130,7 @@ sub replayGain {
 	}
 
 	# Mode 3 is determine dynamically whether to use album or track
-	if ($album->replay_gain() && (trackAlbumMatch($client, -1) || trackAlbumMatch($client, 1))) {
+	if (defined $album->replay_gain() && (trackAlbumMatch($client, -1) || trackAlbumMatch($client, 1))) {
 
 		return $album->replay_gain();
 	}
@@ -1132,7 +1228,7 @@ sub streamingSong {
 }
 
 sub playingSongIndex {
-	my $client = shift;
+	my $client = Slim::Player::Sync::masterOrSelf(shift);
 
 	my $song = playingSong($client);
 	return 0 if !defined($song);
@@ -1164,6 +1260,10 @@ sub resetSongQueue {
 	$#{$queue} = -1;
 	push @$queue, $playingsong;
 	$::d_source && msg("Song queue is now " . join(',', map { $_->{index} } @$queue) . "\n");
+
+	# update CURTRACK of a known playlist back to start
+	my $params = ['reset'];
+	Slim::Player::Playlist::newSongPlaylistCallback($client, $params);
 }
 
 sub markStreamingTrackAsPlayed {
@@ -1176,7 +1276,7 @@ sub markStreamingTrackAsPlayed {
 }
 
 sub trackStartEvent {
-	my $client = shift || return;
+	my $client = Slim::Player::Sync::masterOrSelf(shift) || return;
 
 	$::d_source && msg("Got a track starting event\n");
 	my $queue = $client->currentsongqueue();
@@ -1201,43 +1301,6 @@ sub trackStartEvent {
 	$::d_source && msg("Song queue is now " . join(',', map { $_->{index} } @$queue) . "\n");
 }
 
-# nextsong is for figuring out what the next song will be.
-sub nextsong {
-	my $client = shift;
-
-	my $nextsong;
-	my $currsong = streamingSongIndex($client);
-	
-	return 0 if (Slim::Player::Playlist::count($client) == 0);
-	
-	my $direction = 1;
-	
-	if ($client->rate() < 0) { $direction = -1; }
-	 
-	$nextsong = streamingSongIndex($client) + $direction;
-
-	if ($nextsong >= Slim::Player::Playlist::count($client)) {
-
-		# play the next song and start over if necessary
-		if (Slim::Player::Playlist::shuffle($client) && 
-			Slim::Player::Playlist::repeat($client) == 2 &&
-			Slim::Utils::Prefs::get('reshuffleOnRepeat')) {
-			
-				Slim::Player::Playlist::reshuffle($client, 1);
-		}
-
-		$nextsong = 0;
-	}
-	
-	if ($nextsong < 0) {
-		$nextsong = Slim::Player::Playlist::count($client) - 1;
-	}
-	
-	$::d_source && msg("the next song is number $nextsong, was $currsong\n");
-	
-	return $nextsong;
-}
-
 sub flushStreamingSong {
 	my $client = shift;
 	
@@ -1312,13 +1375,20 @@ sub openSong {
 
 	my $objOrUrl = Slim::Player::Playlist::song($client, streamingSongIndex($client)) || return undef;
 
-	my $track    = blessed($objOrUrl) && $objOrUrl->can('url') ? $objOrUrl : $ds->objectForUrl($objOrUrl, 1, 1);
+	# Bug: 3390 - reload the track if it's changed.
+	my $url      = blessed($objOrUrl) && $objOrUrl->can('url') ? $objOrUrl->url : $objOrUrl;
+	my $track    = $ds->objectForUrl($url, 0, 1);
 
 	if (!blessed($track) || !$track->can('url')) {
 
-		errorMsg("openSong: Error finding an object for [$objOrUrl]!\n");
+		$track = $ds->objectForUrl($url, 1, 1);
 
-		return undef;
+		if (!blessed($track) || !$track->can('url')) {
+
+			errorMsg("openSong: Error finding an object for [$objOrUrl]!\n");
+
+			return undef;
+		}
 	}
 
 	my $fullpath = $track->url;
@@ -1329,11 +1399,6 @@ sub openSong {
 	# parse the filetype
 	if (Slim::Music::Info::isRemoteURL($fullpath)) {
 
-		my $line1 = $client->string('CONNECTING_FOR');
-		my $line2 = Slim::Music::Info::standardTitle($client, $track);
-
-		$client->showBriefly($line1, $line2, undef, 1);
-
 		if ($client->canDirectStream($fullpath)) {
 
 			$directStream = 1;
@@ -1343,104 +1408,84 @@ sub openSong {
 		if (!$directStream) {
 
 			$::d_source && msg("URL is remote : $fullpath\n");
-
-			# we don't get the content type until after the stream is opened
+			
 			my $sock = openRemoteStream($track, $client);
-	
+			
 			if ($sock) {
 	
+				# The stream's content-type was determined in Scan::addToList
+				my $contentType = $track->content_type;
+
 				# if it's an mp3 stream, then let's stream it.
 				if (Slim::Music::Info::isSong($track)) {
 	
-					$::d_source && msg("remoteURL is a song : $fullpath\n");
+					$::d_source && msg("openSong: remoteURL is a song : $fullpath\n");
 	
-					if ($sock->opened() &&
-						!defined(Slim::Utils::Misc::blocking($sock, 0))) {
-						$::d_source && msg("Cannot set remote stream nonblocking\n");
+					if ($sock->opened() && !defined(Slim::Utils::Misc::blocking($sock, 0))) {
+
+						$::d_source && msg("openSong: Cannot set remote stream nonblocking for url: [$fullpath]\n");
+
 						errorOpening($client);
+
 						return undef;
 					}
 	
 					my ($command, $type, $format) = getConvertCommand($client, $track);
 
-					$::d_source && msg("remoteURL command $command type $type format $format\n");
-					$::d_source && msgf("remoteURL stream format : %s\n", Slim::Music::Info::contentType($track));
+					if (!defined $command) {
+
+						$::d_source && msg("openSong: Couldn't create command line for $type playback for [$fullpath]\n");
+
+						errorOpening($client);
+
+						return undef;
+					}
+
+					if ($::d_source) { 
+						msg("openSong: remoteURL command $command type $type format $format\n");
+						msg("openSong: remoteURL stream format : $contentType\n");
+					}
 
 					$client->streamformat($format);
 	
-					unless (defined($command)) {
-						$::d_source && msg("Couldn't create command line for $type playback for $fullpath\n");
-						errorOpening($client);
-						
-						return undef;
-					}
-	
-					my $duration  = $track->durationSeconds();
-	
-					if (defined($duration)) {
-						$song->{duration} = $duration;
-					}
-	
 					# this case is when we play the file through as-is
 					if ($command eq '-') {
+
 						$client->audioFilehandle($sock);
 						$client->audioFilehandleIsSocket(1);
-					}
-					else {
+
+					} else {
+
 						my $maxRate = Slim::Utils::Prefs::maxRate($client);
 						my $quality = $client->prefGet('lameQuality');
 						
 						$command = tokenizeConvertCommand($command, $type, '-', $fullpath, 0 , $maxRate, 1, $quality);
-						$::d_source && msg("tokenized command $command\n");
+
+						if (!defined($command)) {
+
+							$::d_source && msg("openSong: Couldn't create command line for $type playback for [$fullpath]\n");
+
+							errorOpening($client);
+
+							return undef;
+						}
+
+						$::d_source && msg("openSong: Tokenized command $command\n");
+
 						my $pipeline = Slim::Player::Pipeline->new($sock, $command);
+
 						if (!defined($pipeline)) {
-							$::d_source && msg("Error creating conversion pipeline\n");
+
+							$::d_source && msg("openSong: Error creating conversion pipeline for: [$fullpath]\n");
 							errorOpening($client);
+
 							return undef;
 						}
 		
-						my ($command, $type, $format) = getConvertCommand($client, $track);
-
-						$::d_source && msg("remoteURL command $command type $type format $format\n");
-						$::d_source && msgf("remoteURL stream format : %s\n", Slim::Music::Info::contentType($track));
-
-						$client->streamformat($format);
-		
-						unless (defined($command)) {
-							$::d_source && msg("Couldn't create command line for $type playback for $fullpath\n");
-							errorOpening($client);
-							
-							return undef;
-						}
-		
-						my $duration  = $track->durationSeconds();
-		
-						if (defined($duration)) {
-							$song->{duration} = $duration;
-						}
-		
-						# this case is when we play the file through as-is
-						if ($command eq '-') {
-							$client->audioFilehandle($sock);
-							$client->audioFilehandleIsSocket(1);
-						}
-						else {
-							my $maxRate = Slim::Utils::Prefs::maxRate($client);
-							my $quality = $client->prefGet('lameQuality');
-							
-							$command = tokenizeConvertCommand($command, $type, '-', $fullpath, 0 , $maxRate, 1, $quality);
-							$::d_source && msg("tokenized command $command\n");
-							my $pipeline = Slim::Player::Pipeline->new($sock, $command);
-							if (!defined($pipeline)) {
-								$::d_source && msg("Error creating conversion pipeline\n");
-								errorOpening($client);
-								return undef;
-							}
-							$client->audioFilehandle($pipeline);
-							$client->audioFilehandleIsSocket(2);
-						}
-
+						$client->audioFilehandle($pipeline);
+						$client->audioFilehandleIsSocket(2);
 					}
+
 					$client->remoteStreamStartTime(Time::HiRes::time());
 					$client->pauseTime(0);
 
@@ -1482,7 +1527,7 @@ sub openSong {
 	
 				$::d_source && msg("Remote stream failed to open, showing message.\n");
 				$client->audioFilehandle(undef);
-				
+
 				my $line1 = $client->string('PROBLEM_CONNECTING');
 				my $line2 = Slim::Music::Info::standardTitle($client, $track);
 	
@@ -1528,7 +1573,7 @@ sub openSong {
 				return undef;
 			}
 
-			if (!$size || !$duration) {
+			if (!$size && !$duration) {
 
 				$::d_source && msg("openSong: not bothering opening file with zero size or duration\n");
 				errorOpening($client);
@@ -1757,7 +1802,7 @@ sub underMax {
 	if (defined($type) && $type eq 'mp3') {
 
 		my $ds    = Slim::Music::Info::getCurrentDataStore();
-		my $track = $ds->objectForUrl($fullpath);
+		my $track = $ds->objectForUrl($fullpath, 1, 1);
 		my $rate  = 0;
 
 		if (blessed($track) && $track->can('bitrate')) {
@@ -1778,11 +1823,9 @@ sub underMax {
 
 sub getConvertCommand {
 	my $client = shift;
-	my $track  = shift;
+	my $track  = shift || return;
+	my $type   = shift || Slim::Music::Info::contentType($track);
 	
-	return unless $track;
-	
-	my $type     = Slim::Music::Info::contentType($track);
 	my $player;
 	my $clientid;
 	my $command  = undef;
@@ -1860,14 +1903,14 @@ sub getConvertCommand {
 			# to a complete stream that we can send to SB2.
 			# Yucky, but a stopgap until we get FLAC seeking code into
 			# a Perl invokable form.
-			elsif (($type eq "flc") && ($track->url =~ /#([^-]+)-([^-]+)$/)) {
+			elsif (($type eq "flc") && (blessed($track) && $track->url =~ /#([^-]+)-([^-]+)$/)) {
 				$command = $commandTable{"flc-flc-transcode-*"};
 			}
 
 			$undermax = 1;
 
 			# We can't handle WMA Lossless in firmware. So move to the next format type.
-			if ($type eq 'wma' && $checkformat eq 'wma' && $track->lossless) {
+			if (blessed($track) && $type eq 'wma' && $checkformat eq 'wma' && $track->lossless) {
 
 				next;
 			}
@@ -1919,6 +1962,11 @@ sub tokenizeConvertCommand {
 		$fullpath =~ s/([\$\"\`])/\\$1/g;
 	}
 	
+	# Bug 3396, mov123 commands for URLs must pass the URL to mov123, instead of using stdin
+	if ( $command =~ /mov123/ && $fullpath =~ /^http/ ) {
+		$filepath = $fullpath;
+	}
+	
 	$command =~ s/\$FILE\$/"$filepath"/g;
 	$command =~ s/\$URL\$/"$fullpath"/g;
 	$command =~ s/\$RATE\$/$samplerate/g;
@@ -1997,7 +2045,7 @@ sub readNextChunk {
 					my $now   = $client->songBytes() + $song->{offset};
 					my $ds    = Slim::Music::Info::getCurrentDataStore();
 					my $url   = Slim::Player::Playlist::song($client, streamingSongIndex($client));
-					my $track = $ds->objectForUrl($url);
+					my $track = $ds->objectForUrl($url, 1, 1);
 
 					if (!blessed($track) || !$track->can('bitrate')) {
 
@@ -2205,7 +2253,7 @@ sub protocolHandlerForURL {
 	my $url = shift;
 	
 	my ($protocol) = $url =~ /^([a-zA-Z0-9\-]+):/;
-
+	
 	return undef if !$protocol;
 
 	return $protocolHandlers{lc $protocol};
