@@ -21,6 +21,7 @@ use MIME::Base64;
 use Slim::Hardware::mas35x9;
 
 use base qw(Slim::Player::Player);
+use Scalar::Util qw(blessed);
 
 BEGIN {
 	if ($^O =~ /Win32/) {
@@ -151,14 +152,16 @@ sub play {
 	$client->volume($client->volume(),
 					defined($client->tempVolume()));
 
-	# if this is a remote stream, then let's start after 5 seconds even if we haven't filled the buffer yet.
-	my $quickstart = Slim::Music::Info::isRemoteURL($url) ? 5 : undef;
+	# if this is a remote stream, start playback as soon as we have enough buffer filled
+	my $quickstart = Slim::Music::Info::isRemoteURL($url) ? 0.125 : undef;
 
 	Slim::Utils::Timers::killTimers($client, \&quickstart);
 	if ($quickstart) {
-		Slim::Utils::Timers::setTimer($client,Time::HiRes::time() + $quickstart,\&quickstart);
+		Slim::Utils::Timers::setTimer( $client, Time::HiRes::time() + $quickstart, \&quickstart );
 	}
 	
+	$client->lastSong($url);
+
 	return 1;
 }
 
@@ -197,6 +200,7 @@ sub stop {
 	Slim::Networking::Slimproto::stop($client);
 	# disassociate the streaming socket to the client from the client.  HTTP.pm will close the socket on the next select.
 	$client->streamingsocket(undef);
+	$client->lastSong(undef);
 }
 
 sub flush {
@@ -211,14 +215,60 @@ sub flush {
 
 sub quickstart {
 	my $client = shift;
+	
+	$client->requestStatus();
+	
 	my $fullness = $client->bufferFullness();
+	
+	# begin playback once we have this much data in the buffer
+	my $threshold = 20 * 1024;
 
-	# make sure we have at least 10K before starting with a quickstart.  If not, then check again in a second.
-	if ($fullness > 10 * 1024) {
+	# Resume if we've hit the threshold, unless synced (sync unpauses all clients together)
+	if ( $fullness >= $threshold && !Slim::Player::Sync::isSynced($client) ) {
 		$client->resume();
-	} else {
-		$client->requestStatus();
-		Slim::Utils::Timers::setTimer($client,Time::HiRes::time() + 1,\&quickstart);
+	}
+	else {
+
+		# Bug 1827, display better buffering feedback while we wait for data
+		my $percent = sprintf "%d", ( $fullness / $threshold ) * 100;
+		
+		my ( $line1, $line2 );
+		
+		if ( $percent == 0 ) {
+			$line1 = $client->string('NOW_PLAYING') . ' (' . $client->string('CONNECTING_FOR') . ')';
+			
+			if ( $client->linesPerScreen() == 1 ) {
+				$line2 = $client->string('CONNECTING_FOR');
+			}
+		}
+		else {
+			my $status;
+			
+			# When synced, a player may have to wait longer than the buffering time
+			if ( Slim::Player::Sync::isSynced($client) && $percent >= 100 ) {
+				$status = $client->string('WAITING_TO_SYNC');
+			}
+			else {
+				$status = $client->string('BUFFERING') . ' ' . $percent . '%';
+			}
+			
+			$line1 = $client->string('NOW_PLAYING') . ' (' . $status . ')';
+			
+			# Display only buffering text in large text mode
+			if ( $client->linesPerScreen() == 1 ) {
+				$line2 = $status;
+			}
+		}
+		
+		# Find the track title
+		if ( $client->linesPerScreen() > 1 ) {
+			my $objOrUrl = Slim::Player::Playlist::song( $client, Slim::Player::Source::streamingSongIndex($client) );
+			$line2       = Slim::Music::Info::getCurrentTitle( $client, $objOrUrl );
+		}
+		
+		$client->showBriefly( $line1, $line2, 0.5 );
+		
+		Slim::Utils::Timers::setTimer( $client, Time::HiRes::time() + 0.125, \&quickstart );
 	}
 }
 
@@ -506,7 +556,7 @@ sub opened {
 
 # Squeezebox control for tcp stream
 #
-#	u8_t command;		// [1]	's' = start, 'p' = pause, 'u' = unpause, 'q' = stop, 't' = status
+#	u8_t command;		// [1]	's' = start, 'p' = pause, 'u' = unpause, 'q' = stop, 'f' = flush
 #	u8_t autostart;		// [1]	'0' = don't auto-start, '1' = auto-start, '2' = direct streaming
 #	u8_t mode;		// [1]	'm' = mpeg bitstream, 'p' = PCM
 #	u8_t pcm_sample_size;	// [1]	'0' = 8, '1' = 16, '2' = 24, '3' = 32
@@ -591,7 +641,12 @@ sub stream {
 			# to indicate whether the data coming in is
 			# going to have the mms/http chunking headers.
 			if ($server_url) {
-				$pcmsamplesize = '1';
+				if ($server_url =~ /^rhap:/) {
+					$pcmsamplesize = '2';
+				}
+				else {
+					$pcmsamplesize = '1';
+				}
 			}
 			else {
 				$pcmsamplesize = '0';
@@ -609,49 +664,63 @@ sub stream {
 		
 		my $request_string;
 		my ($server_port, $server_ip);
-		if ($command eq 's' && $server_url) {
-			$::d_directstream && msg("This player supports direct streaming for $url as $server_url, let's do it.\n");
-		
-			my ($server, $port, $path, $user, $password) = Slim::Utils::Misc::crackURL($server_url);
-			my ($name, $liases, $addrtype, $length, @addrs) = gethostbyname($server);
-			if ($port && $addrs[0]) {
-				$server_port = $port;
-				$server_ip = unpack('N',$addrs[0]);
-			}
-
-			$request_string = $handler->requestString($server_url);  
-			$autostart += 2;
-			if (!$server_port || !$server_ip) {
-				$::d_directstream && msg("Couldn't get an IP and Port for direct stream ($server_ip:$server_port), failing.\n");
-				$client->failedDirectStream();
-				Slim::Networking::Slimproto::stop($client);
-				return;
-			} else {
-				$::d_directstream && msg("setting up direct stream ($server_ip:$server_port) autostart: $autostart.\n");
-				$::d_directstream && msg("request string: $request_string\n");
-				$client->directURL($url);
-			}			
-	
-		} else {
-			my $path = '/stream.mp3?player='.$client->id;
-		
-			$request_string = "GET $path HTTP/1.0\n";
+		if ($command eq 's') {
 			
-			if (Slim::Utils::Prefs::get('authorize')) {
-				$client->password(generate_random_string(10));
+			# When streaming a new song, we reset the buffer fullness value so quickstart()
+			# doesn't get an outdated fullness result and unpause too early
+			Slim::Networking::Slimproto::fullness( $client, 0 );
+			
+			if ($server_url) {
 				
-				my $password = encode_base64('squeezeboxXXX:' . $client->password);
+				$::d_directstream && msg("This player supports direct streaming for $url as $server_url, let's do it.\n");
+		
+				my ($server, $port, $path, $user, $password) = Slim::Utils::Misc::crackURL($server_url);
+				my ($name, $liases, $addrtype, $length, @addrs) = gethostbyname($server);
+				if ($port && $addrs[0]) {
+					$server_port = $port;
+					$server_ip = unpack('N',$addrs[0]);
+				}
+
+				$request_string = $handler->requestString($client, $server_url, undef, 1);  
+				$autostart = 2;
+				if (!$server_port || !$server_ip) {
+					$::d_directstream && msg("Couldn't get an IP and Port for direct stream ($server_ip:$server_port), failing.\n");
+					$client->failedDirectStream();
+					Slim::Networking::Slimproto::stop($client);
+					return;
+				} else {
+					$::d_directstream && msg("setting up direct stream ($server_ip:$server_port) autostart: $autostart.\n");
+					$::d_directstream && msg("request string: $request_string\n");
+					$client->directURL($url);
+				}			
+	
+			} else {
+				my $path = '/stream.mp3?player='.$client->id;
+		
+				$request_string = "GET $path HTTP/1.0\n";
+			
+				if (Slim::Utils::Prefs::get('authorize')) {
+					$client->password(generate_random_string(10));
 				
-				$request_string .= "Authorization: Basic $password\n";
-			}
-			$server_port = Slim::Utils::Prefs::get('httpport');		# port
-			$server_ip = 0;		# server IP of 0 means use IP of control server
-			$request_string .= "\n";
-			if (length($request_string) % 2) {
+					my $password = encode_base64('squeezeboxXXX:' . $client->password);
+				
+					$request_string .= "Authorization: Basic $password\n";
+				}
+				$server_port = Slim::Utils::Prefs::get('httpport');		# port
+				$server_ip = 0;		# server IP of 0 means use IP of control server
 				$request_string .= "\n";
+				if (length($request_string) % 2) {
+					$request_string .= "\n";
+				}
 			}
 		}
 		
+		# If we're sending an 's' command but got no request string, don't send it
+		# This is used when syncing Rhapsody radio stations so slaves don't request the radio playlist
+		if ( $command eq 's' && !$request_string ) {
+			return;
+		}
+		
 		$::d_slimproto && msg("starting with decoder with format: $formatbyte autostart: $autostart threshold: $bufferThreshold samplesize: $pcmsamplesize samplerate: $pcmsamplerate endian: $pcmendian channels: $pcmchannels\n");
 
 		my $flags = 0;
@@ -673,8 +742,8 @@ sub stream {
 			$flags,		# flags	     
 			0,		# reserved
 			$client->canDoReplayGain($replay_gain),		
-			$server_port,  # use slim server's IP
-			$server_ip
+			$server_port || Slim::Utils::Prefs::get('httpport'),  # use slim server's IP
+			$server_ip || 0,
 		);
 	
 		assert(length($frame) == 24);
@@ -775,7 +844,7 @@ sub sendPitch {
 }
 	
 sub maxPitch {
-	return 120;
+	return 110;
 }
 
 sub minPitch {
