@@ -130,12 +130,16 @@ my @modes = (
 	  width => 320, 
 	  params => [$VISUALIZER_SPECTRUM_ANALYZER, 0, 0, 0x10000, 0, 160, 0, 4, 1, 1, 1, 1, 160, 160, 1, 4, 1, 1, 1, 1] } , 
 	  
-	{ bar => 0, 
+	{ bar => 1, 
 	  secs => 0, 
 	  width => 320, 
-	  params => [$VISUALIZER_NONE], fullness => 1 }
+	  params => [$VISUALIZER_SPECTRUM_ANALYZER, 0, 0, 0x10000, 0, 160, 0, 4, 1, 1, 1, 1, 160, 160, 1, 4, 1, 1, 1, 1], fullness => 1 }
 );
 
+my @WMA_FILE_PROPERTIES_OBJECT_GUID = (0x8c, 0xab, 0xdc, 0xa1, 0xa9, 0x47, 0x11, 0xcf, 0x8e, 0xe4, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65);
+my @WMA_CONTENT_DESCRIPTION_OBJECT_GUID = (0x75, 0xB2, 0x26, 0x33, 0x66, 0x8E, 0x11, 0xCF, 0xA6, 0xD9, 0x00, 0xAA, 0x00, 0x62, 0xCE, 0x6C);
+my @WMA_EXTENDED_CONTENT_DESCRIPTION_OBJECT_GUID = (0xd2, 0xd0, 0xa4, 0x40, 0xe3, 0x07, 0x11, 0xd2, 0x97, 0xf0, 0x00, 0xa0, 0xc9, 0x5e, 0xa8, 0x50);
+
 sub playingModeOptions { 
 	my $client = shift;
 	my %options = (
@@ -217,12 +221,12 @@ sub displayHeight {
 	return 32;
 }
 
-sub maxBass { 0 };
-sub minBass { 0 };
-sub maxTreble { 0 };
-sub minTreble { 0 };
-sub maxPitch { 0 };
-sub minPitch { 0 };
+sub maxBass { 50 };
+sub minBass { 50 };
+sub maxTreble { 50 };
+sub minTreble { 50 };
+sub maxPitch { 100 };
+sub minPitch { 100 };
 
 sub model {
 	return 'squeezebox2';
@@ -276,7 +280,14 @@ sub dBToFixed {
 	# Map a floating point dB value to a 16.16 fixed point value to
 	# send as a new style volume to SB2 (FW 22+).
 	my $floatmult = 10 ** ($db/20);
-	return int(($floatmult * (1 << 16)) + 0.5);
+	
+	# use 8 bits of accuracy for dB values greater than -30dB to avoid rounding errors
+	if ($db >= -30 && $db <= 0) {
+		return int($floatmult * (1 << 8) + 0.5) * (1 << 8);
+	}
+	else {
+		return int(($floatmult * (1 << 16)) + 0.5);
+	}
 }
 
 sub volume {
@@ -656,8 +667,9 @@ sub canDirectStream {
 	my $url = shift;
 
 	my $handler = Slim::Player::Source::protocolHandlerForURL($url);
+
 	if ($handler && $handler->can("canDirectStream")) {
-		return $handler->canDirectStream($url);
+		return $handler->canDirectStream($client, $url);
 	}
 	
 	return undef;
@@ -703,15 +715,19 @@ sub directHeaders {
 	
 		$response = $1;
 		
-		if ($response < 200) {
+		if (($response < 200) || $response > 399) {
 			$::d_directstream && msg("Invalid response code ($response) from remote stream $url\n");
-			$client->failedDirectStream();
-		} elsif ($response > 399) {
-			$::d_directstream && msg("Invalid response code ($response) from remote stream $url\n");
-			$client->failedDirectStream();
+			if ($handler && $handler->can("handleDirectError")) {
+				$handler->handleDirectError($client, $url, $response);
+			}
+			else {
+				$client->failedDirectStream();
+			}
 		} else {
 			my $redir = '';
 			my $metaint = 0;
+			my @guids = ();
+			my $guids_length = 0;
 			my $length;
 			my $title;
 			my $contentType = "audio/mpeg";  # assume it's audio.  Some servers don't send a content type.
@@ -734,7 +750,7 @@ sub directHeaders {
 						$title = Slim::Utils::Unicode::utf8decode_guess($1, 'iso-8859-1');
 					}
 					
-					if ($header =~ /^icy-br:\s*(.+)\015\012$/i) {
+					if ($header =~ /^icy-br:\s*(.+)/i) {
 						$bitrate = $1 * 1000;
 					}
 				
@@ -764,6 +780,19 @@ sub directHeaders {
 			Slim::Music::Info::setCurrentTitle($url, $title) if $title;
 			$::d_directstream && msg("got a stream type:: $contentType  bitrate: $bitrate  title: $title\n");
 
+			if ($contentType eq 'wma') {
+			    push @guids, @WMA_FILE_PROPERTIES_OBJECT_GUID;
+			    push @guids, @WMA_CONTENT_DESCRIPTION_OBJECT_GUID;
+			    push @guids, @WMA_EXTENDED_CONTENT_DESCRIPTION_OBJECT_GUID;
+
+			    $guids_length = $#guids;
+
+			    # sending a length of -1 will return all wma header objects
+			    # for debugging
+			    ##@guids = ();
+			    ##$guids_length = -1;
+			}
+
 			if ($redir) {
 				$::d_directstream && msg("Redirecting to: $redir\n");
 				$client->stop();
@@ -787,7 +816,7 @@ sub directHeaders {
 				    $loop = Slim::Player::Source::shouldLoop($client);
 				}
 				$client->streamformat($contentType);
-				$client->sendFrame('cont', \(pack('NC',$metaint, $loop)));		
+				$client->sendFrame('cont', \(pack('NCnC*',$metaint, $loop, $guids_length, @guids)));		
 			} else {
 				$::d_directstream && msg("Direct stream failed\n");
 				$client->failedDirectStream();
@@ -830,7 +859,7 @@ sub directBodyFrame {
 			# it a chance, else we parse based on the type we know
 			# already.
 			if ($handler && $handler->can('parseDirectBody')) {
-				@items = $handler->parseDirectBody($url, $client->directBody());
+				@items = $handler->parseDirectBody($client, $url, $client->directBody());
 			}
 			else {
 				my $io = IO::String->new($client->directBody());
@@ -856,8 +885,16 @@ sub directBodyFrame {
 sub directMetadata {
 	my $client = shift;
 	my $metadata = shift;
-
-	Slim::Player::Protocols::HTTP::parseMetadata($client, Slim::Player::Playlist::song($client), $metadata);
+	
+	my $url = $client->directURL;
+	my $type = Slim::Music::Info::contentType($url);
+	
+	if ( $type eq 'wma' ) {
+		Slim::Player::Protocols::MMS::parseMetadata( $client, Slim::Player::Playlist::song($client), $metadata );
+	}
+	else {
+		Slim::Player::Protocols::HTTP::parseMetadata( $client, Slim::Player::Playlist::song($client), $metadata );
+	}
 	
 	# new song, so reset counters
 	$client->songBytes(0);
@@ -871,7 +908,9 @@ sub failedDirectStream {
 	$client->directBody(undef);
 
 	Slim::Player::Source::errorOpening($client, $client->string("PROBLEM_CONNECTING"));
-	Slim::Player::Source::underrun($client);
+	
+	# 1 means underrun due to error
+	Slim::Player::Source::underrun($client, 1);
 }
 
 sub canLoop {
