@@ -1,6 +1,6 @@
 package Slim::Utils::Misc;
 
-# $Id: Misc.pm 4972 2005-11-01 00:21:47Z dsully $
+# $Id: Misc.pm 8079 2006-06-21 20:07:46Z dsully $
 
 # SlimServer Copyright (c) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -12,6 +12,7 @@ use base qw(Exporter);
 
 our @EXPORT = qw(assert bt msg msgf watchDog errorMsg);
 
+use Cwd ();
 use File::Spec::Functions qw(:ALL);
 use File::Which ();
 use FindBin qw($Bin);
@@ -42,6 +43,8 @@ BEGIN {
 
 		require Win32::Shortcut;
 		require Win32::OLE::NLS;
+		require Win32::API;
+		require Win32::Process;
 		require Win32;
 
 	} else {
@@ -114,6 +117,43 @@ sub findbin {
 	return $path;	
 }
 
+sub setPriority {
+	my $priority = shift;
+
+	# By default, set the Windows priority to be high - so we get swapped
+	# back in faster, and have a less likely chance of being swapped out
+	# in the first place.
+	# 
+	# For *nix, including OSX, set whatever priority the user gives us.
+	if (Slim::Utils::OSDetect::OS() eq 'win') {
+
+		my $getCurrentProcess = Win32::API->new('kernel32', 'GetCurrentProcess', ['V'], 'N');
+		my $setPriorityClass  = Win32::API->new('kernel32', 'SetPriorityClass',  ['N', 'N'], 'N');
+
+		if (blessed($setPriorityClass) && blessed($getCurrentProcess)) {
+
+			my $processHandle = eval { $getCurrentProcess->Call(0) };
+
+			if (!$processHandle || $@) {
+
+				errorMsg("setPriority: Can't get process handle ($^E) [$@]\n");
+				return;
+			};
+
+			eval { $setPriorityClass->Call($processHandle, Win32::Process::NORMAL_PRIORITY_CLASS()) };
+
+			if ($@) {
+				errorMsg("setPriority: Couldn't set priority to NORMAL ($^E) [$@]\n");
+			}
+		}
+
+	} elsif ($priority) {
+
+		$::d_server && msg("SlimServer - changing process priority to $priority\n");
+                eval { setpriority (0, 0, $priority); };
+	}
+}
+
 sub pathFromWinShortcut {
 	my $fullpath = pathFromFileURL(shift);
 
@@ -182,6 +222,9 @@ sub pathFromFileURL {
 
 		$url = Slim::Utils::Unicode::utf8off($url);
 	}
+	
+	# Bug 3589, support win32 backslashes in URLs, file://C:\foo\bar
+	$url =~ s/\\/\//g;
 
 	my $uri = URI->new($url);
 
@@ -197,7 +240,7 @@ sub pathFromFileURL {
 
 		# only allow absolute file URLs and don't allow .. in files...
 		if ($path !~ /[\/\\]\.\.[\/\\]/) {
-			$file = $uri->file();
+			$file = fixPathCase($uri->file);
 		} 
 
 	} else {
@@ -219,7 +262,7 @@ sub fileURLFromPath {
 	
 	return $path if (Slim::Music::Info::isURL($path));
 
-	my $uri  = URI::file->new($path);
+	my $uri  = URI::file->new( fixPathCase($path) );
 	$uri->host('');
 	return $uri->as_string;
 }
@@ -274,8 +317,6 @@ sub crackURL {
 
 # fixPathCase makes sure that we are using the actual casing of paths in
 # a case-insensitive but case preserving filesystem.
-# currently only implemented for Win32
-
 sub fixPathCase {
 	my $path = shift;
 	my $orig = $path;
@@ -284,15 +325,19 @@ sub fixPathCase {
 		$path = Win32::GetLongPathName($path);
 	}
 
+	# abs_path() will resolve any case sensetive filesystem issues (HFS+)
+	# But don't for the bogus path we use with embedded cue sheets.
+	if ($^O eq 'darwin' && $path !~ m|^/BOGUS/PATH|) {
+		$path = Cwd::abs_path($path);
+	}
+
 	# Use the original path if we didn't get anything back from
 	# GetLongPathName - this can happen if a cuesheet references a
 	# non-existant .wav file, which is often the case.
 	#
 	# At that point, we'd return a bogus value, and start crawling at the
 	# top of the directory tree, which isn't what we want.
-	$path = $orig unless $path;
-
-	return canonpath($path);
+	return $path || $orig;
 }
 		
 # there's not really a better way to do this..
@@ -382,13 +427,18 @@ sub fixPath {
 		$fixed = catfile($audiodir, $file);
 	}
 
-	# I hate Windows.
+	# I hate Windows & HFS+.
 	# A playlist or the like can have a completely different case than
 	# what we get from the filesystem. Fix that all up so we don't create
 	# duplicate entries in the database.
-	if (Slim::Utils::OSDetect::OS() eq "win" && !Slim::Music::Info::isFileURL($fixed)) {
+	if (!Slim::Music::Info::isFileURL($fixed)) {
 
-		$fixed = fixPathCase($fixed);
+		$fixed = canonpath(fixPathCase($fixed));
+
+		# Bug: 2757
+		if (-l $fixed) {
+			$fixed = readlink($fixed);
+		}
 	}
 
 	$::d_paths && ($file ne $fixed) && msg("*****fixed: " . $file . " to " . $fixed . "\n");
@@ -521,17 +571,26 @@ sub virtualToAbsolute {
 	}
 }
 
+sub inAudioFolder {
+	return _checkInFolder(shift, 'audiodir');
+}
+
 sub inPlaylistFolder {
+	return _checkInFolder(shift, 'playlistdir');
+}
+
+sub _checkInFolder {
 	my $path = shift || return;
+	my $pref = shift;
 
 	# Fully qualify the path - and strip out any url prefix.
 	$path = fixPath($path) || return 0;
 	$path = virtualToAbsolute($path) || return 0;
 	$path = pathFromFileURL($path) || return 0;
 
-	my $playlistdir = Slim::Utils::Prefs::get("playlistdir");
+	my $checkdir = Slim::Utils::Prefs::get($pref);
 
-	if ($playlistdir && $path =~ /^\Q$playlistdir\E/) {
+	if ($checkdir && $path =~ /^\Q$checkdir\E/) {
 		return 1;
 	} else {
 		return 0;
@@ -593,7 +652,8 @@ sub readDirectory {
 		# Ignore special named files and directories
 		# __ is a match against our old __history and __mac playlists.
 		# ._Foo is a OS X meta file.
-		next if $item =~ /^(?:_|\.)_/;
+		next if $item =~ /^__\S+\.m3u$/;
+		next if $item =~ /^\._/;
 
 		if ($ignore ne '') {
 			next if $item =~ /$ignore/;
@@ -660,7 +720,7 @@ sub findAndScanDirectoryTree {
 
 	if (!blessed($topLevelObj) || !$topLevelObj->can('path')) {
 
-		msg("Error: Couldn't find a topLevelObj for findAndScanDirectoryTree()\n");
+		errorMsg("Couldn't find a topLevelObj for findAndScanDirectoryTree()\n");
 
 		if (scalar @$levels) {
 			msgf("Passed in value was: [%s]\n", $levels->[-1]);
@@ -680,8 +740,8 @@ sub findAndScanDirectoryTree {
 	if ($fsMTime != $dbMTime) {
 
 		if ($::d_scan) {
-			msg("mtime db: $dbMTime : " . localtime($dbMTime) . "\n");
-			msg("mtime fs: $fsMTime : " . localtime($fsMTime) . "\n");
+			msg("path: $path mtime db: $dbMTime : " . localtime($dbMTime) . "\n");
+			msg("path: $path mtime fs: $fsMTime : " . localtime($fsMTime) . "\n");
 		}
 
 		# Update the mtime in the db.
@@ -693,6 +753,8 @@ sub findAndScanDirectoryTree {
 			'url'       => $path,
 			'recursive' => 0,
 		});
+
+		$ds->forceCommit;
 	}
 
 	# Now read the raw directory and return it. This should always be really fast.
