@@ -1,6 +1,6 @@
 package Slim::Utils::Prefs;
 
-# $Id: Prefs.pm 5082 2005-11-07 23:44:44Z dsully $
+# $Id: Prefs.pm 8033 2006-06-17 17:13:13Z dsully $
 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License, 
@@ -108,6 +108,7 @@ sub init {
 		'webproxy'		=> '',
 		"udpChunkSize"		=> 1400,
 		'itemsPerPage'		=> 50,
+		'disableStatistics'	=> 0,
 		'lookForArtwork'	=> 1,
 		'includeNoArt'		=> 0,
 		'artfolder'		=> '',
@@ -115,7 +116,6 @@ sub init {
 		'coverArt'		=> '',
 		'thumbSize'		=> 100,
 		'itemsPerPass'		=> 5,
-		'keepUnswappedInterval'	=> 30,
 		'plugins-onthefly'	=> 0,
 		'longdateFormat'	=> q(%A, %B |%d, %Y),
 		'shortdateFormat'	=> q(%m/%d/%Y),
@@ -162,8 +162,6 @@ sub init {
 		'dbsource'		=> 'dbi:SQLite:dbname=%s',
 		'dbusername'		=> '',
 		'dbpassword'		=> '',
-		'commonAlbumTitles'	=> ['Greatest Hits', 'Best of...', 'Live'],
-		'commonAlbumTitlesToggle' => 0,
 		'noGenreFilter'		=> 0,
 		'variousArtistAutoIdentification' => 0,
 		'useBandAsAlbumArtist'  => 0,
@@ -463,6 +461,12 @@ sub checkServerPrefs {
 		}
 	}
 
+	# For those that downgrade from 6.5 to 6.3, flip the dbsource back.
+	if ($prefs{'dbsource'} =~ /port=9092/) {
+
+		$prefs{'dbsource'} = $DEFAULT{'dbsource'};
+	}
+
 	for my $version (sort keys %upgradeScripts) {
 
 		if (Slim::Utils::Prefs::get("upgrade-$version-script")) {
