@@ -1,6 +1,6 @@
 package Slim::Utils::Scan;
           
-# $Id: Scan.pm 5038 2005-11-04 21:11:44Z dsully $
+# $Id: Scan.pm 7987 2006-06-15 00:43:24Z dsully $
 
 # SlimServer Copyright (c) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -59,6 +59,7 @@ use Scalar::Util qw(blessed);
 use Time::HiRes;
 
 use Slim::Music::Info;
+use Slim::Networking::AsyncHTTP;
 use Slim::Utils::Misc;
 use Slim::Formats::Parse;
 
@@ -78,7 +79,8 @@ struct (addToList_jobState => [
 	numstack       => '$', # number of levels on the stack
 	numitems       => '$', # number of items currently in the list
 	numitems_start => '$', # number of items in the list before addToList
-	playlisturl	   => '$', # initial value of playlist URL
+	playlisturl    => '$', # initial value of playlist URL
+	originalurl    => '$', # un-fixed URL
 
 	callbackf      => '$', #function to call when we're done
 	callbackargs   => '$', #ref to array of callback args
@@ -90,6 +92,43 @@ my %addToList_jobs = ();    # each job is referenced by its listref.
 #
 sub addToList {
 	my $args = shift;
+	
+	# If we have a remote URL, we need to test that we can connect to it.
+	# This prevents long blocking in the RemoteStream code if the server is
+	# unreachable.  It also allows us to have a common error message
+	# for streams we can't connect to, no matter what the extension.
+	if ( Slim::Music::Info::isHTTPURL( $args->{'url'} ) ) {
+		
+		checkConnect(
+			\&addToList_continue,
+			\&addToList_error,
+			$args,
+		);
+	}
+	else {
+		
+		addToList_continue( $args );
+	}
+}
+
+sub addToList_error {
+	my $args = shift;
+	
+	# a file, directory, or URL to be scanned
+	my $playlisturl  = $args->{'url'};	
+	
+	# Optional: function to call when finished
+	my $callbackf    = $args->{'callback'};
+
+	# Optional: callback args - number of items scanned will be appended.
+	my $callbackArgs = $args->{'callbackArgs'} || [];
+	
+	# Report back to the callback that we scanned 0 items
+	$callbackf && $callbackf->( @{$callbackArgs}, 0, $playlisturl );
+}
+
+sub addToList_continue {
+	my $args = shift;
 
 	# reference to the list which we're to append
 	my $listref      = $args->{'listRef'} || [];
@@ -116,9 +155,27 @@ sub addToList {
 	if (Slim::Music::Info::isWinShortcut($playlisturl)) {
 		$playlisturl = Slim::Utils::Misc::pathFromWinShortcut($playlisturl);
 	}
+	
+	# Do not scan Rhapsody audio or radio files
+	if ( $playlisturl =~ /^rhap.+(?:wma|rhr)$/ ) {
+
+		push @$listref, $playlisturl;
+		$callbackf && (&$callbackf(@{$callbackArgs}, 1));
+		return;
+	}
+	
+	my $playlist_filehandle;
+	if ( Slim::Music::Info::isRemoteURL($playlisturl) && !Slim::Music::Info::isAudioURL($playlisturl) ) {
+
+		# For Remote URLs, we need to connect to the remote stream to check a few things:
+		# 1. The remote URL may not have a file extension therefore we can't guess the content-type
+		# 2. The URL might redirect (a lot of Podcasts do this)
+		# 3. The extension might not match the content-type sent by the server.
+		$playlist_filehandle = checkContentType( $playlisturl, $args->{'client'} );
+	}
 
 	# special case, if we try to add a song, then just add it
-	if (Slim::Music::Info::isSong($playlisturl)) {
+	if (Slim::Music::Info::isSong($playlisturl) || Slim::Music::Info::isAudioURL($playlisturl) ) {
 
 		push @$listref, $playlisturl;
 		$callbackf && (&$callbackf(@{$callbackArgs}, 1));
@@ -126,9 +183,9 @@ sub addToList {
 	} elsif (Slim::Music::Info::isPlaylist($playlisturl)) {
 
 		# regular playlists (m3u, pls, itu, cue) are parsed and loaded immediately and we never recurse and never sort
-		my $count = readList($playlisturl, $listref);
+		my $count = readList( $playlisturl, $listref, $playlist_filehandle, $args->{'client'} );
 
-		$callbackf && (&$callbackf(@{$callbackArgs}, $count));
+		$callbackf && (&$callbackf(@{$callbackArgs}, $count, $args->{'url'}));
 
 	} else {	
 		# Initialize the base directory, with index == -1 to indicate 
@@ -148,6 +205,7 @@ sub addToList {
 		$addToList_jobs{$listref}->callbackf($callbackf);
 		$addToList_jobs{$listref}->callbackargs($callbackArgs);
 		$addToList_jobs{$listref}->playlisturl($playlisturl);
+		$addToList_jobs{$listref}->originalurl($args->{'url'});
 		
 		# if we have a callback function, then schedule it if appropriate
 		
@@ -349,7 +407,7 @@ sub addToList_run {
 	$::d_scan && msg("isList($itempath) == ". (Slim::Music::Info::isList($itempath) || 0) . "\n");
 
 	# todo: don't let us recurse indefinitely
-	if (Slim::Music::Info::isList($itempath)) {
+	if ($itempath && Slim::Music::Info::isList($itempath)) {
 
 		if ($jobState->recursive && !Slim::Music::Info::isRemoteURL($itempath)) {
 
@@ -372,7 +430,7 @@ sub addToList_run {
 	######### Else if it's a single item - look up the sort key (takes a while)
 	$::d_scan && msg("not a list: $itempath\n");
 
-	if (Slim::Music::Info::isSong($itempath)) {
+	if ($itempath && Slim::Music::Info::isSong($itempath)) {
 
 		$::d_scan && msg("adding single item: $itempath, type " . Slim::Music::Info::contentType($itempath) . "\n");
 
@@ -410,27 +468,66 @@ sub addToList_done {
 		for my $item (@$listref) {
 			msg("  $item\n");
 		}
+		
 	}
 
-	$callbackf && (&$callbackf(@$callbackargs, $jobState->numitems));
+	$callbackf && (&$callbackf(@$callbackargs, $jobState->numitems, $jobState->originalurl));
+}
+
+sub checkContentType {
+	my $url    = shift;
+	my $client = shift;
+	
+	# If the URL begins with MMS, assume wma
+	if ( $url =~ /^mms/i ) {
+		Slim::Music::Info::setContentType( $url, 'wma' );
+		return;
+	}
+
+	my $sock = Slim::Player::Source::openRemoteStream($url, $client);
+	if ( $sock ) {
+		my $contentType = Slim::Music::Info::mimeToType($sock->contentType) || $sock->contentType;
+		
+		# Bug 3396, some m4a audio is incorrectly served as audio/mpeg.
+		# In this case, prefer the file extension to the content-type
+		if ( $url =~ /(m4a|aac)$/i ) {
+			$contentType = 'mov';
+		}
+		
+		Slim::Music::Info::setContentType( $url, $contentType );
+		
+		$::d_scan && msg("Scan::checkContentType: Remote content type is $contentType [$url]\n");
+	}
+	
+	# If it's a playlist, we return the opened filehandle so it doesn't need
+	# to be requested a second time in readList
+	if ( Slim::Music::Info::isPlaylist($url) ) {
+		return $sock;
+	}
+	else {
+		$sock->close if $sock;
+		return;
+	}
 }
 
 sub readList {   # reads a directory or playlist and returns the contents as an array
-	my ($playlisturl, $listref) = @_;
+	my ($playlisturl, $listref, $playlist_filehandle, $client) = @_;
 
 	$::d_scan && msg("Scan::readList gonna read $playlisturl\n");
 
-	my ($playlist_filehandle, $numitems);
+	my $numitems = 0;
 	
-	$numitems = 0;
 	my $startingsize = scalar @$listref;
 	my $ds = Slim::Music::Info::getCurrentDataStore();
 
 	my $playlistpath = $playlisturl;
 	
 	if (Slim::Music::Info::isRemoteURL($playlisturl)) {
-		$::d_scan && msg("Scan::readList opening remote stream $playlisturl\n");
-		$playlist_filehandle = Slim::Player::Source::openRemoteStream($playlisturl);
+		
+		if ( !defined $playlist_filehandle ) {
+			$::d_scan && msg("Scan::readList opening remote stream $playlisturl\n");
+			$playlist_filehandle = Slim::Player::Source::openRemoteStream($playlisturl, $client);
+		}
 
 		unless ($playlist_filehandle) {
 			warn "cannot connect to http daemon to get playlist";
@@ -496,7 +593,7 @@ sub readList {   # reads a directory or playlist and returns the contents as an
 		}
 
 		# 315529200 is a bogus windows time value
-		my $obj = $ds->objectForUrl($playlistpath, 0);
+		my $obj = $ds->objectForUrl($playlistpath, 1, 1);
 		my $pls = Slim::Music::Info::cachedPlaylist($obj);
 
 		if (Slim::Music::Info::isPlaylistURL($playlistpath) ||
@@ -603,7 +700,18 @@ sub readList {   # reads a directory or playlist and returns the contents as an
 		#
 		# Bug 2048 - Don't add playlist files from the audiodir,
 		# unless they are cue sheets.
-		} elsif ((Slim::Music::Import::stillScanning() && Slim::Music::Info::isCUE($playlisturl)) || 
+		#
+		# Bug 2790 - People decide that playlist directories with
+		# windows shortcuts are a good idea. !inAudioFolder() was
+		# inPlaylistFolder() - but a shortcut points out of the
+		# playlist folder. So what we really want here is:
+		#
+		# Add a cue sheet from anywhere, as long as we're still
+		# scanning.
+		#
+		# Otherwise, playlists can be from anywhere but the audiodir.
+		} elsif (
+			(Slim::Music::Import::stillScanning() && Slim::Music::Info::isCUE($playlisturl)) || 
 			Slim::Utils::Misc::inPlaylistFolder($playlisturl)) {
 
 			# it's a playlist file
@@ -662,11 +770,7 @@ sub readList {   # reads a directory or playlist and returns the contents as an
 		# Add playlists to the database.
 		if ($numitems && scalar @$listref) {
 
-			# Create a playlist container
-			my $title = Slim::Web::HTTP::unescape(basename($playlisturl));
-			   $title =~ s/\.\w{3}$//;
-
-			my $ct    = Slim::Music::Info::contentType($playlisturl);
+			my $ct = Slim::Music::Info::contentType($playlisturl);
 
 			# With the special url if the playlist is in the
 			# designated playlist folder. Otherwise, Dean wants
@@ -683,6 +787,20 @@ sub readList {   # reads a directory or playlist and returns the contents as an
 			) {
 				$ct = 'ssp';
 			}
+			
+			# Bugs 2589, 2723 
+			# If a playlist item has no title or is just a URL, give it
+			# a friendlier title from the parent item
+			my $track = $ds->objectForUrl($playlisturl, 1, 1);
+			my $title = $track->title;
+			for my $item ( @{$listref} ) {
+				if ( blessed $item ) {
+					if ( !$item->title || $item->title =~ /^(http|mms)/i ) {
+						$item->title( $title );
+						$item->update;
+					}
+				}
+			}
 
 			Slim::Music::Info::updateCacheEntry($playlisturl, {
 				'TITLE' => $title,
@@ -702,7 +820,53 @@ sub readList {   # reads a directory or playlist and returns the contents as an
 
 	Slim::Music::Info::markAsScanned($playlistpath);
 	
-	return $numitems
+	return $numitems;
+}
+
+# Check that we can connect to the remote server
+sub checkConnect {
+	my ( $cb, $ecb, $args ) = @_;
+	
+	my ($server, $port, $path, $user, $password) = Slim::Utils::Misc::crackURL( $args->{'url'} );
+	
+	my $http = Slim::Networking::AsyncHTTP->new(
+		Host     => $server,
+		PeerPort => $port,
+		
+		errorCallback => \&checkConnectError,
+		writeCallback => \&checkConnectResponse,
+		callbackArgs  => [ $cb, $ecb, $args ],
+	);
+}
+
+sub checkConnectResponse {
+	my $http = shift;
+	my ( $cb, $ecb, $args ) = @_;
+	
+	if ( !$http->connected ) {
+		return checkConnectError( $http, $cb, $ecb, $args );
+	}
+	
+	$http->close;
+	
+	$::d_scan && msgf("Scan::checkConnect URL OK: %s\n",
+		$args->{'url'}
+	);
+	
+	$cb->( $args );
+}
+
+sub checkConnectError {
+	my $http = shift;
+	my ( $cb, $ecb, $args ) = @_;
+	
+	$http->close;
+	
+	$::d_scan && msgf("Scan::checkHTTP URL failed: %s\n",
+		$args->{'url'},
+	);
+	
+	$ecb->( $args );
 }
 
 1;
