@@ -1,6 +1,6 @@
 package Slim::Web::HTTP;
 
-# $Id: HTTP.pm 5077 2005-11-07 06:44:18Z dean $
+# $Id: HTTP.pm 7980 2006-06-14 21:19:05Z mherger $
 
 # SlimServer Copyright (c) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -10,6 +10,7 @@ package Slim::Web::HTTP;
 use strict;
 
 use Benchmark;
+use CGI::Cookie;
 use Digest::MD5;
 use FileHandle;
 use File::Basename qw(basename);
@@ -28,6 +29,7 @@ use Sys::Hostname;
 use Template;
 use Tie::RegexpHash;
 use URI::Escape;
+use HTML::Entities;
 use Slim::Networking::mDNS;
 use Slim::Networking::Select;
 use Slim::Player::HTTP;
@@ -51,6 +53,9 @@ BEGIN {
 		require Errno;
 		import Errno qw(EWOULDBLOCK EINPROGRESS);
 	}
+
+	# Use our custom Template::Context subclass
+	$Template::Config::CONTEXT = 'Slim::Web::Template::Context';
 }
 
 use constant defaultSkin => 'Default';
@@ -130,6 +135,7 @@ sub init {
 		qr/^status_header\.(?:htm|xml)/	=> \&Slim::Web::Pages::status_header,
 		qr/^status\.(?:htm|xml)/	=> \&Slim::Web::Pages::status,
 		qr/^setup\.(?:htm|xml)/		=> \&Slim::Web::Setup::setup_HTTP,
+		qr/^squeezenetwork\.(?:htm|xml)/	=> \&Slim::Web::Pages::squeezeNetwork,
 		qr/^tunein\.(?:htm|xml)/	=> \&Slim::Web::Pages::tuneIn,
 		qr/^update_firmware\.(?:htm|xml)/ => \&Slim::Web::Pages::update_firmware,
 	);
@@ -335,6 +341,11 @@ sub processHTTP {
 		# store the cauth code in the request object (headers are handy!)
 		$request->push_header("X-Slim-CSRF",$csrfAuth);
 	}
+	
+	# Read cookie(s)
+	if ( my $cookie = $request->header('Cookie') ) {
+		$params->{'cookies'} = { CGI::Cookie->parse($cookie) };
+	}
 
 	# this bundles up all our response headers and content
 	my $response = HTTP::Response->new();
@@ -404,7 +415,8 @@ sub processHTTP {
 					# string with the appropriate magic set.
 					if ($value ne '*' && $value ne '') {
 
-						$value = Slim::Utils::Unicode::utf8decode($value);
+						$value = Slim::Utils::Unicode::utf8on($value);
+						$value = Slim::Utils::Unicode::utf8encode_locale($value);
 					}
 
 					$params->{$name} = $value;
@@ -641,6 +653,13 @@ sub processURL {
 			$client->prefSet('transcodeBitrate',undef);
 		}
 	}
+	
+	# player specified from cookie
+	if ( !defined $client && $params->{'cookies'} ) {
+		if ( my $player = $params->{'cookies'}->{'SlimServer-player'} ) {
+			$client = Slim::Player::Client::getClient( $player->value );
+		}
+	}
 
 	# if we don't have a player specified, just pick one if there is one...
 	if (!defined($client) && Slim::Player::Client::clientCount() > 0) {
@@ -739,6 +758,14 @@ sub generateHTTPResponse {
 	if (defined($client)) {
 		$params->{'player'} = $client->id();
 		$params->{'myClientState'} = $client;
+		
+		# save the player id in a cookie
+		my $cookie = CGI::Cookie->new(
+			-name    => 'SlimServer-player',
+			-value   => $params->{'player'},
+			-expires => '+1y',
+		);
+		$response->headers->push_header( 'Set-Cookie' => $cookie );
 	}
 
 	# this might do well to break up into methods
@@ -1337,7 +1364,7 @@ sub addStreamingResponse {
 #   May want to enable this later, if we find that that it has any effect on some platforms...
 #	setsockopt $httpClient, SOL_SOCKET, SO_SNDBUF, MAXCHUNKSIZE;
 	
-	Slim::Networking::Select::addWrite($httpClient, \&sendStreamingResponse);
+	Slim::Networking::Select::addWrite($httpClient, \&sendStreamingResponse, 1);
 
 	# we aren't going to read from this socket anymore so don't select on it...
 	Slim::Networking::Select::addRead($httpClient, undef);
@@ -1590,7 +1617,7 @@ sub sendStreamingResponse {
 sub tryStreamingLater {
 	my $client = shift;
 	my $httpClient = shift;
-	Slim::Networking::Select::addWrite($httpClient, \&sendStreamingResponse);
+	Slim::Networking::Select::addWrite($httpClient, \&sendStreamingResponse, 1);
 }
 
 sub nonBreaking {
@@ -1897,9 +1924,9 @@ sub closeHTTPSocket {
 	my $httpClient = shift;
 	my $streaming = shift;
 	
-	Slim::Networking::Select::addRead($httpClient, undef);
-	Slim::Networking::Select::addWrite($httpClient, undef);
-	Slim::Networking::Select::addError($httpClient, undef);
+	Slim::Networking::Select::removeRead($httpClient);
+	Slim::Networking::Select::removeWrite($httpClient);
+	Slim::Networking::Select::removeError($httpClient);
 
 	# clean up the various caches
 	delete($outbuf{$httpClient});
@@ -2092,7 +2119,7 @@ sub isRequestCSRFSafe {
 		if ( ! isCsrfAuthCodeValid($request) ) {
 
 			$params->{'suggestion'} = "Invalid referrer and no valid cauth code.";
-			$::d_http && msg("No valid CSRF auth code: [" . join(' ', ($request->method(), $request->uri(), $request-header('X-Slim-CSRF'))) . "]\n");
+			$::d_http && msg("No valid CSRF auth code: [" . join(' ', ($request->method(), $request->uri(), $request->header('X-Slim-CSRF'))) . "]\n");
 
 		} else {
 
