@@ -1,6 +1,6 @@
 package Slim::Web::Pages;
 
-# $Id: Pages.pm 5002 2005-11-02 20:13:03Z dsully $
+# $Id: Pages.pm 8068 2006-06-20 17:41:46Z andy $
 
 # SlimServer Copyright (c) 2001-2004 Sean Adams, Slim Devices Inc.
 # This program is free software; you can redistribute it and/or
@@ -40,13 +40,14 @@ sub init {
 	addLinks("browse",{'BROWSE_NEW_MUSIC' => "browsedb.html?hierarchy=age,track&level=0"});
 	addLinks("search", {'SEARCH' => "search.html?liveSearch=1"});
 	addLinks("search", {'ADVANCEDSEARCH' => "advanced_search.html"});
+	addLinks("plugins",{'SQUEEZENETWORK_SWITCH' => "squeezenetwork.html"});
 	addLinks("help",{'GETTING_STARTED' => "html/docs/quickstart.html"});
 	addLinks("help",{'PLAYER_SETUP' => "html/docs/ipconfig.html"});
 	addLinks("help",{'USING_REMOTE' => "html/docs/interface.html"});
 	addLinks("help",{'HELP_REMOTE' => "html/help_remote.html"});
 	addLinks("help",{'HELP_RADIO' => "html/docs/radio.html"});
 	addLinks("help",{'REMOTE_STREAMING' => "html/docs/remotestreaming.html"});
-	addLinks("help",{'FAQ' => "html/docs/faq.html"});
+	addLinks("help",{'FAQ' => "http://faq.slimdevices.com/\" target=\"faq"});
 	addLinks("help",{'SOFTSQUEEZE' => "html/softsqueeze/index.html"});
 	addLinks("help",{'TECHNICAL_INFORMATION' => "html/docs/index.html"});
 }
@@ -169,30 +170,36 @@ sub addLibraryStats {
 	my $ds   = Slim::Music::Info::getCurrentDataStore();
 	my $find = {};
 
-	$find->{'genre'}       = $genre  if $genre;
+	$find->{'genre'}       = $genre  if $genre  && !$album;
 	$find->{'contributor'} = $artist if $artist && !$album;
 	$find->{'album'}       = $album  if $album;
 
-	$params->{'song_count'}   = _lcPlural($ds->count('track', $find), 'SONG', 'SONGS');
-	$params->{'album_count'}  = _lcPlural($ds->count('album', $find), 'ALBUM', 'ALBUMS');
+	if (Slim::Utils::Prefs::get('disableStatistics')) {
+		$params->{'song_count'}   = 0;
+		$params->{'album_count'}  = 0;
+		$params->{'artist_count'} = 0;
+	} else {
+	
+		# Right now hitlist.html is the only page that uses genre_count -
+		# which can be expensive. Only generate it if we need to.
+		if ($params->{'path'} =~ /hitlist/) {
+	
+			$params->{'genre_count'} = _lcPlural($ds->count('genre', $find), 'GENRE', 'GENRES');
+		}
+	
+		# Bug 1913 - don't put counts for contributor & tracks when an artist
+		# is a composer on a different artist's tracks.
+		if ($artist && $artist eq $ds->variousArtistsObject->id) {
+	
+			delete $find->{'contributor'};
+	
+			$find->{'album.compilation'} = 1;
+		}
 
-	# Right now hitlist.html is the only page that uses genre_count -
-	# which can be expensive. Only generate it if we need to.
-	if ($params->{'path'} =~ /hitlist/) {
-
-		$params->{'genre_count'} = _lcPlural($ds->count('genre', $find), 'GENRE', 'GENRES');
+		$params->{'song_count'}   = _lcPlural($ds->count('track', $find), 'SONG', 'SONGS');
+		$params->{'album_count'}  = _lcPlural($ds->count('album', $find), 'ALBUM', 'ALBUMS');
+		$params->{'artist_count'} = _lcPlural($ds->count('contributor', $find), 'ARTIST', 'ARTISTS');
 	}
-
-	# Bug 1913 - don't put counts for contributor & tracks when an artist
-	# is a composer on a different artist's tracks.
-	if ($artist && $artist eq $ds->variousArtistsObject->id) {
-
-		delete $find->{'contributor'};
-
-		$find->{'album.compilation'} = 1;
-	}
-
-	$params->{'artist_count'} = _lcPlural($ds->count('contributor', $find), 'ARTIST', 'ARTISTS');
 }
 
 # Send the status page (what we're currently playing, contents of the playlist)
@@ -394,6 +401,9 @@ sub playlist {
 		$params->{'player_needs_upgrade'} = '1';
 		return Slim::Web::HTTP::filltemplatefile("playlist_needs_upgrade.html", $params);
 	}
+	
+	# If synced, use the master's playlist
+	$client = Slim::Player::Sync::masterOrSelf($client);
 
 	$params->{'playercount'} = Slim::Player::Client::clientCount();
 	
@@ -746,7 +756,7 @@ sub advancedSearch {
 
 			# Bitrate needs to changed a bit
 			if ($key =~ /bitrate$/) {
-				$params->{$key} *= 100;
+				$params->{$key} *= 1000;
 			}
 
 			# Duration is also special
@@ -1047,7 +1057,6 @@ sub browsedb {
 
 	my $itemnumber = 0;
 	my $lastAnchor = '';
-	my $descend;
 	my %names = ();
 	my @attrs = ();
 	my %findCriteria = ();	
@@ -1163,6 +1172,13 @@ sub browsedb {
 			$params->{$attr} = $ds->objectForId($attr, $params->{$attr});
 		}
 	}
+	
+	# Bug 3311, disable editing for iTunes, MoodLogic, and MusicMagic playlists
+	if ( ref $params->{'playlist'} ) {
+		if ( $params->{'playlist'}->content_type =~ /(?:itu|mlp|mmp)/ ) {
+			$params->{'noEdit'} = 1;
+		}
+	}
 
 	my $otherparams = join('&',
 		'player=' . Slim::Web::HTTP::escape($player || ''),
@@ -1174,11 +1190,16 @@ sub browsedb {
 	my $levelInfo = $fieldInfo->{$levels[$level]} || $fieldInfo->{'default'};
 	my $items     = &{$levelInfo->{'find'}}($ds, $levels[$level], \%findCriteria);
 
+	my $descend   = ($level >= $maxLevel) ? undef : 'true';
+
+	my ($start, $end);
+
+	# Don't bother with idle streams if we only have SB2 clients
+	my $needIdleStreams = Slim::Player::Client::needIdleStreams();
+
 	if ($items && scalar(@$items)) {
 
-		my ($start, $end);
-
-		my $ignoreArticles = $levelInfo->{'ignoreArticles'};
+		main::idleStreams() if $needIdleStreams;
 
 		if (defined $params->{'nopagebar'}) {
 
@@ -1188,7 +1209,7 @@ sub browsedb {
 				\$params->{'browselist_header'},
 				$params->{'skinOverride'},
 				$params->{'itemsPerPage'},
-				$ignoreArticles ? (scalar(@$items) > 1) : 0,
+				$levelInfo->{'ignoreArticles'} ? (scalar(@$items) > 1) : 0,
 			);
 
 		} elsif (&{$levelInfo->{'alphaPageBar'}}(\%findCriteria)) {
@@ -1219,104 +1240,118 @@ sub browsedb {
 				$params->{'itemsPerPage'},
 			);
 		}
+	}
 
-		#$params->{'browse_list'} .= ${Slim::Web::HTTP::filltemplatefile("browsedb_list.html", \%list_form)};
+	if (scalar(@$items) > 1 && !$levelInfo->{'suppressAll'}) {
 
-		$descend = ($level >= $maxLevel) ? undef : 'true';
+		main::idleStreams() if $needIdleStreams;
 
-		if (scalar(@$items) > 1 && !$levelInfo->{'suppressAll'}) {
+		if ($params->{'includeItemStats'} && !Slim::Utils::Misc::stillScanning()) {
+			# XXX include statistics
+		}
 
-			if ($params->{'includeItemStats'} && !Slim::Utils::Misc::stillScanning()) {
-				# XXX include statistics
-			}
+		my $nextLevelInfo;
 
-			my $nextLevelInfo;
+		if ($descend) {
+			$nextLevelInfo = $fieldInfo->{ $levels[$level+1] } || $fieldInfo->{'default'};
+		} else {
+			$nextLevelInfo = $fieldInfo->{'track'};
+		}
 
-			if ($descend) {
-				$nextLevelInfo = $fieldInfo->{ $levels[$level+1] } || $fieldInfo->{'default'};
-			} else {
-				$nextLevelInfo = $fieldInfo->{'track'};
-			}
+		if ($level == 0) {
 
-			if ($level == 0) {
+			# Sometimes we want a special transform for
+			# the 'All' case - such as New Music.
+			#
+			# Otherwise we might have a regular descend
+			# transform, such as the artwork case.
+			if ($levelInfo->{'allTransform'}) {
 
-				# Sometimes we want a special transform for
-				# the 'All' case - such as New Music.
-				#
-				# Otherwise we might have a regular descend
-				# transform, such as the artwork case.
-				if ($levelInfo->{'allTransform'}) {
+				 $list_form{'hierarchy'} = $levelInfo->{'allTransform'};
 
-					 $list_form{'hierarchy'} = $levelInfo->{'allTransform'};
+			} elsif ($levelInfo->{'descendTransform'}) {
 
-				} elsif ($levelInfo->{'descendTransform'}) {
-
-					 $list_form{'hierarchy'} = $levelInfo->{'descendTransform'};
-
-				} else {
-
-					 $list_form{'hierarchy'} = join(',', @levels[1..$#levels]);
-				}
-
-				$list_form{'level'} = 0;
+				 $list_form{'hierarchy'} = $levelInfo->{'descendTransform'};
 
 			} else {
 
-				$list_form{'hierarchy'}	= $hierarchy;
-				$list_form{'level'}	= $descend ? $level+1 : $level;
+				 $list_form{'hierarchy'} = join(',', @levels[1..$#levels]);
 			}
 
-			if ($nextLevelInfo->{'allTitle'}) {
-				$list_form{'text'} = string($nextLevelInfo->{'allTitle'});
-			}
+			$list_form{'level'} = 0;
 
-			$list_form{'descend'}      = 1;
-			$list_form{'player'}       = $player;
-			$list_form{'odd'}	   = ($itemnumber + 1) % 2;
-			$list_form{'skinOverride'} = $params->{'skinOverride'};
-			$list_form{'attributes'}   = (scalar(@attrs) ? ('&' . join("&", @attrs)) : '');
+		} else {
 
-			# For some queries - such as New Music - we want to
-			# get the list of tracks to play from the fieldInfo
-			if ($levels[$level] eq 'age' && $levelInfo->{'allTransform'}) {
+			$list_form{'hierarchy'}	= $hierarchy;
+			$list_form{'level'}	= $descend ? $level+1 : $level;
+		}
 
-				$list_form{'attributes'} .= sprintf('&fieldInfo=%s', $levelInfo->{'allTransform'});
-			}
+		if ($nextLevelInfo->{'allTitle'}) {
+			$list_form{'text'} = string($nextLevelInfo->{'allTitle'});
+		}
+
+		$list_form{'descend'}      = 1;
+		$list_form{'player'}       = $player;
+		$list_form{'odd'}	   = ($itemnumber + 1) % 2;
+		$list_form{'skinOverride'} = $params->{'skinOverride'};
+		$list_form{'attributes'}   = (scalar(@attrs) ? ('&' . join("&", @attrs)) : '');
+
+		# For some queries - such as New Music - we want to
+		# get the list of tracks to play from the fieldInfo
+		if ($levels[$level] eq 'age' && $levelInfo->{'allTransform'}) {
+
+			$list_form{'attributes'} .= sprintf('&fieldInfo=%s', $levelInfo->{'allTransform'});
+		}
+
+		$itemnumber++;
+
+		$params->{'browse_list'} .= ${Slim::Web::HTTP::filltemplatefile("browsedb_list.html", \%list_form)};
+	}
+
+	# Dynamic VA/Compilation listing
+	# This needs to be done outside the if ($items) loop - as there are
+	# cases when VA needs to be populated, but there are no regular
+	# artists for a genre. See bugs: 2662 & 3046
+	if ($levels[$level] eq 'artist' && Slim::Utils::Prefs::get('variousArtistAutoIdentification')) {
+
+		my %list_form  = %$params;
+		my $vaObj      = $ds->variousArtistsObject;
+		my @attributes = (@attrs, 'album.compilation=1', sprintf('artist=%d', $vaObj->id));
+
+		# Only show VA item if there's valid listings below
+		# the current level.
+		my %find = map { split /=/ } @attrs, 'album.compilation=1';
+
+		main::idleStreams() if $needIdleStreams;
+
+		if ($ds->count('album', \%find)) {
+
+			$list_form{'text'}        = $vaObj->name;
+			$list_form{'descend'}     = ($level >= $maxLevel) ? undef : 'true';
+			$list_form{'hierarchy'}   = $hierarchy;
+			$list_form{'level'}	  = $level + 1;
+			$list_form{'odd'}	  = ($itemnumber + 1) % 2;
+			$list_form{'attributes'}  = (scalar(@attributes) ? ('&' . join("&", @attributes, )) : '');
+
+			$params->{'browse_list'} .= ${Slim::Web::HTTP::filltemplatefile("browsedb_list.html", \%list_form)};
 
 			$itemnumber++;
 
-			$params->{'browse_list'} .= ${Slim::Web::HTTP::filltemplatefile("browsedb_list.html", \%list_form)};
+			main::idleStreams() if $needIdleStreams;
+		}
+	}
+
+	if ($items && scalar(@$items)) {
+
+		# Sanity check - Bug: 3053
+		if ($start > scalar @$items || $start < 0) {
+			$start = 0;
 		}
 
-		# Dynamic VA/Compilation listing
-		if ($levels[$level] eq 'artist' && Slim::Utils::Prefs::get('variousArtistAutoIdentification')) {
-
-			my %list_form  = %$params;
-			my $vaObj      = $ds->variousArtistsObject;
-			my @attributes = (@attrs, 'album.compilation=1', sprintf('artist=%d', $vaObj->id));
-
-			# Only show VA item if there's valid listings below
-			# the current level.
-			my %find = map { split /=/ } @attrs, 'album.compilation=1';
-
-			if ($ds->count('album', \%find)) {
-
-				$list_form{'text'}        = $vaObj->name;
-				$list_form{'descend'}     = $descend;
-				$list_form{'hiearchy'}    = $hierarchy;
-				$list_form{'level'}	  = $level + 1;
-				$list_form{'odd'}	  = ($itemnumber + 1) % 2;
-				$list_form{'attributes'}  = (scalar(@attributes) ? ('&' . join("&", @attributes, )) : '');
-
-				$params->{'browse_list'} .= ${Slim::Web::HTTP::filltemplatefile("browsedb_list.html", \%list_form)};
-
-				$itemnumber++;
-			}
+		if ($end > scalar @$items || $end < 0) {
+			$end = scalar(@$items) - 1;
 		}
 
-		# Don't bother with idle streams if we only have SB2 clients
-		my $needIdleStreams = Slim::Player::Client::needIdleStreams();
-
 		for my $item ( @{$items}[$start..$end] ) {
 
 			my %list_form = %$params;
@@ -1374,9 +1409,7 @@ sub browsedb {
 				$params->{'browse_list'} .= ${Slim::Web::HTTP::filltemplatefile("browsedb_list.html", \%list_form)};
 			}
 
-			if ($needIdleStreams) {
-				main::idleStreams();
-			}
+			main::idleStreams() if $needIdleStreams;
 		}
 
 		if ($level == $maxLevel && $levels[$level] eq 'track') {
@@ -1391,7 +1424,7 @@ sub browsedb {
 	}
 
 	# Give players a bit of time.
-	main::idleStreams();
+	main::idleStreams() if $needIdleStreams;
 
 	$params->{'descend'} = $descend;
 
@@ -1495,15 +1528,15 @@ sub browsetree {
 
 		# Turn the utf8 flag on for proper display - since this is
 		# coming directly from the filesystem.
-		$list_form{'text'}	    = Slim::Utils::Unicode::utf8decode_locale($relPath);
+		$list_form{'text'}	    = Slim::Utils::Unicode::utf8on( Slim::Music::Info::fileName($item->url) ),
 
 		$list_form{'hierarchy'}	    = join('/', @levels, $item->id);
 		$list_form{'descend'}	    = Slim::Music::Info::isList($item) ? 1 : 0;
 		$list_form{'odd'}	    = ($itemnumber + 1) % 2;
 		$list_form{'itemobj'}	    = $item;
 
-		# Don't display the edit dialog for cue sheets.
-		if ($item->isCUE) {
+		# Don't display the edit dialog for playlists (includes CUE sheets).
+		if ($item->isPlaylist) {
 			$list_form{'noEdit'} = '&noEdit=1';
 		}
 
@@ -1876,6 +1909,25 @@ sub tuneIn {
 	return Slim::Web::HTTP::filltemplatefile('tunein.html', $params);
 }
 
+sub squeezeNetwork {
+	my ($client, $params) = @_;
+
+	if ($client) {
+		$params->{'playername'} = $client->name;
+	
+		Slim::Utils::Timers::setTimer(
+			$client,
+			time + 2,
+			sub {
+				my $client = shift;
+				$client->execute([ 'squeezenetwork', 'connect' ]);
+			},
+		);
+	}
+	
+	return Slim::Web::HTTP::filltemplatefile('squeezenetwork.html', $params);
+}
+
 1;
 
 __END__
