@@ -113,24 +113,58 @@ BEGIN {
 
 	# Given a list of modules, attempt to load them, otherwise pass back
 	# the failed list to the caller.
+
+	# Allow command line debugging.
+	my $d_startup = (grep { /d_startup/ } @ARGV) ? 1 : 0;
+
 	sub tryModuleLoad {
 		my @modules = @_;
 
 		my @failed  = ();
 
+		my (%oldINC, @newModules);
+
 		for my $module (@modules) {
 
+			%oldINC = %INC;
+
+			# Don't spit out any redefined warnings
+			local $^W = 0;
+
 			eval "use $module";
 
 			if ($@) {
-				Symbol::delete_package($module);
-
 				push @failed, $module;
 
-				$module =~ s|::|/|g;
-				$module .= '.pm';
+				@newModules = grep { !$oldINC{$_} } keys %INC;
 
-				delete $INC{$module};
+				for my $newModule (@newModules) {
+
+					# Don't bother removing/reloading
+					# these, as they're part of core Perl.
+					if ($newModule =~ /^(?:DynaLoader|Carp|overload)/) {
+						next;
+					}
+
+					my $newModuleSymbol = $newModule;
+
+					$newModuleSymbol =~ s|/|::|g;
+					$newModuleSymbol =~ s|\.pm$||;
+
+					# This relies on delete_package returning a true value if it succeeded.
+					my $removed = eval {
+						Symbol::delete_package($newModuleSymbol);
+					};
+
+					if ($removed) {
+						$d_startup && print "Removing [$newModuleSymbol] from the symbol table - load failed.\n";
+						delete $INC{$newModule};
+					}
+				}
+
+			} else {
+
+				$d_startup && print "Loaded module: [$module] ok!\n";
 			}
 		}
 
@@ -139,47 +173,77 @@ BEGIN {
 
 	# Here's what we want to try and load. This will need to be updated
 	# when a new XS based module is added to our CPAN tree.
-	my @modules = qw(Time::HiRes DBD::SQLite DBI XML::Parser HTML::Parser Compress::Zlib);
+	my @required_modules = qw(Time::HiRes DBD::SQLite DBI XML::Parser::Expat HTML::Parser Compress::Zlib Digest::SHA1);
+
+	# Bug 3324
+	# In some cases, GD wasn't loading even though there was a working
+	# installation in the user's CPAN path. This is an attempt to use the
+	# use the system described above to load GD if possible.
+	my @optional_modules = qw(GD);
 
 	if ($] <= 5.007) {
-		push @modules, qw(Storable Digest::MD5);
+		push @required_modules, qw(Storable Digest::MD5);
 	}
 
 	my @SlimINC = (
-		$Bin, 
+		catdir($Bin,'CPAN','arch',(join ".", map {ord} split //, $^V), $Config::Config{'archname'}),
+		catdir($Bin,'CPAN','arch',(join ".", map {ord} split //, $^V), $Config::Config{'archname'}, 'auto'),
+		catdir($Bin,'CPAN','arch',(join ".", map {ord} (split //, $^V)[0,1]), $Config::Config{'archname'}),
+		catdir($Bin,'CPAN','arch',(join ".", map {ord} (split //, $^V)[0,1]), $Config::Config{'archname'}, 'auto'),
+		catdir($Bin,'CPAN','arch',$Config::Config{'archname'}),
+		catdir($Bin,'lib'), 
 		catdir($Bin,'CPAN'), 
-		catdir($Bin,'CPAN','arch',(join ".", map {ord} split //, $^V), $Config::Config{'archname'}), 
-		catdir($Bin,'CPAN','arch',(join ".", map {ord} split //, $^V), $Config::Config{'archname'}, 'auto'), 
-		catdir($Bin,'CPAN','arch',(join ".", map {ord} (split //, $^V)[0,1]), $Config::Config{'archname'}), 
-		catdir($Bin,'CPAN','arch',(join ".", map {ord} (split //, $^V)[0,1]), $Config::Config{'archname'}, 'auto'), 
-		catdir($Bin,'CPAN','arch',$Config::Config{archname})
 	);
 
+	$d_startup && printf("Got \@INC containing:\n%s\n\n", join("\n", @INC));
+
 	# This works like 'use lib'
 	# prepend our directories to @INC so we look there first.
 	unshift @INC, @SlimINC;
 
+	$d_startup && printf("Extended \@INC to contain:\n%s\n\n", join("\n", @INC));
+
 	# Try and load the modules - some will fail if we don't include the
 	# binaries for that version/architecture combo
-	my @failed = tryModuleLoad(@modules);
+	my @required_failed = tryModuleLoad(@required_modules);
+	my @optional_failed = tryModuleLoad(@optional_modules);
 
-	# Remove our CPAN path so we can try loading the failed modules from
-	# the default system @INC
+	if (scalar @required_failed && $d_startup) {
+		printf("The following modules failed to load on the first attempt: [%s] - will try again.\n\n", join(', ', @required_failed));
+	}
+
+	if (scalar @optional_failed && $d_startup) {
+		printf("The following optional modules failed to load on the first attempt: [%s] - will try again\n\n", join(', ', @optional_failed));
+	}
+
+	# Remove our paths so we can try loading the failed modules from the default system @INC
 	splice(@INC, 0, scalar @SlimINC);
 
-	my @reallyFailed = tryModuleLoad(@failed);
+	my @required_really_failed = tryModuleLoad(@required_failed);
+	my @optional_really_failed = tryModuleLoad(@optional_failed);
 
-	if (scalar @reallyFailed) {
+	if (scalar @optional_really_failed && $d_startup) {
+		printf("The following optional modules failed to load: [%s] after their second try.\n\n", join(', ', @optional_really_failed));
+	}
 
-		printf("The following modules failed to load: %s\n\n", join(' ', @reallyFailed));
+	if (scalar @required_really_failed) {
 
-		#print "Compress::Zlib and XML::Parser are optional modules and are not required to run SlimServer.\n\n";
+		printf("The following modules failed to load: %s\n\n", join(' ', @required_really_failed));
 
 		print "To download and compile them, please run: $Bin/Bin/build-perl-modules.pl\n\n";
+		print "Exiting..\n";
+
+		exit;
 	}
 
 	# And we're done with the trying - put our CPAN path back on @INC.
 	unshift @INC, @SlimINC;
+
+	if ($d_startup) {
+		print "The following modules are loaded:\n";
+		print map { "\t$_ => $INC{$_}\n" } keys %INC;
+		print "\n";
+	}
 };
 
 use Time::HiRes;
@@ -241,6 +305,7 @@ use Slim::Utils::Strings qw(string);
 use Slim::Utils::Timers;
 use Slim::Networking::Slimproto;
 use Slim::Networking::SimpleAsyncHTTP;
+use Slim::Utils::UPnPMediaServer;
 
 use vars qw($VERSION $REVISION @AUTHORS);
 
@@ -261,7 +326,7 @@ use vars qw($VERSION $REVISION @AUTHORS);
 	'Dan Sully',
 );
 
-$VERSION  = '6.2.1';
+$VERSION  = '6.3.0';
 
 # old preferences settings, only used by the .slim.conf configuration.
 # real settings are stored in the new preferences file:  .slim.pref
@@ -314,6 +379,7 @@ use vars qw(
 	$d_source_v
 	$d_sql
 	$d_stdio
+	$d_startup
 	$d_stream
 	$d_stream_v
 	$d_sync
@@ -437,13 +503,9 @@ sub init {
 	$::d_server && msg("SlimServer settings effective user and group if requested...\n");
 	changeEffectiveUserAndGroup();
 
-	if ($priority) {
-		$::d_server && msg("SlimServer - changing process priority to $priority\n");
-		eval { setpriority (0, 0, $priority); };
-		msg("setpriority failed error: $@\n") if $@;
-	}
+	Slim::Utils::Misc::setPriority($priority);
 
-# do platform specific environment setup
+	# do platform specific environment setup
 	# we have some special directories under OSX.
 	if (Slim::Utils::OSDetect::OS() eq 'mac') {
 		mkdir $ENV{'HOME'} . "/Library/SlimDevices";
@@ -457,7 +519,7 @@ sub init {
 		unshift @INC, "/Library/SlimDevices/";
 	}
 
-# initialize slimserver subsystems
+	# initialize slimserver subsystems
 	$::d_server && msg("SlimServer settings init...\n");
 	initSettings();
 
@@ -467,7 +529,7 @@ sub init {
 	$::d_server && msg("SlimServer Setup init...\n");
 	Slim::Web::Setup::initSetup();
 
-# initialize all player UI subsystems (if it's not just a scan process)
+	# initialize all player UI subsystems (if it's not just a scan process)
 	if (!$scanOnly) {
 		$::d_server && msg("SlimServer setting language...\n");
 		Slim::Utils::Strings::setLanguage(Slim::Utils::Prefs::get("language"));
@@ -494,12 +556,18 @@ sub init {
 	
 		$::d_server && msg("mDNS init...\n");
 		Slim::Networking::mDNS->init;
+		
+		$::d_server && msg("AsyncHTTP init...\n");
+		Slim::Networking::AsyncHTTP->init;
 	
 		$::d_server && msg("SlimServer HTTP init...\n");
 		Slim::Web::HTTP::init();
 	
 		$::d_server && msg("SlimServer CLI init...\n");
 		Slim::Control::CLI::init();
+
+		$::d_server && msg("SlimServer UPnP init...\n");
+		Slim::Utils::UPnPMediaServer::init();
 	
 		if (Slim::Utils::Prefs::get('xplsupport')) {
 			$::d_server && msg("SlimServer xPL init...\n");
@@ -536,7 +604,7 @@ sub init {
 	$::d_server && msg("SlimServer checkDataSource...\n");
 	checkDataSource();
 
-# regular server has a couple more initial operations.
+	# regular server has a couple more initial operations.
 	if (!$scanOnly) {
 		$::d_server && msg("SlimServer persist playlists...\n");
 		if (Slim::Utils::Prefs::get('persistPlaylists')) {
@@ -544,10 +612,9 @@ sub init {
 		}
 
 		checkVersion();
-		keepSlimServerInMemory();
 	}
 	
-# otherwise, get ready to loop
+	# otherwise, get ready to loop
 	$lastlooptime = Time::HiRes::time();
 	$loopcount = 0;
 	$loopsecond = int($lastlooptime);
@@ -653,22 +720,27 @@ sub idle {
 
 sub idleStreams {
 	my $timeout = shift || 0;
+	my $streamOnly = shift; # set for select to only check streaming sockets
+
 	my $select_time = 0;
+	my $check_timers = 1;
 	my $to;
 
 	if ($timeout) {
-		my $select_time = Slim::Utils::Timers::nextTimer();
-		if (!defined($select_time) || $select_time > $timeout) { $select_time = $timeout };	    
+		$select_time = Slim::Utils::Timers::nextTimer();
+		if ( !defined($select_time) || $select_time > $timeout ) {
+			$check_timers = 0;
+			$select_time = $timeout;
+		}
 	}
 
-	$::d_time && msg("select_time - idleStreams: $select_time\n");
+	$::d_time && msg("select_time - idleStreams: $select_time, checkTimers: $check_timers\n");
 
-	Slim::Networking::Select::select($select_time);
-
-	if ($::d_perf) { $to = watchDog(); }
-
-	Slim::Utils::Timers::checkTimers();		    
-	if ($::d_perf) { $to = watchDog($to, "checkTimers - idleStreams"); }
+	Slim::Networking::Select::select($select_time, $streamOnly);
+	
+	if ( $check_timers ) {
+		Slim::Utils::Timers::checkTimers();
+	}
 }
 
 sub showUsage {
@@ -772,6 +844,7 @@ to the console via stderr:
     --d_source_v     => Verbose information about source audio files
     --d_sql          => Verbose SQL debugging
     --d_stdio        => Standard I/O command debugging
+    --d_startup      => Startup/Bootstrap debugging for @INC
     --d_stream       => Information about player streaming protocol 
     --d_stream_v     => Verbose information about player streaming protocol 
     --d_sync         => Information about multi player synchronization
@@ -856,6 +929,7 @@ sub initOptions {
 		'd_source_v'		=> \$d_source_v,
 		'd_sql'				=> \$d_sql,
 		'd_stdio'			=> \$d_stdio,
+		'd_startup'			=> \$d_startup,
 		'd_stream'			=> \$d_stream,
 		'd_stream_v'		=> \$d_stream_v,
 		'd_sync'			=> \$d_sync,
@@ -920,23 +994,35 @@ sub initSettings {
 	# and that they do not end in / or \
 
 	if (defined(Slim::Utils::Prefs::get("playlistdir")) && Slim::Utils::Prefs::get("playlistdir") ne '') {
+
 		$playlistdir = Slim::Utils::Prefs::get("playlistdir");
 		$playlistdir =~ s|[/\\]$||;
-		$playlistdir = Slim::Utils::Misc::fixPathCase($playlistdir);
+
+		$playlistdir = Slim::Utils::Misc::fixPath($playlistdir);
+		$playlistdir = Slim::Utils::Misc::pathFromFileURL($playlistdir);
+
 		Slim::Utils::Prefs::set("playlistdir",$playlistdir);
 	}
 
 	if (defined(Slim::Utils::Prefs::get("audiodir")) && Slim::Utils::Prefs::get("audiodir") ne '') {
+
 		$audiodir = Slim::Utils::Prefs::get("audiodir");
 		$audiodir =~ s|[/\\]$||;
-		$audiodir = Slim::Utils::Misc::fixPathCase($audiodir);
+
+		$audiodir = Slim::Utils::Misc::fixPath($audiodir);
+		$audiodir = Slim::Utils::Misc::pathFromFileURL($audiodir);
+
 		Slim::Utils::Prefs::set("audiodir",$audiodir);
 	}
 	
 	if (defined(Slim::Utils::Prefs::get("cachedir")) && Slim::Utils::Prefs::get("cachedir") ne '') {
+
 		$cachedir = Slim::Utils::Prefs::get("cachedir");
 		$cachedir =~ s|[/\\]$||;
-		$cachedir = Slim::Utils::Misc::fixPathCase($cachedir);
+
+		$cachedir = Slim::Utils::Misc::fixPath($cachedir);
+		$cachedir = Slim::Utils::Misc::pathFromFileURL($cachedir);
+
 		Slim::Utils::Prefs::set("cachedir",$cachedir);
 	}
 
@@ -972,8 +1058,10 @@ sub daemonize {
 	# Update plugin / Mac::Applescript::Glue , and setting the process
 	# name after we fork. So don't do it on Mac. The System Preferences
 	# start/stop still works.
-
-	if (Slim::Utils::OSDetect::OS() ne 'mac') {
+	# 
+	# Also don't do it for *nix - as startup scripts have problems (FC5)
+	# when our process name gets changed.
+	if (Slim::Utils::OSDetect::OS() eq 'win') {
 		$0 = "slimserver";
 	}
 
@@ -1110,45 +1198,6 @@ sub checkVersionError {
 	msg(Slim::Utils::Strings::string('CHECKVERSION_ERROR') . "\n" . $http->error . "\n");
 }
 
-# See Bug 1934 - Some VM's (*cough*Windows*cough*) don't deal well with
-# swapping. So try and keep our memory image in RAM and not swap out.
-sub keepSlimServerInMemory {
-
-	my $interval = Slim::Utils::Prefs::get('keepUnswappedInterval');
-	$interval = 30 if (!(defined($interval)));
-
-	$::d_server && msg("Requesting web page to keep SlimServer unswapped, re-request interval is $interval minutes.\n");
-
-	# Disabled if interval set to less than zero seconds
-	if ($interval <= 0)
-	{
-	    $::d_server && msg("Interval is less than or equal to zero, Keeping SlimServer unswapped is disabled.\n");
-	    return;
-	}
-
-	my $url  = '';
-	my $port = Slim::Utils::Prefs::get('httpport');
-
-	if (Slim::Utils::Prefs::get('authorize')) {
-
-		$url = sprintf("http://%s:%s\@localhost:$port/browsedb.html?hierarchy=age,track&level=0",
-			Slim::Utils::Prefs::get('username'), Slim::Utils::Prefs::get('password')
-		);
-
-	} else {
-
-		$url = "http://localhost:$port/browsedb.html?hierarchy=age,track&level=0";
-	}
-
-	my $http = Slim::Networking::SimpleAsyncHTTP->new(sub {}, sub {});
-
-	$http->get($url);
-
-	# Interval is configured in minutes...
-	$interval = $interval * 60;
-	Slim::Utils::Timers::setTimer(0, Time::HiRes::time() + $interval, \&keepSlimServerInMemory);
-}
-
 #------------------------------------------
 #
 # Clean up resources and exit.
